package geogebra.common.kernel.barycentric;

import geogebra.common.kernel.Construction;
import geogebra.common.kernel.Kernel;
import geogebra.common.kernel.algos.AlgoElement;
import geogebra.common.kernel.algos.Algos;
import geogebra.common.kernel.arithmetic.Equation;
import geogebra.common.kernel.arithmetic.NumberValue;
import geogebra.common.kernel.arithmetic.ValidExpression;
import geogebra.common.kernel.commands.AlgebraProcessor;
import geogebra.common.kernel.geos.GeoElement;
import geogebra.common.kernel.geos.GeoPoint;
import geogebra.common.kernel.implicit.GeoImplicitPoly;
import geogebra.common.kernel.parser.ParseException;
import geogebra.common.kernel.parser.Parser;
import geogebra.common.main.MyError;


/**
 * @author Darko Drakulic
 * @version 23-10-2011
 * 
 *  This class makes a curve in barycentric coordinates 
 *          
 */

public class AlgoCubic extends AlgoElement {

	
	private GeoPoint A, B, C; // input
	private NumberValue n;	// number of curve
	private GeoImplicitPoly poly; // output
	/**
	 * Creates new triangle cubic algo
	 * @param cons construction
	 * @param label label
	 * @param A first point
	 * @param B second point
	 * @param C third point
	 * @param e index in CTC
	 */
	public AlgoCubic(Construction cons, String label, GeoPoint A, GeoPoint B,
			GeoPoint C, NumberValue e) {
		super(cons);
		this.A = A;
		this.B = B;
		this.C = C;
		this.n = e;
		poly = new GeoImplicitPoly(cons);
		setInputOutput();
		compute();		
		poly.setLabel(label);
	}

	@Override
	public Algos getClassName() {
		return Algos.AlgoCubic;
	}

	// for AlgoElement
	@Override
	protected void setInputOutput() {
		input = new GeoElement[4];
		input[0] = A;
		input[1] = B;
		input[2] = C;
		input[3] = n.toGeoElement();

		setOutputLength(1);
		setOutput(0, poly);
		setDependencies(); // done by AlgoElement
	}
	/**
	 * Returns the resulting curve
	 * @return the resulting curve
	 */
	public GeoImplicitPoly getResult() {
		return poly;
	}

	@Override
	public final void compute() {
		// Check if the points are aligned
		double c = A.distance(B);
		double b = C.distance(A);
		double a = B.distance(C);
		double x1 = A.inhomX;
		double y1 = A.inhomY;
		double x2 = B.inhomX;
		double y2 = B.inhomY;
		double x3 = C.inhomX;
		double y3 = C.inhomY;
		
		String equation = "";
		
		double det = (-x2 + x3)*(y1 - y3) + (x1 - x3)*(y2 - y3);
		if(Kernel.isZero(det)){
			poly.setUndefined();
			return;
		}
		String Astr =  "(" + (x3-x2)/det + "*y  + "+
				(y2 - y3)/det+"*x - " + ((x3-x2)*y3+(y2 - y3)*x3)/det + ")";
		String Bstr =  "(" + (x1-x3)/det + "*y  + "+
				(y3 - y1)/det+"*x - " + ((x1-x3)*y1+(y3 - y1)*x1)/det + ")";
		String Cstr =  "(" + (x2-x1)/det + "*y  + "+
				(y1 - y2)/det+"*x - " + ((x2-x1)*y2+(y1 - y2)*x2)/det + ")";
		
		double ab_c, a_bc, _abc;
		
		switch ((int) n.getDouble())
		{
			case 1: 
				equation = "a^4*c^2*A^2*B + a^2*b^2*c^2*A^2*B - 2*b^4*c^2*A^2*B - 2*a^2*c^4*A^2*B + b^2*c^4*A^2*B + c^6*A^2*B +  2*a^4*c^2*A*B^2 - a^2*b^2*c^2*A*B^2 - b^4*c^2*A*B^2 - a^2*c^4*A*B^2 + 2*b^2*c^4*A*B^2 - c^6*A*B^2 - a^4*b^2*A^2*C +  2*a^2*b^4*A^2*C - b^6*A^2*C - a^2*b^2*c^2*A^2*C - b^4*c^2*A^2*C + 2*b^2*c^4*A^2*C + a^6*B^2*C - 2*a^4*b^2*B^2*C +  a^2*b^4*B^2*C + a^4*c^2*B^2*C + a^2*b^2*c^2*B^2*C - 2*a^2*c^4*B^2*C - 2*a^4*b^2*A*C^2 + a^2*b^4*A*C^2 + b^6*A*C^2 + a^2*b^2*c^2*A*C^2 - 2*b^4*c^2*A*C^2 + b^2*c^4*A*C^2 - a^6*B*C^2 - a^4*b^2*B*C^2 + 2*a^2*b^4*B*C^2 + 2*a^4*c^2*B*C^2 - a^2*b^2*c^2*B*C^2 - a^2*c^4*B*C^2 = 0";
				break;
			case 2: 
				equation = "c^2*A^2*B - c^2*A*B^2 - b^2*A^2*C + a^2*B^2*C + b^2*A*C^2 - a^2*B*C^2 = 0";
				break;
			case 3: 
				equation = 	"a^2*b^2*c^2*A^2*B - b^4*c^2*A^2*B + b^2*c^4*A^2*B + a^4*c^2*A*B^2 - a^2*b^2*c^2*A*B^2 - a^2*c^4*A*B^2 - a^2*b^2*c^2*A^2*C - b^4*c^2*A^2*C + b^2*c^4*A^2*C + a^4*c^2*B^2*C + a^2*b^2*c^2*B^2*C - a^2*c^4*B^2*C - a^4*b^2*A*C^2 + a^2*b^4*A*C^2 + a^2*b^2*c^2*A*C^2 - a^4*b^2*B*C^2 + a^2*b^4*B*C^2 - a^2*b^2*c^2*B*C^2 = 0";
				break;
			case 4: 
				equation = 	"a^4*c^2*A^2*B + 2*a^2*b^2*c^2*A^2*B - 3*b^4*c^2*A^2*B - 2*a^2*c^4*A^2*B + 2*b^2*c^4*A^2*B + c^6*A^2*B + 3*a^4*c^2*A*B^2 - 2*a^2*b^2*c^2*A*B^2 - b^4*c^2*A*B^2 - 2*a^2*c^4*A*B^2 + 2*b^2*c^4*A*B^2 - c^6*A*B^2 - a^4*b^2*A^2*C + 2*a^2*b^4*A^2*C - b^6*A^2*C - 2*a^2*b^2*c^2*A^2*C - 2*b^4*c^2*A^2*C + 3*b^2*c^4*A^2*C + a^6*B^2*C - 2*a^4*b^2*B^2*C + a^2*b^4*B^2*C + 2*a^4*c^2*B^2*C + 2*a^2*b^2*c^2*B^2*C - 3*a^2*c^4*B^2*C - 3*a^4*b^2*A*C^2 + 2*a^2*b^4*A*C^2 + b^6*A*C^2 + 2*a^2*b^2*c^2*A*C^2 - 2*b^4*c^2*A*C^2 + b^2*c^4*A*C^2 - a^6*B*C^2 - 2*a^4*b^2*B*C^2 + 3*a^2*b^4*B*C^2 + 2*a^4*c^2*B*C^2 - 2*a^2*b^2*c^2*B*C^2 - a^2*c^4*B*C^2 = 0";
				break;
			case 5: 
				equation = 	"a^4*c^2*A^2*B - a^2*b^2*c^2*A^2*B - 2*a^2*c^4*A^2*B - b^2*c^4*A^2*B + c^6*A^2*B + a^2*b^2*c^2*A*B^2 - b^4*c^2*A*B^2 + a^2*c^4*A*B^2 + 2*b^2*c^4*A*B^2 - c^6*A*B^2 - a^4*b^2*A^2*C + 2*a^2*b^4*A^2*C - b^6*A^2*C + a^2*b^2*c^2*A^2*C + b^4*c^2*A^2*C + a^6*B^2*C - 2*a^4*b^2*B^2*C + a^2*b^4*B^2*C - a^4*c^2*B^2*C - a^2*b^2*c^2*B^2*C - a^2*b^4*A*C^2 + b^6*A*C^2 - a^2*b^2*c^2*A*C^2 - 2*b^4*c^2*A*C^2 + b^2*c^4*A*C^2 - a^6*B*C^2 + a^4*b^2*B*C^2 + 2*a^4*c^2*B*C^2 + a^2*b^2*c^2*B*C^2 - a^2*c^4*B*C^2 = 0";
				break;
			case 6: 
				equation = 	"-a^4*c^2*A^2*B + b^4*c^2*A^2*B + 2*a^2*c^4*A^2*B - c^6*A^2*B - a^4*c^2*A*B^2 + b^4*c^2*A*B^2 - 2*b^2*c^4*A*B^2 + c^6*A*B^2 + a^4*b^2*A^2*C - 2*a^2*b^4*A^2*C + b^6*A^2*C - b^2*c^4*A^2*C - a^6*B^2*C + 2*a^4*b^2*B^2*C - a^2*b^4*B^2*C + a^2*c^4*B^2*C + a^4*b^2*A*C^2 - b^6*A*C^2 + 2*b^4*c^2*A*C^2 - b^2*c^4*A*C^2 + a^6*B*C^2 - a^2*b^4*B*C^2 - 2*a^4*c^2*B*C^2 + a^2*c^4*B*C^2 = 0";
				break;
			case 7: 
				equation = 	"a^2*A^2*B - b^2*A^2*B + c^2*A^2*B + a^2*A*B^2 - b^2*A*B^2 - c^2*A*B^2 - a^2*A^2*C - b^2*A^2*C + c^2*A^2*C + a^2*B^2*C + b^2*B^2*C - c^2*B^2*C - a^2*A*C^2 + b^2*A*C^2 + c^2*A*C^2 - a^2*B*C^2 + b^2*B*C^2 - c^2*B*C^2 = 0";
				break;
			case 8: 
				equation = 	"a^4*A^2*B - b^4*A^2*B - a^2*c^2*A^2*B + c^4*A^2*B + a^4*A*B^2 - b^4*A*B^2 + b^2*c^2*A*B^2 - c^4*A*B^2 - a^4*A^2*C + a^2*b^2*A^2*C - b^4*A^2*C + c^4*A^2*C + a^4*B^2*C - a^2*b^2*B^2*C + b^4*B^2*C - c^4*B^2*C - a^4*A*C^2 + b^4*A*C^2 - b^2*c^2*A*C^2 + c^4*A*C^2 - a^4*B*C^2 + b^4*B*C^2 + a^2*c^2*B*C^2 - c^4*B*C^2 = 0";
				break;
			case 9: 
				equation = 	"-a^2*c^4*A^2*B - b^2*c^4*A^2*B + c^6*A^2*B + a^2*c^4*A*B^2 + b^2*c^4*A*B^2 - c^6*A*B^2 + a^2*b^4*A^2*C - b^6*A^2*C + b^4*c^2*A^2*C + 2*a^4*b^2*A*B*C - 2*a^2*b^4*A*B*C - 2*a^4*c^2*A*B*C + 2*b^4*c^2*A*B*C + 2*a^2*c^4*A*B*C - 2*b^2*c^4*A*B*C + a^6*B^2*C - a^4*b^2*B^2*C - a^4*c^2*B^2*C - a^2*b^4*A*C^2 + b^6*A*C^2 - b^4*c^2*A*C^2 - a^6*B*C^2 + a^4*b^2*B*C^2 + a^4*c^2*B*C^2 = 0";
				break;
			case 10: 
				equation = 	"-2*(a^2 + b^2 + c^2)*A*B*C + (a^2 + b^2 - c^2)*(A^2 + B^2)*C + (a^2 - b^2 + c^2)*B*(A^2 + C^2) + (-a^2 + b^2 + c^2)*A*(B^2 + C^2) = 0";
				break;
			case 11: 
				equation = 	"a^6*A^2*B - a^4*b^2*A^2*B - a^2*b^4*A^2*B + b^6*A^2*B - a^4*c^2*A^2*B + 2*a^2*b^2*c^2*A^2*B - b^4*c^2*A^2*B - a^2*c^4*A^2*B - b^2*c^4*A^2*B + c^6*A^2*B + a^6*A*B^2 - a^4*b^2*A*B^2 - a^2*b^4*A*B^2 + b^6*A*B^2 - a^4*c^2*A*B^2 + 2*a^2*b^2*c^2*A*B^2 - b^4*c^2*A*B^2 - a^2*c^4*A*B^2 - b^2*c^4*A*B^2 + c^6*A*B^2 + a^6*A^2*C - a^4*b^2*A^2*C - a^2*b^4*A^2*C + b^6*A^2*C - a^4*c^2*A^2*C + 2*a^2*b^2*c^2*A^2*C - b^4*c^2*A^2*C - a^2*c^4*A^2*C - b^2*c^4*A^2*C + c^6*A^2*C + 2*a^6*A*B*C - 2*a^4*b^2*A*B*C - 2*a^2*b^4*A*B*C + 2*b^6*A*B*C - 2*a^4*c^2*A*B*C + 12*a^2*b^2*c^2*A*B*C - 2*b^4*c^2*A*B*C - 2*a^2*c^4*A*B*C - 2*b^2*c^4*A*B*C + 2*c^6*A*B*C + a^6*B^2*C - a^4*b^2*B^2*C - a^2*b^4*B^2*C + b^6*B^2*C - a^4*c^2*B^2*C + 2*a^2*b^2*c^2*B^2*C - b^4*c^2*B^2*C - a^2*c^4*B^2*C - b^2*c^4*B^2*C + c^6*B^2*C + a^6*A*C^2 - a^4*b^2*A*C^2 - a^2*b^4*A*C^2 + b^6*A*C^2 - a^4*c^2*A*C^2 + 2*a^2*b^2*c^2*A*C^2 - b^4*c^2*A*C^2 - a^2*c^4*A*C^2 - b^2*c^4*A*C^2 + c^6*A*C^2 + a^6*B*C^2 - a^4*b^2*B*C^2 - a^2*b^4*B*C^2 + b^6*B*C^2 - a^4*c^2*B*C^2 + 2*a^2*b^2*c^2*B*C^2 - b^4*c^2*B*C^2 - a^2*c^4*B*C^2 - b^2*c^4*B*C^2 + c^6*B*C^2 = 0";
				break;
			case 12:
				equation = 	" a^2*b^2*c^2*A^2*B + a^2*b^2*c^2*A*B^2 + a^2*b^2*c^2*A^2*C - a^4*b^2*A*B*C - a^2*b^4*A*B*C - a^4*c^2*A*B*C - b^4*c^2*A*B*C - a^2*c^4*A*B*C - b^2*c^4*A*B*C + a^2*b^2*c^2*B^2*C + a^2*b^2*c^2*A*C^2 + a^2*b^2*c^2*B*C^2 = 0";
				break;
			case 13: 
				equation = 	"a^3*A^2*B - a^2*b*A^2*B - a*b^2*A^2*B + b^3*A^2*B - a^2*c*A^2*B + 2*a*b*c*A^2*B - b^2*c*A^2*B - a*c^2*A^2*B - b*c^2*A^2*B + c^3*A^2*B + a^3*A*B^2 - a^2*b*A*B^2 - a*b^2*A*B^2 + b^3*A*B^2 - a^2*c*A*B^2 + 2*a*b*c*A*B^2 - b^2*c*A*B^2 - a*c^2*A*B^2 - b*c^2*A*B^2 + c^3*A*B^2 + a^3*A^2*C - a^2*b*A^2*C - a*b^2*A^2*C + b^3*A^2*C - a^2*c*A^2*C + 2*a*b*c*A^2*C - b^2*c*A^2*C - a*c^2*A^2*C - b*c^2*A^2*C + c^3*A^2*C + 2*a^3*A*B*C - 2*a^2*b*A*B*C - 2*a*b^2*A*B*C + 2*b^3*A*B*C - 2*a^2*c*A*B*C + 12*a*b*c*A*B*C - 2*b^2*c*A*B*C - 2*a*c^2*A*B*C - 2*b*c^2*A*B*C + 2*c^3*A*B*C + a^3*B^2*C - a^2*b*B^2*C - a*b^2*B^2*C + b^3*B^2*C - a^2*c*B^2*C + 2*a*b*c*B^2*C - b^2*c*B^2*C - a*c^2*B^2*C - b*c^2*B^2*C + c^3*B^2*C + a^3*A*C^2 - a^2*b*A*C^2 - a*b^2*A*C^2 + b^3*A*C^2 - a^2*c*A*C^2 + 2*a*b*c*A*C^2 - b^2*c*A*C^2 - a*c^2*A*C^2 - b*c^2*A*C^2 + c^3*A*C^2 + a^3*B*C^2 - a^2*b*B*C^2 - a*b^2*B*C^2 + b^3*B*C^2 - a^2*c*B*C^2 + 2*a*b*c*B*C^2 - b^2*c*B*C^2 - a*c^2*B*C^2 - b*c^2*B*C^2 + c^3*B*C^2 = 0";
				break;
			case 14: 
				equation = 	"a*b*c*A^2*B + a*b*c*A*B^2 + a*b*c*A^2*C - a^2*b*A*B*C - a*b^2*A*B*C - a^2*c*A*B*C - b^2*c*A*B*C - a*c^2*A*B*C - b*c^2*A*B*C + a*b*c*B^2*C + a*b*c*A*C^2 + a*b*c*B*C^2 = 0";
				break;
			case 15: 
				equation = 	"A^2*B + A*B^2 + A^2*C - 6*A*B*C + B^2*C + A*C^2 + B*C^2 = 0";
				break;
			case 16: 
				equation = 	"A^2*B + A*B^2 + A^2*C + B^2*C + A*C^2 + B*C^2 == 0";
				break;
			case 17: 
				equation = 	"b^4*c^2*A^2*B - a^2*c^4*A^2*B + a^4*c^2*A*B^2 - b^2*c^4*A*B^2 - a^2*b^4*A^2*C + b^2*c^4*A^2*C - a^4*b^2*B^2*C + a^2*c^4*B^2*C + a^4*b^2*A*C^2 - b^4*c^2*A*C^2 + a^2*b^4*B*C^2 - a^4*c^2*B*C^2 = 0";
				break;
			case 18: 
				equation = 	"-a^2*c^2*A^2*B + c^4*A^2*B + b^2*c^2*A*B^2 - c^4*A*B^2 + a^2*b^2*A^2*C - b^4*A^2*C + a^4*B^2*C - a^2*b^2*B^2*C + b^4*A*C^2 - b^2*c^2*A*C^2 - a^4*B*C^2 + a^2*c^2*B*C^2 = 0";
				break;
			case 19: 
				equation = 	"-a^4*b^2*c^2*A^2*B + a^2*b^4*c^2*A^2*B - b^4*c^4*A^2*B + b^2*c^6*A^2*B - a^4*b^2*c^2*A*B^2 + a^2*b^4*c^2*A*B^2 + a^4*c^4*A*B^2 - a^2*c^6*A*B^2 + a^4*b^2*c^2*A^2*C - b^6*c^2*A^2*C - a^2*b^2*c^4*A^2*C + b^4*c^4*A^2*C + a^6*c^2*B^2*C - a^2*b^4*c^2*B^2*C - a^4*c^4*B^2*C + a^2*b^2*c^4*B^2*C - a^4*b^4*A*C^2 + a^2*b^6*A*C^2 + a^4*b^2*c^2*A*C^2 - a^2*b^2*c^4*A*C^2 - a^6*b^2*B*C^2 + a^4*b^4*B*C^2 - a^2*b^4*c^2*B*C^2 + a^2*b^2*c^4*B*C^2 = 0";
				break;
			case 20: 
				equation = 	"b^4*c^2*A^2*B + a^2*c^4*A^2*B - a^4*c^2*A*B^2 - b^2*c^4*A*B^2 - a^2*b^4*A^2*C - b^2*c^4*A^2*C + a^4*b^2*B^2*C + a^2*c^4*B^2*C + a^4*b^2*A*C^2 + b^4*c^2*A*C^2 - a^2*b^4*B*C^2 - a^4*c^2*B*C^2 = 0";
				break;
			case 21: 
				equation = 	"-a^2*b^2*c^2*A^2*B + b^2*c^4*A^2*B - a^2*b^2*c^2*A*B^2 + a^2*c^4*A*B^2 - a^2*b^2*c^2*A^2*C + b^4*c^2*A^2*C + a^4*c^2*B^2*C - a^2*b^2*c^2*B^2*C + a^2*b^4*A*C^2 - a^2*b^2*c^2*A*C^2 + a^4*b^2*B*C^2 - a^2*b^2*c^2*B*C^2 = 0";
				break;
			case 22: 
				equation = 	"a^4*c^2*A^2*B - a^2*b^2*c^2*A^2*B - 2*b^4*c^2*A^2*B + 3*b^2*c^4*A^2*B - c^6*A^2*B - 2*a^4*c^2*A*B^2 - a^2*b^2*c^2*A*B^2 + b^4*c^2*A*B^2 + 3*a^2*c^4*A*B^2 - c^6*A*B^2 + a^4*b^2*A^2*C - b^6*A^2*C - a^2*b^2*c^2*A^2*C + 3*b^4*c^2*A^2*C - 2*b^2*c^4*A^2*C + 2*a^6*A*B*C - 2*a^4*b^2*A*B*C - 2*a^2*b^4*A*B*C + 2*b^6*A*B*C - 2*a^4*c^2*A*B*C + 6*a^2*b^2*c^2*A*B*C - 2*b^4*c^2*A*B*C - 2*a^2*c^4*A*B*C - 2*b^2*c^4*A*B*C + 2*c^6*A*B*C - a^6*B^2*C + a^2*b^4*B^2*C + 3*a^4*c^2*B^2*C - a^2*b^2*c^2*B^2*C - 2*a^2*c^4*B^2*C - 2*a^4*b^2*A*C^2 + 3*a^2*b^4*A*C^2 - b^6*A*C^2 - a^2*b^2*c^2*A*C^2 + b^2*c^4*A*C^2 - a^6*B*C^2 + 3*a^4*b^2*B*C^2 - 2*a^2*b^4*B*C^2 - a^2*b^2*c^2*B*C^2 + a^2*c^4*B*C^2 = 0";
				break;
			case 23: 
				equation = 	"-a^8*b^2*A^2*B + 3*a^6*b^4*A^2*B - 3*a^4*b^6*A^2*B + a^2*b^8*A^2*B + a^6*b^2*c^2*A^2*B - 2*a^4*b^4*c^2*A^2*B + 2*a^2*b^6*c^2*A^2*B - b^8*c^2*A^2*B - a^6*c^4*A^2*B - a^4*b^2*c^4*A^2*B + b^6*c^4*A^2*B + 3*a^4*c^6*A^2*B + a^2*b^2*c^6*A^2*B - b^4*c^6*A^2*B - 3*a^2*c^8*A^2*B + c^10*A^2*B - a^8*b^2*A*B^2 + 3*a^6*b^4*A*B^2 - 3*a^4*b^6*A*B^2 + a^2*b^8*A*B^2 + a^8*c^2*A*B^2 - 2*a^6*b^2*c^2*A*B^2 + 2*a^4*b^4*c^2*A*B^2 - a^2*b^6*c^2*A*B^2 - a^6*c^4*A*B^2 + a^2*b^4*c^4*A*B^2 + b^6*c^4*A*B^2 + a^4*c^6*A*B^2 - a^2*b^2*c^6*A*B^2 - 3*b^4*c^6*A*B^2 + 3*b^2*c^8*A*B^2 - c^10*A*B^2 + a^6*b^4*A^2*C - 3*a^4*b^6*A^2*C + 3*a^2*b^8*A^2*C - b^10*A^2*C + a^8*c^2*A^2*C - a^6*b^2*c^2*A^2*C + a^4*b^4*c^2*A^2*C - a^2*b^6*c^2*A^2*C - 3*a^6*c^4*A^2*C + 2*a^4*b^2*c^4*A^2*C + b^6*c^4*A^2*C + 3*a^4*c^6*A^2*C - 2*a^2*b^2*c^6*A^2*C - b^4*c^6*A^2*C - a^2*c^8*A^2*C + b^2*c^8*A^2*C + a^10*B^2*C - 3*a^8*b^2*B^2*C + 3*a^6*b^4*B^2*C - a^4*b^6*B^2*C + a^6*b^2*c^2*B^2*C - a^4*b^4*c^2*B^2*C + a^2*b^6*c^2*B^2*C - b^8*c^2*B^2*C - a^6*c^4*B^2*C - 2*a^2*b^4*c^4*B^2*C + 3*b^6*c^4*B^2*C + a^4*c^6*B^2*C + 2*a^2*b^2*c^6*B^2*C - 3*b^4*c^6*B^2*C - a^2*c^8*B^2*C + b^2*c^8*B^2*C - a^8*b^2*A*C^2 + a^6*b^4*A*C^2 - a^4*b^6*A*C^2 + b^10*A*C^2 + a^8*c^2*A*C^2 + 2*a^6*b^2*c^2*A*C^2 + a^2*b^6*c^2*A*C^2 - 3*b^8*c^2*A*C^2 - 3*a^6*c^4*A*C^2 - 2*a^4*b^2*c^4*A*C^2 - a^2*b^4*c^4*A*C^2 + 3*b^6*c^4*A*C^2 + 3*a^4*c^6*A*C^2 + a^2*b^2*c^6*A*C^2 - b^4*c^6*A*C^2 - a^2*c^8*A*C^2 - a^10*B*C^2 + a^6*b^4*B*C^2 - a^4*b^6*B*C^2 + a^2*b^8*B*C^2 + 3*a^8*c^2*B*C^2 - a^6*b^2*c^2*B*C^2 - 2*a^2*b^6*c^2*B*C^2 - b^8*c^2*B*C^2 - 3*a^6*c^4*B*C^2 + a^4*b^2*c^4*B*C^2 + 2*a^2*b^4*c^4*B*C^2 + 3*b^6*c^4*B*C^2 + a^4*c^6*B*C^2 - a^2*b^2*c^6*B*C^2 - 3*b^4*c^6*B*C^2 + b^2*c^8*B*C^2 = 0";
				break;
			case 24: 
				equation = 	"b^2*c^2*A^2*B + a^2*c^2*A*B^2 + b^2*c^2*A^2*C + a^2*c^2*B^2*C + a^2*b^2*A*C^2 + a^2*b^2*B*C^2 = 0";
				break;
			case 25: 
				equation = 	"a^6*A^2*B - 3*a^4*b^2*A^2*B + 3*a^2*b^4*A^2*B - b^6*A^2*B + a^2*b^2*c^2*A^2*B - b^4*c^2*A^2*B - 2*a^2*c^4*A^2*B + b^2*c^4*A^2*B + c^6*A^2*B + a^6*A*B^2 - 3*a^4*b^2*A*B^2 + 3*a^2*b^4*A*B^2 - b^6*A*B^2 + a^4*c^2*A*B^2 - a^2*b^2*c^2*A*B^2 - a^2*c^4*A*B^2 + 2*b^2*c^4*A*B^2 - c^6*A*B^2 - a^6*A^2*C + 2*a^2*b^4*A^2*C - b^6*A^2*C + 3*a^4*c^2*A^2*C - a^2*b^2*c^2*A^2*C - b^4*c^2*A^2*C - 3*a^2*c^4*A^2*C + b^2*c^4*A^2*C + c^6*A^2*C - 2*a^4*b^2*A*B*C + 2*a^2*b^4*A*B*C + 2*a^4*c^2*A*B*C - 2*b^4*c^2*A*B*C - 2*a^2*c^4*A*B*C + 2*b^2*c^4*A*B*C + a^6*B^2*C - 2*a^4*b^2*B^2*C + b^6*B^2*C + a^4*c^2*B^2*C + a^2*b^2*c^2*B^2*C - 3*b^4*c^2*B^2*C - a^2*c^4*B^2*C + 3*b^2*c^4*B^2*C - c^6*B^2*C - a^6*A*C^2 - a^4*b^2*A*C^2 + a^2*b^4*A*C^2 + b^6*A*C^2 + 3*a^4*c^2*A*C^2 + a^2*b^2*c^2*A*C^2 - 2*b^4*c^2*A*C^2 - 3*a^2*c^4*A*C^2 + c^6*A*C^2 - a^6*B*C^2 - a^4*b^2*B*C^2 + a^2*b^4*B*C^2 + b^6*B*C^2 + 2*a^4*c^2*B*C^2 - a^2*b^2*c^2*B*C^2 - 3*b^4*c^2*B*C^2 + 3*b^2*c^4*B*C^2 - c^6*B*C^2 = 0";
				break;
			case 26: 
				equation = 	"-a^4*c^2*A^2*B + 2*a^2*b^2*c^2*A^2*B - b^4*c^2*A^2*B + a^2*c^4*A^2*B + b^2*c^4*A^2*B + a^4*c^2*A*B^2 - 2*a^2*b^2*c^2*A*B^2 + b^4*c^2*A*B^2 - a^2*c^4*A*B^2 - b^2*c^4*A*B^2 + a^4*b^2*A^2*C - a^2*b^4*A^2*C - 2*a^2*b^2*c^2*A^2*C - b^4*c^2*A^2*C + b^2*c^4*A^2*C + 2*a^4*b^2*A*B*C - 2*a^2*b^4*A*B*C - 2*a^4*c^2*A*B*C + 2*b^4*c^2*A*B*C + 2*a^2*c^4*A*B*C - 2*b^2*c^4*A*B*C + a^4*b^2*B^2*C - a^2*b^4*B^2*C + a^4*c^2*B^2*C + 2*a^2*b^2*c^2*B^2*C - a^2*c^4*B^2*C - a^4*b^2*A*C^2 + a^2*b^4*A*C^2 + 2*a^2*b^2*c^2*A*C^2 + b^4*c^2*A*C^2 - b^2*c^4*A*C^2 - a^4*b^2*B*C^2 + a^2*b^4*B*C^2 - a^4*c^2*B*C^2 - 2*a^2*b^2*c^2*B*C^2 + a^2*c^4*B*C^2 = 0";
				break;
			case 27: 
				equation = 	"a^6*c^2*A^2*B - 2*a^4*b^2*c^2*A^2*B + a^2*b^4*c^2*A^2*B - 2*a^4*c^4*A^2*B + 2*a^2*b^2*c^4*A^2*B + a^2*c^6*A^2*B + a^4*b^2*c^2*A*B^2 - 2*a^2*b^4*c^2*A*B^2 + b^6*c^2*A*B^2 + 2*a^2*b^2*c^4*A*B^2 - 2*b^4*c^4*A*B^2 + b^2*c^6*A*B^2 + a^6*b^2*A^2*C - 2*a^4*b^4*A^2*C + a^2*b^6*A^2*C - 2*a^4*b^2*c^2*A^2*C + 2*a^2*b^4*c^2*A^2*C + a^2*b^2*c^4*A^2*C + 2*a^6*b^2*A*B*C - 4*a^4*b^4*A*B*C + 2*a^2*b^6*A*B*C + 2*a^6*c^2*A*B*C - 2*a^4*b^2*c^2*A*B*C - 2*a^2*b^4*c^2*A*B*C + 2*b^6*c^2*A*B*C - 4*a^4*c^4*A*B*C - 2*a^2*b^2*c^4*A*B*C - 4*b^4*c^4*A*B*C + 2*a^2*c^6*A*B*C + 2*b^2*c^6*A*B*C + a^6*b^2*B^2*C - 2*a^4*b^4*B^2*C + a^2*b^6*B^2*C + 2*a^4*b^2*c^2*B^2*C - 2*a^2*b^4*c^2*B^2*C + a^2*b^2*c^4*B^2*C + a^4*b^2*c^2*A*C^2 + 2*a^2*b^4*c^2*A*C^2 + b^6*c^2*A*C^2 - 2*a^2*b^2*c^4*A*C^2 - 2*b^4*c^4*A*C^2 + b^2*c^6*A*C^2 + a^6*c^2*B*C^2 + 2*a^4*b^2*c^2*B*C^2 + a^2*b^4*c^2*B*C^2 - 2*a^4*c^4*B*C^2 - 2*a^2*b^2*c^4*B*C^2 + a^2*c^6*B*C^2 = 0";
				break;
			case 28: 
				equation = 	"a^4*c^2*A^2*B - a^2*b^2*c^2*A^2*B - a^2*c^4*A^2*B + a^2*b^2*c^2*A*B^2 - b^4*c^2*A*B^2 + b^2*c^4*A*B^2 - a^4*b^2*A^2*C + a^2*b^4*A^2*C + a^2*b^2*c^2*A^2*C - 2*a^4*b^2*A*B*C + 2*a^2*b^4*A*B*C + 2*a^4*c^2*A*B*C - 2*b^4*c^2*A*B*C - 2*a^2*c^4*A*B*C + 2*b^2*c^4*A*B*C - a^4*b^2*B^2*C + a^2*b^4*B^2*C - a^2*b^2*c^2*B^2*C - a^2*b^2*c^2*A*C^2 - b^4*c^2*A*C^2 + b^2*c^4*A*C^2 + a^4*c^2*B*C^2 + a^2*b^2*c^2*B*C^2 - a^2*c^4*B*C^2 = 0";
				break;
			case 29: 
				ab_c = Math.cos((1.0/3.0)*Math.acos((a*a + b*b - c*c)/(2*a*b)));
				a_bc = Math.cos((1.0/3.0)*Math.acos((a*a - b*b + c*c)/(2*a*c)));
				_abc = Math.cos((1.0/3.0)*Math.acos((-a*a + b*b + c*c)/(2*b*c)));
				
				equation = 	"-b^2*c*A^2*C*" + ab_c + " + a^2*c*B^2*C*" + ab_c + "+ b*c^2*A^2*B*" + a_bc + " - a^2*b*B*C^2*" + a_bc + " - 2*a*c^2*A*B^2*" + ab_c*a_bc + " + 2*a*b^2*A*C^2*" + ab_c*a_bc + " - a*c^2*A*B^2*" + _abc + " + a*b^2*A*C^2*" + _abc + " + " + "2*b*c^2*A^2*B*" + ab_c*_abc +" - 2*a^2*b*B*C^2*" + ab_c*_abc + " - 2*b^2*c*A^2*C*" + a_bc*_abc + " + 2*a^2*c*B^2*C*" + a_bc*_abc + " = 0";
				break;
			case 30: 
				ab_c = Math.cos((1.0/3.0)*(2*Math.PI + Math.acos((a*a + b*b - c*c)/(2*a*b))));
				a_bc = Math.cos((1.0/3.0)*(2*Math.PI + Math.acos((a*a - b*b + c*c)/(2*a*c))));
				_abc = Math.cos((1.0/3.0)*(2*Math.PI + Math.acos((-a*a + b*b + c*c)/(2*b*c))));
				
				equation = "-b^2*c*A^2*C*" + ab_c + " + a^2*c*B^2*C*" + ab_c + " + b*c^2*A^2*B*" + a_bc + "- a^2*b*B*C^2*" + a_bc + "- 2*a*c^2*A*B^2*" + ab_c*a_bc + " + 2*a*b^2*A*C^2*" + ab_c*a_bc + " - a*c^2*A*B^2*" + _abc + " + a*b^2*A*C^2*" + _abc + "+ 2*b*c^2*A^2*B*" + ab_c*_abc + " - 2*a^2*b*B*C^2*" + ab_c*_abc + " - 2*b^2*c*A^2*C*" + a_bc*_abc + " + 2*a^2*c*B^2*C*" + a_bc*_abc + " = 0";
				break;
			case 31: 
				ab_c = Math.cos((1.0/3.0)*(-2*Math.PI + Math.acos((a*a + b*b - c*c)/(2*a*b))));
				a_bc = Math.cos((1.0/3.0)*(-2*Math.PI + Math.acos((a*a - b*b + c*c)/(2*a*c))));
				_abc = Math.cos((1.0/3.0)*(-2*Math.PI + Math.acos((-a*a + b*b + c*c)/(2*b*c))));
				
				equation = 	"-b^2*c*A^2*C*" + ab_c + " + a^2*c*B^2*C*" + ab_c + "+ b*c^2*A^2*B*" + a_bc + " - a^2*b*B*C^2*" + a_bc + " - 2*a*c^2*A*B^2*" + ab_c*a_bc + " + 2*a*b^2*A*C^2*" + ab_c*a_bc + " - a*c^2*A*B^2*" + _abc + " + a*b^2*A*C^2*" + _abc + " + 2*b*c^2*A^2*B*" + ab_c*_abc + " - 2*a^2*b*B*C^2*" + ab_c*_abc + " - 2*b^2*c*A^2*C*" + a_bc*_abc + " + 2*a^2*c*B^2*C*" + a_bc*_abc + " = 0";
				break;
			case 32: 
				equation = 	" a^6*A^2*B - 3*a^4*b^2*A^2*B + 3*a^2*b^4*A^2*B - b^6*A^2*B + 3*a^4*c^2*A^2*B + 6*a^2*b^2*c^2*A^2*B - 9*b^4*c^2*A^2*B - 9*a^2*c^4*A^2*B + 5*b^2*c^4*A^2*B + 5*c^6*A^2*B + a^6*A*B^2 - 3*a^4*b^2*A*B^2 + 3*a^2*b^4*A*B^2 - b^6*A*B^2 + 9*a^4*c^2*A*B^2 - 6*a^2*b^2*c^2*A*B^2 - 3*b^4*c^2*A*B^2 - 5*a^2*c^4*A*B^2 + 9*b^2*c^4*A*B^2 - 5*c^6*A*B^2 - a^6*A^2*C - 3*a^4*b^2*A^2*C + 9*a^2*b^4*A^2*C - 5*b^6*A^2*C + 3*a^4*c^2*A^2*C - 6*a^2*b^2*c^2*A^2*C - 5*b^4*c^2*A^2*C - 3*a^2*c^4*A^2*C + 9*b^2*c^4*A^2*C + c^6*A^2*C + 5*a^6*B^2*C - 9*a^4*b^2*B^2*C + 3*a^2*b^4*B^2*C + b^6*B^2*C + 5*a^4*c^2*B^2*C + 6*a^2*b^2*c^2*B^2*C - 3*b^4*c^2*B^2*C - 9*a^2*c^4*B^2*C + 3*b^2*c^4*B^2*C - c^6*B^2*C - a^6*A*C^2 - 9*a^4*b^2*A*C^2 + 5*a^2*b^4*A*C^2 + 5*b^6*A*C^2 + 3*a^4*c^2*A*C^2 + 6*a^2*b^2*c^2*A*C^2 - 9*b^4*c^2*A*C^2 - 3*a^2*c^4*A*C^2 + 3*b^2*c^4*A*C^2 + c^6*A*C^2 - 5*a^6*B*C^2 - 5*a^4*b^2*B*C^2 + 9*a^2*b^4*B*C^2 + b^6*B*C^2 + 9*a^4*c^2*B*C^2 - 6*a^2*b^2*c^2*B*C^2 - 3*b^4*c^2*B*C^2 - 3*a^2*c^4*B*C^2 + 3*b^2*c^4*B*C^2 - c^6*B*C^2 = 0";
				break;
			case 33: 
				equation = 	"a^2*c*A^2*B - b^2*c*A^2*B + a*c^2*A^2*B + b*c^2*A^2*B + a^2*c*A*B^2 - b^2*c*A*B^2 - a*c^2*A*B^2 - b*c^2*A*B^2 - a^2*b*A^2*C - a*b^2*A^2*C - b^2*c*A^2*C + b*c^2*A^2*C + a^2*b*B^2*C + a*b^2*B^2*C + a^2*c*B^2*C - a*c^2*B^2*C - a^2*b*A*C^2 + a*b^2*A*C^2 + b^2*c*A*C^2 + b*c^2*A*C^2 - a^2*b*B*C^2 + a*b^2*B*C^2 - a^2*c*B*C^2 - a*c^2*B*C^2 = 0";
				break;
			case 34: 
				equation = 	"a*c*A^2*B - b*c*A*B^2 - a*b*A^2*C + a*b*B^2*C + b*c*A*C^2 - a*c*B*C^2 = 0";
				break;
			case 35: 
				equation = 	"-a^2*b^2*c^2*A^2*B + b^4*c^2*A^2*B + a^2*c^4*A^2*B - b^2*c^4*A^2*B - a^4*c^2*A*B^2 + a^2*b^2*c^2*A*B^2 + a^2*c^4*A*B^2 - b^2*c^4*A*B^2 - a^2*b^4*A^2*C + a^2*b^2*c^2*A^2*C + b^4*c^2*A^2*C - b^2*c^4*A^2*C + a^4*b^2*B^2*C - a^4*c^2*B^2*C - a^2*b^2*c^2*B^2*C + a^2*c^4*B^2*C + a^4*b^2*A*C^2 - a^2*b^4*A*C^2 - a^2*b^2*c^2*A*C^2 + b^4*c^2*A*C^2 + a^4*b^2*B*C^2 - a^2*b^4*B*C^2 - a^4*c^2*B*C^2 + a^2*b^2*c^2*B*C^2 = 0";
				break;
			case 36: 
				equation = 	"a^20*A^2*B - 5*a^18*b^2*A^2*B + 13*a^16*b^4*A^2*B - 27*a^14*b^6*A^2*B + 45*a^12*b^8*A^2*B - 53*a^10*b^10*A^2*B + 45*a^8*b^12*A^2*B - 33*a^6*b^14*A^2*B + 22*a^4*b^16*A^2*B - 10*a^2*b^18*A^2*B + 2*b^20*A^2*B - 3*a^18*c^2*A^2*B + 9*a^16*b^2*c^2*A^2*B - 8*a^14*b^4*c^2*A^2*B + 3*a^12*b^6*c^2*A^2*B - 5*a^10*b^8*c^2*A^2*B - 3*a^8*b^10*c^2*A^2*B + 30*a^6*b^12*c^2*A^2*B - 43*a^4*b^14*c^2*A^2*B + 26*a^2*b^16*c^2*A^2*B - 6*b^18*c^2*A^2*B + 6*a^16*c^4*A^2*B - 8*a^14*b^2*c^4*A^2*B - 18*a^12*b^4*c^4*A^2*B + 40*a^10*b^6*c^4*A^2*B - 20*a^8*b^8*c^4*A^2*B - 13*a^6*b^10*c^4*A^2*B + 30*a^4*b^12*c^4*A^2*B - 25*a^2*b^14*c^4*A^2*B + 8*b^16*c^4*A^2*B - 13*a^14*c^6*A^2*B + 22*a^12*b^2*c^6*A^2*B + 15*a^10*b^4*c^6*A^2*B - 40*a^8*b^6*c^6*A^2*B + 10*a^6*b^8*c^6*A^2*B - 3*a^4*b^10*c^6*A^2*B + 20*a^2*b^12*c^6*A^2*B - 11*b^14*c^6*A^2*B + 18*a^12*c^8*A^2*B - 40*a^10*b^2*c^8*A^2*B + 10*a^8*b^4*c^8*A^2*B + 30*a^6*b^6*c^8*A^2*B - 5*a^4*b^8*c^8*A^2*B - 28*a^2*b^10*c^8*A^2*B + 15*b^12*c^8*A^2*B - 13*a^10*c^10*A^2*B + 30*a^8*b^2*c^10*A^2*B - 20*a^6*b^4*c^10*A^2*B - 10*a^4*b^6*c^10*A^2*B + 25*a^2*b^8*c^10*A^2*B - 12*b^10*c^10*A^2*B + 6*a^8*c^12*A^2*B - 9*a^6*b^2*c^12*A^2*B + 6*a^4*b^4*c^12*A^2*B - 9*a^2*b^6*c^12*A^2*B + 6*b^8*c^12*A^2*B - 3*a^6*c^14*A^2*B + 3*a^4*b^2*c^14*A^2*B + 3*a^2*b^4*c^14*A^2*B - 3*b^6*c^14*A^2*B + a^4*c^16*A^2*B - 2*a^2*b^2*c^16*A^2*B + b^4*c^16*A^2*B + 2*a^20*A*B^2 - 10*a^18*b^2*A*B^2 + 22*a^16*b^4*A*B^2 - 33*a^14*b^6*A*B^2 + 45*a^12*b^8*A*B^2 - 53*a^10*b^10*A*B^2 + 45*a^8*b^12*A*B^2 - 27*a^6*b^14*A*B^2 + 13*a^4*b^16*A*B^2 - 5*a^2*b^18*A*B^2 + b^20*A*B^2 - 6*a^18*c^2*A*B^2 + 26*a^16*b^2*c^2*A*B^2 - 43*a^14*b^4*c^2*A*B^2 + 30*a^12*b^6*c^2*A*B^2 - 3*a^10*b^8*c^2*A*B^2 - 5*a^8*b^10*c^2*A*B^2 + 3*a^6*b^12*c^2*A*B^2 - 8*a^4*b^14*c^2*A*B^2 + 9*a^2*b^16*c^2*A*B^2 - 3*b^18*c^2*A*B^2 + 8*a^16*c^4*A*B^2 - 25*a^14*b^2*c^4*A*B^2 + 30*a^12*b^4*c^4*A*B^2 - 13*a^10*b^6*c^4*A*B^2 - 20*a^8*b^8*c^4*A*B^2 + 40*a^6*b^10*c^4*A*B^2 - 18*a^4*b^12*c^4*A*B^2 - 8*a^2*b^14*c^4*A*B^2 + 6*b^16*c^4*A*B^2 - 11*a^14*c^6*A*B^2 + 20*a^12*b^2*c^6*A*B^2 - 3*a^10*b^4*c^6*A*B^2 + 10*a^8*b^6*c^6*A*B^2 - 40*a^6*b^8*c^6*A*B^2 + 15*a^4*b^10*c^6*A*B^2 + 22*a^2*b^12*c^6*A*B^2 - 13*b^14*c^6*A*B^2 + 15*a^12*c^8*A*B^2 - 28*a^10*b^2*c^8*A*B^2 - 5*a^8*b^4*c^8*A*B^2 + 30*a^6*b^6*c^8*A*B^2 + 10*a^4*b^8*c^8*A*B^2 - 40*a^2*b^10*c^8*A*B^2 + 18*b^12*c^8*A*B^2 - 12*a^10*c^10*A*B^2 + 25*a^8*b^2*c^10*A*B^2 - 10*a^6*b^4*c^10*A*B^2 - 20*a^4*b^6*c^10*A*B^2 + 30*a^2*b^8*c^10*A*B^2 - 13*b^10*c^10*A*B^2 + 6*a^8*c^12*A*B^2 - 9*a^6*b^2*c^12*A*B^2 + 6*a^4*b^4*c^12*A*B^2 - 9*a^2*b^6*c^12*A*B^2 + 6*b^8*c^12*A*B^2 - 3*a^6*c^14*A*B^2 + 3*a^4*b^2*c^14*A*B^2 + 3*a^2*b^4*c^14*A*B^2 - 3*b^6*c^14*A*B^2 + a^4*c^16*A*B^2 - 2*a^2*b^2*c^16*A*B^2 + b^4*c^16*A*B^2 + a^20*A^2*C - 3*a^18*b^2*A^2*C + 6*a^16*b^4*A^2*C - 13*a^14*b^6*A^2*C + 18*a^12*b^8*A^2*C - 13*a^10*b^10*A^2*C + 6*a^8*b^12*A^2*C - 3*a^6*b^14*A^2*C + a^4*b^16*A^2*C - 5*a^18*c^2*A^2*C + 9*a^16*b^2*c^2*A^2*C - 8*a^14*b^4*c^2*A^2*C + 22*a^12*b^6*c^2*A^2*C - 40*a^10*b^8*c^2*A^2*C + 30*a^8*b^10*c^2*A^2*C - 9*a^6*b^12*c^2*A^2*C + 3*a^4*b^14*c^2*A^2*C - 2*a^2*b^16*c^2*A^2*C + 13*a^16*c^4*A^2*C - 8*a^14*b^2*c^4*A^2*C - 18*a^12*b^4*c^4*A^2*C + 15*a^10*b^6*c^4*A^2*C + 10*a^8*b^8*c^4*A^2*C - 20*a^6*b^10*c^4*A^2*C + 6*a^4*b^12*c^4*A^2*C + 3*a^2*b^14*c^4*A^2*C + b^16*c^4*A^2*C - 27*a^14*c^6*A^2*C + 3*a^12*b^2*c^6*A^2*C + 40*a^10*b^4*c^6*A^2*C - 40*a^8*b^6*c^6*A^2*C + 30*a^6*b^8*c^6*A^2*C - 10*a^4*b^10*c^6*A^2*C - 9*a^2*b^12*c^6*A^2*C - 3*b^14*c^6*A^2*C + 45*a^12*c^8*A^2*C - 5*a^10*b^2*c^8*A^2*C - 20*a^8*b^4*c^8*A^2*C + 10*a^6*b^6*c^8*A^2*C - 5*a^4*b^8*c^8*A^2*C + 25*a^2*b^10*c^8*A^2*C + 6*b^12*c^8*A^2*C - 53*a^10*c^10*A^2*C - 3*a^8*b^2*c^10*A^2*C - 13*a^6*b^4*c^10*A^2*C - 3*a^4*b^6*c^10*A^2*C - 28*a^2*b^8*c^10*A^2*C - 12*b^10*c^10*A^2*C + 45*a^8*c^12*A^2*C + 30*a^6*b^2*c^12*A^2*C + 30*a^4*b^4*c^12*A^2*C + 20*a^2*b^6*c^12*A^2*C + 15*b^8*c^12*A^2*C - 33*a^6*c^14*A^2*C - 43*a^4*b^2*c^14*A^2*C - 25*a^2*b^4*c^14*A^2*C - 11*b^6*c^14*A^2*C + 22*a^4*c^16*A^2*C + 26*a^2*b^2*c^16*A^2*C + 8*b^4*c^16*A^2*C - 10*a^2*c^18*A^2*C - 6*b^2*c^18*A^2*C + 2*c^20*A^2*C + 4*a^20*A*B*C - 16*a^18*b^2*A*B*C + 30*a^16*b^4*A*B*C - 44*a^14*b^6*A*B*C + 62*a^12*b^8*A*B*C - 72*a^10*b^10*A*B*C + 62*a^8*b^12*A*B*C - 44*a^6*b^14*A*B*C + 30*a^4*b^16*A*B*C - 16*a^2*b^18*A*B*C + 4*b^20*A*B*C - 16*a^18*c^2*A*B*C + 52*a^16*b^2*c^2*A*B*C - 68*a^14*b^4*c^2*A*B*C + 42*a^12*b^6*c^2*A*B*C - 10*a^10*b^8*c^2*A*B*C - 10*a^8*b^10*c^2*A*B*C + 42*a^6*b^12*c^2*A*B*C - 68*a^4*b^14*c^2*A*B*C + 52*a^2*b^16*c^2*A*B*C - 16*b^18*c^2*A*B*C + 30*a^16*c^4*A*B*C - 68*a^14*b^2*c^4*A*B*C + 72*a^12*b^4*c^4*A*B*C - 30*a^10*b^6*c^4*A*B*C - 4*a^8*b^8*c^4*A*B*C - 30*a^6*b^10*c^4*A*B*C + 72*a^4*b^12*c^4*A*B*C - 68*a^2*b^14*c^4*A*B*C + 30*b^16*c^4*A*B*C - 44*a^14*c^6*A*B*C + 42*a^12*b^2*c^6*A*B*C - 30*a^10*b^4*c^6*A*B*C + 16*a^8*b^6*c^6*A*B*C + 16*a^6*b^8*c^6*A*B*C - 30*a^4*b^10*c^6*A*B*C + 42*a^2*b^12*c^6*A*B*C - 44*b^14*c^6*A*B*C + 62*a^12*c^8*A*B*C - 10*a^10*b^2*c^8*A*B*C - 4*a^8*b^4*c^8*A*B*C + 16*a^6*b^6*c^8*A*B*C - 4*a^4*b^8*c^8*A*B*C - 10*a^2*b^10*c^8*A*B*C + 62*b^12*c^8*A*B*C - 72*a^10*c^10*A*B*C - 10*a^8*b^2*c^10*A*B*C - 30*a^6*b^4*c^10*A*B*C - 30*a^4*b^6*c^10*A*B*C - 10*a^2*b^8*c^10*A*B*C - 72*b^10*c^10*A*B*C + 62*a^8*c^12*A*B*C + 42*a^6*b^2*c^12*A*B*C + 72*a^4*b^4*c^12*A*B*C + 42*a^2*b^6*c^12*A*B*C + 62*b^8*c^12*A*B*C - 44*a^6*c^14*A*B*C - 68*a^4*b^2*c^14*A*B*C - 68*a^2*b^4*c^14*A*B*C - 44*b^6*c^14*A*B*C + 30*a^4*c^16*A*B*C + 52*a^2*b^2*c^16*A*B*C + 30*b^4*c^16*A*B*C - 16*a^2*c^18*A*B*C - 16*b^2*c^18*A*B*C + 4*c^20*A*B*C + a^16*b^4*B^2*C - 3*a^14*b^6*B^2*C + 6*a^12*b^8*B^2*C - 13*a^10*b^10*B^2*C + 18*a^8*b^12*B^2*C - 13*a^6*b^14*B^2*C + 6*a^4*b^16*B^2*C - 3*a^2*b^18*B^2*C + b^20*B^2*C - 2*a^16*b^2*c^2*B^2*C + 3*a^14*b^4*c^2*B^2*C - 9*a^12*b^6*c^2*B^2*C + 30*a^10*b^8*c^2*B^2*C - 40*a^8*b^10*c^2*B^2*C + 22*a^6*b^12*c^2*B^2*C - 8*a^4*b^14*c^2*B^2*C + 9*a^2*b^16*c^2*B^2*C - 5*b^18*c^2*B^2*C + a^16*c^4*B^2*C + 3*a^14*b^2*c^4*B^2*C + 6*a^12*b^4*c^4*B^2*C - 20*a^10*b^6*c^4*B^2*C + 10*a^8*b^8*c^4*B^2*C + 15*a^6*b^10*c^4*B^2*C - 18*a^4*b^12*c^4*B^2*C - 8*a^2*b^14*c^4*B^2*C + 13*b^16*c^4*B^2*C - 3*a^14*c^6*B^2*C - 9*a^12*b^2*c^6*B^2*C - 10*a^10*b^4*c^6*B^2*C + 30*a^8*b^6*c^6*B^2*C - 40*a^6*b^8*c^6*B^2*C + 40*a^4*b^10*c^6*B^2*C + 3*a^2*b^12*c^6*B^2*C - 27*b^14*c^6*B^2*C + 6*a^12*c^8*B^2*C + 25*a^10*b^2*c^8*B^2*C - 5*a^8*b^4*c^8*B^2*C + 10*a^6*b^6*c^8*B^2*C - 20*a^4*b^8*c^8*B^2*C - 5*a^2*b^10*c^8*B^2*C + 45*b^12*c^8*B^2*C - 12*a^10*c^10*B^2*C - 28*a^8*b^2*c^10*B^2*C - 3*a^6*b^4*c^10*B^2*C - 13*a^4*b^6*c^10*B^2*C - 3*a^2*b^8*c^10*B^2*C - 53*b^10*c^10*B^2*C + 15*a^8*c^12*B^2*C + 20*a^6*b^2*c^12*B^2*C + 30*a^4*b^4*c^12*B^2*C + 30*a^2*b^6*c^12*B^2*C + 45*b^8*c^12*B^2*C - 11*a^6*c^14*B^2*C - 25*a^4*b^2*c^14*B^2*C - 43*a^2*b^4*c^14*B^2*C - 33*b^6*c^14*B^2*C + 8*a^4*c^16*B^2*C + 26*a^2*b^2*c^16*B^2*C + 22*b^4*c^16*B^2*C - 6*a^2*c^18*B^2*C - 10*b^2*c^18*B^2*C + 2*c^20*B^2*C + 2*a^20*A*C^2 - 6*a^18*b^2*A*C^2 + 8*a^16*b^4*A*C^2 - 11*a^14*b^6*A*C^2 + 15*a^12*b^8*A*C^2 - 12*a^10*b^10*A*C^2 + 6*a^8*b^12*A*C^2 - 3*a^6*b^14*A*C^2 + a^4*b^16*A*C^2 - 10*a^18*c^2*A*C^2 + 26*a^16*b^2*c^2*A*C^2 - 25*a^14*b^4*c^2*A*C^2 + 20*a^12*b^6*c^2*A*C^2 - 28*a^10*b^8*c^2*A*C^2 + 25*a^8*b^10*c^2*A*C^2 - 9*a^6*b^12*c^2*A*C^2 + 3*a^4*b^14*c^2*A*C^2 - 2*a^2*b^16*c^2*A*C^2 + 22*a^16*c^4*A*C^2 - 43*a^14*b^2*c^4*A*C^2 + 30*a^12*b^4*c^4*A*C^2 - 3*a^10*b^6*c^4*A*C^2 - 5*a^8*b^8*c^4*A*C^2 - 10*a^6*b^10*c^4*A*C^2 + 6*a^4*b^12*c^4*A*C^2 + 3*a^2*b^14*c^4*A*C^2 + b^16*c^4*A*C^2 - 33*a^14*c^6*A*C^2 + 30*a^12*b^2*c^6*A*C^2 - 13*a^10*b^4*c^6*A*C^2 + 10*a^8*b^6*c^6*A*C^2 + 30*a^6*b^8*c^6*A*C^2 - 20*a^4*b^10*c^6*A*C^2 - 9*a^2*b^12*c^6*A*C^2 - 3*b^14*c^6*A*C^2 + 45*a^12*c^8*A*C^2 - 3*a^10*b^2*c^8*A*C^2 - 20*a^8*b^4*c^8*A*C^2 - 40*a^6*b^6*c^8*A*C^2 + 10*a^4*b^8*c^8*A*C^2 + 30*a^2*b^10*c^8*A*C^2 + 6*b^12*c^8*A*C^2 - 53*a^10*c^10*A*C^2 - 5*a^8*b^2*c^10*A*C^2 + 40*a^6*b^4*c^10*A*C^2 + 15*a^4*b^6*c^10*A*C^2 - 40*a^2*b^8*c^10*A*C^2 - 3*b^10*c^10*A*C^2 + 45*a^8*c^12*A*C^2 + 3*a^6*b^2*c^12*A*C^2 - 18*a^4*b^4*c^12*A*C^2 + 22*a^2*b^6*c^12*A*C^2 + 18*b^8*c^12*A*C^2 - 27*a^6*c^14*A*C^2 - 8*a^4*b^2*c^14*A*C^2 - 8*a^2*b^4*c^14*A*C^2 - 13*b^6*c^14*A*C^2 + 13*a^4*c^16*A*C^2 + 9*a^2*b^2*c^16*A*C^2 + 6*b^4*c^16*A*C^2 - 5*a^2*c^18*A*C^2 - 3*b^2*c^18*A*C^2 + c^20*A*C^2 + a^16*b^4*B*C^2 - 3*a^14*b^6*B*C^2 + 6*a^12*b^8*B*C^2 - 12*a^10*b^10*B*C^2 + 15*a^8*b^12*B*C^2 - 11*a^6*b^14*B*C^2 + 8*a^4*b^16*B*C^2 - 6*a^2*b^18*B*C^2 + 2*b^20*B*C^2 - 2*a^16*b^2*c^2*B*C^2 + 3*a^14*b^4*c^2*B*C^2 - 9*a^12*b^6*c^2*B*C^2 + 25*a^10*b^8*c^2*B*C^2 - 28*a^8*b^10*c^2*B*C^2 + 20*a^6*b^12*c^2*B*C^2 - 25*a^4*b^14*c^2*B*C^2 + 26*a^2*b^16*c^2*B*C^2 - 10*b^18*c^2*B*C^2 + a^16*c^4*B*C^2 + 3*a^14*b^2*c^4*B*C^2 + 6*a^12*b^4*c^4*B*C^2 - 10*a^10*b^6*c^4*B*C^2 - 5*a^8*b^8*c^4*B*C^2 - 3*a^6*b^10*c^4*B*C^2 + 30*a^4*b^12*c^4*B*C^2 - 43*a^2*b^14*c^4*B*C^2 + 22*b^16*c^4*B*C^2 - 3*a^14*c^6*B*C^2 - 9*a^12*b^2*c^6*B*C^2 - 20*a^10*b^4*c^6*B*C^2 + 30*a^8*b^6*c^6*B*C^2 + 10*a^6*b^8*c^6*B*C^2 - 13*a^4*b^10*c^6*B*C^2 + 30*a^2*b^12*c^6*B*C^2 - 33*b^14*c^6*B*C^2 + 6*a^12*c^8*B*C^2 + 30*a^10*b^2*c^8*B*C^2 + 10*a^8*b^4*c^8*B*C^2 - 40*a^6*b^6*c^8*B*C^2 - 20*a^4*b^8*c^8*B*C^2 - 3*a^2*b^10*c^8*B*C^2 + 45*b^12*c^8*B*C^2 - 13*a^10*c^10*B*C^2 - 40*a^8*b^2*c^10*B*C^2 + 15*a^6*b^4*c^10*B*C^2 + 40*a^4*b^6*c^10*B*C^2 - 5*a^2*b^8*c^10*B*C^2 - 53*b^10*c^10*B*C^2 + 18*a^8*c^12*B*C^2 + 22*a^6*b^2*c^12*B*C^2 - 18*a^4*b^4*c^12*B*C^2 + 3*a^2*b^6*c^12*B*C^2 + 45*b^8*c^12*B*C^2 - 13*a^6*c^14*B*C^2 - 8*a^4*b^2*c^14*B*C^2 - 8*a^2*b^4*c^14*B*C^2 - 27*b^6*c^14*B*C^2 + 6*a^4*c^16*B*C^2 + 9*a^2*b^2*c^16*B*C^2 + 13*b^4*c^16*B*C^2 - 3*a^2*c^18*B*C^2 - 5*b^2*c^18*B*C^2 + c^20*B*C^2 = 0";
				break;
			case 37: 
				equation = 	"a^12*A^2*B - 3*a^10*b^2*A^2*B + 10*a^6*b^6*A^2*B - 15*a^4*b^8*A^2*B + 9*a^2*b^10*A^2*B - 2*b^12*A^2*B - 3*a^10*c^2*A^2*B + 9*a^8*b^2*c^2*A^2*B - 12*a^6*b^4*c^2*A^2*B + 12*a^4*b^6*c^2*A^2*B - 9*a^2*b^8*c^2*A^2*B + 3*b^10*c^2*A^2*B + 3*a^8*c^4*A^2*B - 6*a^6*b^2*c^4*A^2*B + 6*a^4*b^4*c^4*A^2*B - 3*a^2*b^6*c^4*A^2*B - 2*a^6*c^6*A^2*B - 3*a^4*b^2*c^6*A^2*B + 3*a^2*b^4*c^6*A^2*B + b^6*c^6*A^2*B + 3*a^4*c^8*A^2*B + 3*a^2*b^2*c^8*A^2*B - 3*b^4*c^8*A^2*B - 3*a^2*c^10*A^2*B + c^12*A^2*B + 2*a^12*A*B^2 - 9*a^10*b^2*A*B^2 + 15*a^8*b^4*A*B^2 - 10*a^6*b^6*A*B^2 + 3*a^2*b^10*A*B^2 - b^12*A*B^2 - 3*a^10*c^2*A*B^2 + 9*a^8*b^2*c^2*A*B^2 - 12*a^6*b^4*c^2*A*B^2 + 12*a^4*b^6*c^2*A*B^2 - 9*a^2*b^8*c^2*A*B^2 + 3*b^10*c^2*A*B^2 + 3*a^6*b^2*c^4*A*B^2 - 6*a^4*b^4*c^4*A*B^2 + 6*a^2*b^6*c^4*A*B^2 - 3*b^8*c^4*A*B^2 - a^6*c^6*A*B^2 - 3*a^4*b^2*c^6*A*B^2 + 3*a^2*b^4*c^6*A*B^2 + 2*b^6*c^6*A*B^2 + 3*a^4*c^8*A*B^2 - 3*a^2*b^2*c^8*A*B^2 - 3*b^4*c^8*A*B^2 + 3*b^2*c^10*A*B^2 - c^12*A*B^2 - a^12*A^2*C + 3*a^10*b^2*A^2*C - 3*a^8*b^4*A^2*C + 2*a^6*b^6*A^2*C - 3*a^4*b^8*A^2*C + 3*a^2*b^10*A^2*C - b^12*A^2*C + 3*a^10*c^2*A^2*C - 9*a^8*b^2*c^2*A^2*C + 6*a^6*b^4*c^2*A^2*C + 3*a^4*b^6*c^2*A^2*C - 3*a^2*b^8*c^2*A^2*C + 12*a^6*b^2*c^4*A^2*C - 6*a^4*b^4*c^4*A^2*C - 3*a^2*b^6*c^4*A^2*C + 3*b^8*c^4*A^2*C - 10*a^6*c^6*A^2*C - 12*a^4*b^2*c^6*A^2*C + 3*a^2*b^4*c^6*A^2*C - b^6*c^6*A^2*C + 15*a^4*c^8*A^2*C + 9*a^2*b^2*c^8*A^2*C - 9*a^2*c^10*A^2*C - 3*b^2*c^10*A^2*C + 2*c^12*A^2*C + a^12*B^2*C - 3*a^10*b^2*B^2*C + 3*a^8*b^4*B^2*C - 2*a^6*b^6*B^2*C + 3*a^4*b^8*B^2*C - 3*a^2*b^10*B^2*C + b^12*B^2*C + 3*a^8*b^2*c^2*B^2*C - 3*a^6*b^4*c^2*B^2*C - 6*a^4*b^6*c^2*B^2*C + 9*a^2*b^8*c^2*B^2*C - 3*b^10*c^2*B^2*C - 3*a^8*c^4*B^2*C + 3*a^6*b^2*c^4*B^2*C + 6*a^4*b^4*c^4*B^2*C - 12*a^2*b^6*c^4*B^2*C + a^6*c^6*B^2*C - 3*a^4*b^2*c^6*B^2*C + 12*a^2*b^4*c^6*B^2*C + 10*b^6*c^6*B^2*C - 9*a^2*b^2*c^8*B^2*C - 15*b^4*c^8*B^2*C + 3*a^2*c^10*B^2*C + 9*b^2*c^10*B^2*C - 2*c^12*B^2*C - 2*a^12*A*C^2 + 3*a^10*b^2*A*C^2 + a^6*b^6*A*C^2 - 3*a^4*b^8*A*C^2 + b^12*A*C^2 + 9*a^10*c^2*A*C^2 - 9*a^8*b^2*c^2*A*C^2 - 3*a^6*b^4*c^2*A*C^2 + 3*a^4*b^6*c^2*A*C^2 + 3*a^2*b^8*c^2*A*C^2 - 3*b^10*c^2*A*C^2 - 15*a^8*c^4*A*C^2 + 12*a^6*b^2*c^4*A*C^2 + 6*a^4*b^4*c^4*A*C^2 - 3*a^2*b^6*c^4*A*C^2 + 3*b^8*c^4*A*C^2 + 10*a^6*c^6*A*C^2 - 12*a^4*b^2*c^6*A*C^2 - 6*a^2*b^4*c^6*A*C^2 - 2*b^6*c^6*A*C^2 + 9*a^2*b^2*c^8*A*C^2 + 3*b^4*c^8*A*C^2 - 3*a^2*c^10*A*C^2 - 3*b^2*c^10*A*C^2 + c^12*A*C^2 - a^12*B*C^2 + 3*a^8*b^4*B*C^2 - a^6*b^6*B*C^2 - 3*a^2*b^10*B*C^2 + 2*b^12*B*C^2 + 3*a^10*c^2*B*C^2 - 3*a^8*b^2*c^2*B*C^2 - 3*a^6*b^4*c^2*B*C^2 + 3*a^4*b^6*c^2*B*C^2 + 9*a^2*b^8*c^2*B*C^2 - 9*b^10*c^2*B*C^2 - 3*a^8*c^4*B*C^2 + 3*a^6*b^2*c^4*B*C^2 - 6*a^4*b^4*c^4*B*C^2 - 12*a^2*b^6*c^4*B*C^2 + 15*b^8*c^4*B*C^2 + 2*a^6*c^6*B*C^2 + 6*a^4*b^2*c^6*B*C^2 + 12*a^2*b^4*c^6*B*C^2 - 10*b^6*c^6*B*C^2 - 3*a^4*c^8*B*C^2 - 9*a^2*b^2*c^8*B*C^2 + 3*a^2*c^10*B*C^2 + 3*b^2*c^10*B*C^2 - c^12*B*C^2 = 0";
				break;
			case 38: 
				equation = 	"b^4*c^2*A^3 - b^2*c^4*A^3 + 2*a^2*b^2*c^2*A^2*B - b^4*c^2*A^2*B - 2*a^2*c^4*A^2*B - b^2*c^4*A^2*B + 2*c^6*A^2*B + a^4*c^2*A*B^2 - 2*a^2*b^2*c^2*A*B^2 + a^2*c^4*A*B^2 + 2*b^2*c^4*A*B^2 - 2*c^6*A*B^2 - a^4*c^2*B^3 + a^2*c^4*B^3 + 2*a^2*b^4*A^2*C - 2*b^6*A^2*C - 2*a^2*b^2*c^2*A^2*C + b^4*c^2*A^2*C + b^2*c^4*A^2*C + 4*a^4*b^2*A*B*C - 4*a^2*b^4*A*B*C - 4*a^4*c^2*A*B*C + 4*b^4*c^2*A*B*C + 4*a^2*c^4*A*B*C - 4*b^2*c^4*A*B*C + 2*a^6*B^2*C - 2*a^4*b^2*B^2*C - a^4*c^2*B^2*C + 2*a^2*b^2*c^2*B^2*C - a^2*c^4*B^2*C - a^4*b^2*A*C^2 - a^2*b^4*A*C^2 + 2*b^6*A*C^2 + 2*a^2*b^2*c^2*A*C^2 - 2*b^4*c^2*A*C^2 - 2*a^6*B*C^2 + a^4*b^2*B*C^2 + a^2*b^4*B*C^2 + 2*a^4*c^2*B*C^2 - 2*a^2*b^2*c^2*B*C^2 + a^4*b^2*C^3 - a^2*b^4*C^3 = 0";
				break;
			case 39: 
				equation = 	"-a^6*b^2*c^4*A^2*B - a^4*b^4*c^4*A^2*B + a^2*b^6*c^4*A^2*B + b^8*c^4*A^2*B + 2*a^4*b^2*c^6*A^2*B - a^2*b^4*c^6*A^2*B - 3*b^6*c^6*A^2*B + 3*b^4*c^8*A^2*B - b^2*c^10*A^2*B - a^8*c^4*A*B^2 - a^6*b^2*c^4*A*B^2 + a^4*b^4*c^4*A*B^2 + a^2*b^6*c^4*A*B^2 + 3*a^6*c^6*A*B^2 + a^4*b^2*c^6*A*B^2 - 2*a^2*b^4*c^6*A*B^2 - 3*a^4*c^8*A*B^2 + a^2*c^10*A*B^2 + a^6*b^4*c^2*A^2*C - 2*a^4*b^6*c^2*A^2*C + b^10*c^2*A^2*C + a^4*b^4*c^4*A^2*C + a^2*b^6*c^4*A^2*C - 3*b^8*c^4*A^2*C - a^2*b^4*c^6*A^2*C + 3*b^6*c^6*A^2*C - b^4*c^8*A^2*C - 2*a^6*b^4*c^2*A*B*C + 2*a^4*b^6*c^2*A*B*C + 2*a^6*b^2*c^4*A*B*C - 2*a^2*b^6*c^4*A*B*C - 2*a^4*b^2*c^6*A*B*C + 2*a^2*b^4*c^6*A*B*C - a^10*c^2*B^2*C + 2*a^6*b^4*c^2*B^2*C - a^4*b^6*c^2*B^2*C + 3*a^8*c^4*B^2*C - a^6*b^2*c^4*B^2*C - a^4*b^4*c^4*B^2*C - 3*a^6*c^6*B^2*C + a^4*b^2*c^6*B^2*C + a^4*c^8*B^2*C + a^8*b^4*A*C^2 - 3*a^6*b^6*A*C^2 + 3*a^4*b^8*A*C^2 - a^2*b^10*A*C^2 + a^6*b^4*c^2*A*C^2 - a^4*b^6*c^2*A*C^2 - a^4*b^4*c^4*A*C^2 + 2*a^2*b^6*c^4*A*C^2 - a^2*b^4*c^6*A*C^2 + a^10*b^2*B*C^2 - 3*a^8*b^4*B*C^2 + 3*a^6*b^6*B*C^2 - a^4*b^8*B*C^2 + a^6*b^4*c^2*B*C^2 - a^4*b^6*c^2*B*C^2 - 2*a^6*b^2*c^4*B*C^2 + a^4*b^4*c^4*B*C^2 + a^4*b^2*c^6*B*C^2 = 0";
				break;
			case 40: 
				equation = 	"a^2*b*c^2*A^2*B - a*b^2*c^2*A^2*B + b^2*c^3*A^2*B - b*c^4*A^2*B + a^2*b*c^2*A*B^2 - a*b^2*c^2*A*B^2 - a^2*c^3*A*B^2 + a*c^4*A*B^2 - a^2*b^2*c*A^2*C + b^4*c*A^2*C + a*b^2*c^2*A^2*C - b^3*c^2*A^2*C - a^4*c*B^2*C + a^2*b^2*c*B^2*C + a^3*c^2*B^2*C - a^2*b*c^2*B^2*C + a^2*b^3*A*C^2 - a*b^4*A*C^2 - a^2*b^2*c*A*C^2 + a*b^2*c^2*A*C^2 + a^4*b*B*C^2 - a^3*b^2*B*C^2 + a^2*b^2*c*B*C^2 - a^2*b*c^2*B*C^2 = 0";
				break;
			case 41: 
				equation = 	"a^6*A^2*B - 3*a^4*b^2*A^2*B + 3*a^2*b^4*A^2*B - b^6*A^2*B + 3*a^4*c^2*A^2*B - 2*a^2*b^2*c^2*A^2*B - b^4*c^2*A^2*B - a^2*c^4*A^2*B + 5*b^2*c^4*A^2*B - 3*c^6*A^2*B + a^6*A*B^2 - 3*a^4*b^2*A*B^2 + 3*a^2*b^4*A*B^2 - b^6*A*B^2 + a^4*c^2*A*B^2 + 2*a^2*b^2*c^2*A*B^2 - 3*b^4*c^2*A*B^2 - 5*a^2*c^4*A*B^2 + b^2*c^4*A*B^2 + 3*c^6*A*B^2 - a^6*A^2*C - 3*a^4*b^2*A^2*C + a^2*b^4*A^2*C + 3*b^6*A^2*C + 3*a^4*c^2*A^2*C + 2*a^2*b^2*c^2*A^2*C - 5*b^4*c^2*A^2*C - 3*a^2*c^4*A^2*C + b^2*c^4*A^2*C + c^6*A^2*C - 16*a^4*b^2*A*B*C + 16*a^2*b^4*A*B*C + 16*a^4*c^2*A*B*C - 16*b^4*c^2*A*B*C - 16*a^2*c^4*A*B*C + 16*b^2*c^4*A*B*C - 3*a^6*B^2*C - a^4*b^2*B^2*C + 3*a^2*b^4*B^2*C + b^6*B^2*C + 5*a^4*c^2*B^2*C - 2*a^2*b^2*c^2*B^2*C - 3*b^4*c^2*B^2*C - a^2*c^4*B^2*C + 3*b^2*c^4*B^2*C - c^6*B^2*C - a^6*A*C^2 - a^4*b^2*A*C^2 + 5*a^2*b^4*A*C^2 - 3*b^6*A*C^2 + 3*a^4*c^2*A*C^2 - 2*a^2*b^2*c^2*A*C^2 - b^4*c^2*A*C^2 - 3*a^2*c^4*A*C^2 + 3*b^2*c^4*A*C^2 + c^6*A*C^2 + 3*a^6*B*C^2 - 5*a^4*b^2*B*C^2 + a^2*b^4*B*C^2 + b^6*B*C^2 + a^4*c^2*B*C^2 + 2*a^2*b^2*c^2*B*C^2 - 3*b^4*c^2*B*C^2 - 3*a^2*c^4*B*C^2 + 3*b^2*c^4*B*C^2 - c^6*B*C^2 = 0";
				break;
			case 42: 
				equation = 	"-a^10*c^2*A^2*B + a^8*b^2*c^2*A^2*B - 4*a^4*b^6*c^2*A^2*B + a^2*b^8*c^2*A^2*B + 3*b^10*c^2*A^2*B + 2*a^8*c^4*A^2*B - 5*a^6*b^2*c^4*A^2*B + 6*a^4*b^4*c^4*A^2*B + 5*a^2*b^6*c^4*A^2*B - 8*b^8*c^4*A^2*B + 2*a^6*c^6*A^2*B + 3*a^4*b^2*c^6*A^2*B - 9*a^2*b^4*c^6*A^2*B + 2*b^6*c^6*A^2*B - 4*a^4*c^8*A^2*B + 4*a^2*b^2*c^8*A^2*B + 6*b^4*c^8*A^2*B - a^2*c^10*A^2*B - 5*b^2*c^10*A^2*B + 2*c^12*A^2*B - 3*a^10*c^2*A*B^2 - a^8*b^2*c^2*A*B^2 + 4*a^6*b^4*c^2*A*B^2 - a^2*b^8*c^2*A*B^2 + b^10*c^2*A*B^2 + 8*a^8*c^4*A*B^2 - 5*a^6*b^2*c^4*A*B^2 - 6*a^4*b^4*c^4*A*B^2 + 5*a^2*b^6*c^4*A*B^2 - 2*b^8*c^4*A*B^2 - 2*a^6*c^6*A*B^2 + 9*a^4*b^2*c^6*A*B^2 - 3*a^2*b^4*c^6*A*B^2 - 2*b^6*c^6*A*B^2 - 6*a^4*c^8*A*B^2 - 4*a^2*b^2*c^8*A*B^2 + 4*b^4*c^8*A*B^2 + 5*a^2*c^10*A*B^2 + b^2*c^10*A*B^2 - 2*c^12*A*B^2 + a^10*b^2*A^2*C - 2*a^8*b^4*A^2*C - 2*a^6*b^6*A^2*C + 4*a^4*b^8*A^2*C + a^2*b^10*A^2*C - 2*b^12*A^2*C - a^8*b^2*c^2*A^2*C + 5*a^6*b^4*c^2*A^2*C - 3*a^4*b^6*c^2*A^2*C - 4*a^2*b^8*c^2*A^2*C + 5*b^10*c^2*A^2*C - 6*a^4*b^4*c^4*A^2*C + 9*a^2*b^6*c^4*A^2*C - 6*b^8*c^4*A^2*C + 4*a^4*b^2*c^6*A^2*C - 5*a^2*b^4*c^6*A^2*C - 2*b^6*c^6*A^2*C - a^2*b^2*c^8*A^2*C + 8*b^4*c^8*A^2*C - 3*b^2*c^10*A^2*C + 2*a^12*B^2*C - a^10*b^2*B^2*C - 4*a^8*b^4*B^2*C + 2*a^6*b^6*B^2*C + 2*a^4*b^8*B^2*C - a^2*b^10*B^2*C - 5*a^10*c^2*B^2*C + 4*a^8*b^2*c^2*B^2*C + 3*a^6*b^4*c^2*B^2*C - 5*a^4*b^6*c^2*B^2*C + a^2*b^8*c^2*B^2*C + 6*a^8*c^4*B^2*C - 9*a^6*b^2*c^4*B^2*C + 6*a^4*b^4*c^4*B^2*C + 2*a^6*c^6*B^2*C + 5*a^4*b^2*c^6*B^2*C - 4*a^2*b^4*c^6*B^2*C - 8*a^4*c^8*B^2*C + a^2*b^2*c^8*B^2*C + 3*a^2*c^10*B^2*C + 3*a^10*b^2*A*C^2 - 8*a^8*b^4*A*C^2 + 2*a^6*b^6*A*C^2 + 6*a^4*b^8*A*C^2 - 5*a^2*b^10*A*C^2 + 2*b^12*A*C^2 + a^8*b^2*c^2*A*C^2 + 5*a^6*b^4*c^2*A*C^2 - 9*a^4*b^6*c^2*A*C^2 + 4*a^2*b^8*c^2*A*C^2 - b^10*c^2*A*C^2 - 4*a^6*b^2*c^4*A*C^2 + 6*a^4*b^4*c^4*A*C^2 + 3*a^2*b^6*c^4*A*C^2 - 4*b^8*c^4*A*C^2 - 5*a^2*b^4*c^6*A*C^2 + 2*b^6*c^6*A*C^2 + a^2*b^2*c^8*A*C^2 + 2*b^4*c^8*A*C^2 - b^2*c^10*A*C^2 - 2*a^12*B*C^2 + 5*a^10*b^2*B*C^2 - 6*a^8*b^4*B*C^2 - 2*a^6*b^6*B*C^2 + 8*a^4*b^8*B*C^2 - 3*a^2*b^10*B*C^2 + a^10*c^2*B*C^2 - 4*a^8*b^2*c^2*B*C^2 + 9*a^6*b^4*c^2*B*C^2 - 5*a^4*b^6*c^2*B*C^2 - a^2*b^8*c^2*B*C^2 + 4*a^8*c^4*B*C^2 - 3*a^6*b^2*c^4*B*C^2 - 6*a^4*b^4*c^4*B*C^2 + 4*a^2*b^6*c^4*B*C^2 - 2*a^6*c^6*B*C^2 + 5*a^4*b^2*c^6*B*C^2 - 2*a^4*c^8*B*C^2 - a^2*b^2*c^8*B*C^2 + a^2*c^10*B*C^2 = 0";
				break;
			case 43: 
				equation = 	"a^2*c^2*A^2*B + b^2*c^2*A^2*B - 2*c^4*A^2*B - a^2*c^2*A*B^2 - b^2*c^2*A*B^2 + 2*c^4*A*B^2 - a^2*b^2*A^2*C + 2*b^4*A^2*C - b^2*c^2*A^2*C - 2*a^4*B^2*C + a^2*b^2*B^2*C + a^2*c^2*B^2*C + a^2*b^2*A*C^2 - 2*b^4*A*C^2 + b^2*c^2*A*C^2 + 2*a^4*B*C^2 - a^2*b^2*B*C^2 - a^2*c^2*B*C^2 = 0";
				break;
			case 44: 
				equation = 	"a^10*c^2*A^2*B - a^8*b^2*c^2*A^2*B - 2*a^6*b^4*c^2*A^2*B + 2*a^4*b^6*c^2*A^2*B + a^2*b^8*c^2*A^2*B - b^10*c^2*A^2*B - 4*a^8*c^4*A^2*B + 2*a^6*b^2*c^4*A^2*B + 2*a^4*b^4*c^4*A^2*B - 2*a^2*b^6*c^4*A^2*B + 2*b^8*c^4*A^2*B + 6*a^6*c^6*A^2*B + 2*a^2*b^4*c^6*A^2*B - 4*a^4*c^8*A^2*B - 2*a^2*b^2*c^8*A^2*B - 2*b^4*c^8*A^2*B + a^2*c^10*A^2*B + b^2*c^10*A^2*B + a^10*c^2*A*B^2 - a^8*b^2*c^2*A*B^2 - 2*a^6*b^4*c^2*A*B^2 + 2*a^4*b^6*c^2*A*B^2 + a^2*b^8*c^2*A*B^2 - b^10*c^2*A*B^2 - 2*a^8*c^4*A*B^2 + 2*a^6*b^2*c^4*A*B^2 - 2*a^4*b^4*c^4*A*B^2 - 2*a^2*b^6*c^4*A*B^2 + 4*b^8*c^4*A*B^2 - 2*a^4*b^2*c^6*A*B^2 - 6*b^6*c^6*A*B^2 + 2*a^4*c^8*A*B^2 + 2*a^2*b^2*c^8*A*B^2 + 4*b^4*c^8*A*B^2 - a^2*c^10*A*B^2 - b^2*c^10*A*B^2 - a^10*b^2*A^2*C + 4*a^8*b^4*A^2*C - 6*a^6*b^6*A^2*C + 4*a^4*b^8*A^2*C - a^2*b^10*A^2*C + a^8*b^2*c^2*A^2*C - 2*a^6*b^4*c^2*A^2*C + 2*a^2*b^8*c^2*A^2*C - b^10*c^2*A^2*C + 2*a^6*b^2*c^4*A^2*C - 2*a^4*b^4*c^4*A^2*C - 2*a^2*b^6*c^4*A^2*C + 2*b^8*c^4*A^2*C - 2*a^4*b^2*c^6*A^2*C + 2*a^2*b^4*c^6*A^2*C - a^2*b^2*c^8*A^2*C - 2*b^4*c^8*A^2*C + b^2*c^10*A^2*C + a^10*b^2*B^2*C - 4*a^8*b^4*B^2*C + 6*a^6*b^6*B^2*C - 4*a^4*b^8*B^2*C + a^2*b^10*B^2*C + a^10*c^2*B^2*C - 2*a^8*b^2*c^2*B^2*C + 2*a^4*b^6*c^2*B^2*C - a^2*b^8*c^2*B^2*C - 2*a^8*c^4*B^2*C + 2*a^6*b^2*c^4*B^2*C + 2*a^4*b^4*c^4*B^2*C - 2*a^2*b^6*c^4*B^2*C - 2*a^4*b^2*c^6*B^2*C + 2*a^2*b^4*c^6*B^2*C + 2*a^4*c^8*B^2*C + a^2*b^2*c^8*B^2*C - a^2*c^10*B^2*C - a^10*b^2*A*C^2 + 2*a^8*b^4*A*C^2 - 2*a^4*b^8*A*C^2 + a^2*b^10*A*C^2 + a^8*b^2*c^2*A*C^2 - 2*a^6*b^4*c^2*A*C^2 + 2*a^4*b^6*c^2*A*C^2 - 2*a^2*b^8*c^2*A*C^2 + b^10*c^2*A*C^2 + 2*a^6*b^2*c^4*A*C^2 + 2*a^4*b^4*c^4*A*C^2 - 4*b^8*c^4*A*C^2 - 2*a^4*b^2*c^6*A*C^2 + 2*a^2*b^4*c^6*A*C^2 + 6*b^6*c^6*A*C^2 - a^2*b^2*c^8*A*C^2 - 4*b^4*c^8*A*C^2 + b^2*c^10*A*C^2 - a^10*b^2*B*C^2 + 2*a^8*b^4*B*C^2 - 2*a^4*b^8*B*C^2 + a^2*b^10*B*C^2 - a^10*c^2*B*C^2 + 2*a^8*b^2*c^2*B*C^2 - 2*a^6*b^4*c^2*B*C^2 + 2*a^4*b^6*c^2*B*C^2 - a^2*b^8*c^2*B*C^2 + 4*a^8*c^4*B*C^2 - 2*a^4*b^4*c^4*B*C^2 - 2*a^2*b^6*c^4*B*C^2 - 6*a^6*c^6*B*C^2 - 2*a^4*b^2*c^6*B*C^2 + 2*a^2*b^4*c^6*B*C^2 + 4*a^4*c^8*B*C^2 + a^2*b^2*c^8*B*C^2 - a^2*c^10*B*C^2 = 0";
				break;
			case 45: 
				equation = 	"a^6*c^2*A^2*B - a^2*b^4*c^2*A^2*B - 2*a^4*c^4*A^2*B + a^2*c^6*A^2*B + a^4*b^2*c^2*A*B^2 - b^6*c^2*A*B^2 + 2*b^4*c^4*A*B^2 - b^2*c^6*A*B^2 - a^6*b^2*A^2*C + 2*a^4*b^4*A^2*C - a^2*b^6*A^2*C + a^2*b^2*c^4*A^2*C + a^6*b^2*B^2*C - 2*a^4*b^4*B^2*C + a^2*b^6*B^2*C - a^2*b^2*c^4*B^2*C - a^4*b^2*c^2*A*C^2 + b^6*c^2*A*C^2 - 2*b^4*c^4*A*C^2 + b^2*c^6*A*C^2 - a^6*c^2*B*C^2 + a^2*b^4*c^2*B*C^2 + 2*a^4*c^4*B*C^2 - a^2*c^6*B*C^2 = 0";
				break;
			case 46: 
				equation = 	"Sqrt((a + b - c)*(a - b + c)*(-a + b + c)*(a + b + c))*((-a^4)*A^2*B - 4*a^2*b^2*A^2*B + 5*b^4*A^2*B - a^2*c^2*A^2*B - b^2*c^2*A^2*B - 4*c^4*A^2*B - 5*a^4*A*B^2 + 4*a^2*b^2*A*B^2 + b^4*A*B^2 + a^2*c^2*A*B^2 + b^2*c^2*A*B^2 + 4*c^4*A*B^2 + a^4*A^2*C + a^2*b^2*A^2*C + 4*b^4*A^2*C + 4*a^2*c^2*A^2*C + b^2*c^2*A^2*C - 5*c^4*A^2*C - 4*a^4*B^2*C - a^2*b^2*B^2*C - b^4*B^2*C - a^2*c^2*B^2*C - 4*b^2*c^2*B^2*C + 5*c^4*B^2*C + 5*a^4*A*C^2 - a^2*b^2*A*C^2 - 4*b^4*A*C^2 - 4*a^2*c^2*A*C^2 - b^2*c^2*A*C^2 - c^4*A*C^2 + 4*a^4*B*C^2 + a^2*b^2*B*C^2 - 5*b^4*B*C^2 + a^2*c^2*B*C^2 + 4*b^2*c^2*B*C^2 + c^4*B*C^2) + Sqrt(3)*(a^6*A^2*B - 3*a^4*b^2*A^2*B + 3*a^2*b^4*A^2*B - b^6*A^2*B - 2*a^4*c^2*A^2*B - 6*a^2*b^2*c^2*A^2*B + 8*b^4*c^2*A^2*B + a^2*c^4*A^2*B - 7*b^2*c^4*A^2*B + a^6*A*B^2 - 3*a^4*b^2*A*B^2 + 3*a^2*b^4*A*B^2 - b^6*A*B^2 - 8*a^4*c^2*A*B^2 + 6*a^2*b^2*c^2*A*B^2 + 2*b^4*c^2*A*B^2 + 7*a^2*c^4*A*B^2 - b^2*c^4*A*B^2 - a^6*A^2*C + 2*a^4*b^2*A^2*C - a^2*b^4*A^2*C + 3*a^4*c^2*A^2*C + 6*a^2*b^2*c^2*A^2*C + 7*b^4*c^2*A^2*C - 3*a^2*c^4*A^2*C - 8*b^2*c^4*A^2*C + c^6*A^2*C + a^4*b^2*B^2*C - 2*a^2*b^4*B^2*C + b^6*B^2*C - 7*a^4*c^2*B^2*C - 6*a^2*b^2*c^2*B^2*C - 3*b^4*c^2*B^2*C + 8*a^2*c^4*B^2*C + 3*b^2*c^4*B^2*C - c^6*B^2*C - a^6*A*C^2 + 8*a^4*b^2*A*C^2 - 7*a^2*b^4*A*C^2 + 3*a^4*c^2*A*C^2 - 6*a^2*b^2*c^2*A*C^2 + b^4*c^2*A*C^2 - 3*a^2*c^4*A*C^2 - 2*b^2*c^4*A*C^2 + c^6*A*C^2 + 7*a^4*b^2*B*C^2 - 8*a^2*b^4*B*C^2 + b^6*B*C^2 - a^4*c^2*B*C^2 + 6*a^2*b^2*c^2*B*C^2 - 3*b^4*c^2*B*C^2 + 2*a^2*c^4*B*C^2 + 3*b^2*c^4*B*C^2 - c^6*B*C^2) = 0";
				break;
			case -46: 
				equation = 	"Sqrt((a + b - c)*(a - b + c)*(-a + b + c)*(a + b + c))*(a^4*A^2*B + 4*a^2*b^2*A^2*B - 5*b^4*A^2*B + a^2*c^2*A^2*B + b^2*c^2*A^2*B + 4*c^4*A^2*B + 5*a^4*A*B^2 - 4*a^2*b^2*A*B^2 - b^4*A*B^2 - a^2*c^2*A*B^2 - b^2*c^2*A*B^2 - 4*c^4*A*B^2 - a^4*A^2*C - a^2*b^2*A^2*C - 4*b^4*A^2*C - 4*a^2*c^2*A^2*C - b^2*c^2*A^2*C + 5*c^4*A^2*C + 4*a^4*B^2*C + a^2*b^2*B^2*C + b^4*B^2*C + a^2*c^2*B^2*C + 4*b^2*c^2*B^2*C - 5*c^4*B^2*C - 5*a^4*A*C^2 + a^2*b^2*A*C^2 + 4*b^4*A*C^2 + 4*a^2*c^2*A*C^2 + b^2*c^2*A*C^2 + c^4*A*C^2 - 4*a^4*B*C^2 - a^2*b^2*B*C^2 + 5*b^4*B*C^2 - a^2*c^2*B*C^2 - 4*b^2*c^2*B*C^2 - c^4*B*C^2) + Sqrt(3)*(a^6*A^2*B - 3*a^4*b^2*A^2*B + 3*a^2*b^4*A^2*B - b^6*A^2*B - 2*a^4*c^2*A^2*B - 6*a^2*b^2*c^2*A^2*B + 8*b^4*c^2*A^2*B + a^2*c^4*A^2*B - 7*b^2*c^4*A^2*B + a^6*A*B^2 - 3*a^4*b^2*A*B^2 + 3*a^2*b^4*A*B^2 - b^6*A*B^2 - 8*a^4*c^2*A*B^2 + 6*a^2*b^2*c^2*A*B^2 + 2*b^4*c^2*A*B^2 + 7*a^2*c^4*A*B^2 - b^2*c^4*A*B^2 - a^6*A^2*C + 2*a^4*b^2*A^2*C - a^2*b^4*A^2*C + 3*a^4*c^2*A^2*C + 6*a^2*b^2*c^2*A^2*C + 7*b^4*c^2*A^2*C - 3*a^2*c^4*A^2*C - 8*b^2*c^4*A^2*C + c^6*A^2*C + a^4*b^2*B^2*C - 2*a^2*b^4*B^2*C + b^6*B^2*C - 7*a^4*c^2*B^2*C - 6*a^2*b^2*c^2*B^2*C - 3*b^4*c^2*B^2*C + 8*a^2*c^4*B^2*C + 3*b^2*c^4*B^2*C - c^6*B^2*C - a^6*A*C^2 + 8*a^4*b^2*A*C^2 - 7*a^2*b^4*A*C^2 + 3*a^4*c^2*A*C^2 - 6*a^2*b^2*c^2*A*C^2 + b^4*c^2*A*C^2 - 3*a^2*c^4*A*C^2 - 2*b^2*c^4*A*C^2 + c^6*A*C^2 + 7*a^4*b^2*B*C^2 - 8*a^2*b^4*B*C^2 + b^6*B*C^2 - a^4*c^2*B*C^2 + 6*a^2*b^2*c^2*B*C^2 - 3*b^4*c^2*B*C^2 + 2*a^2*c^4*B*C^2 + 3*b^2*c^4*B*C^2 - c^6*B*C^2) = 0";
				break;
			case 47: 
				equation = 	"-3*a^4*c^2*A^2*B - 2*a^2*b^2*c^2*A^2*B + 5*b^4*c^2*A^2*B + 2*a^2*c^4*A^2*B - 6*b^2*c^4*A^2*B + c^6*A^2*B - 5*a^4*c^2*A*B^2 + 2*a^2*b^2*c^2*A*B^2 + 3*b^4*c^2*A*B^2 + 6*a^2*c^4*A*B^2 - 2*b^2*c^4*A*B^2 - c^6*A*B^2 + 3*a^4*b^2*A^2*C - 2*a^2*b^4*A^2*C - b^6*A^2*C + 2*a^2*b^2*c^2*A^2*C + 6*b^4*c^2*A^2*C - 5*b^2*c^4*A^2*C + 8*a^4*b^2*A*B*C - 8*a^2*b^4*A*B*C - 8*a^4*c^2*A*B*C + 8*b^4*c^2*A*B*C + 8*a^2*c^4*A*B*C - 8*b^2*c^4*A*B*C + a^6*B^2*C + 2*a^4*b^2*B^2*C - 3*a^2*b^4*B^2*C - 6*a^4*c^2*B^2*C - 2*a^2*b^2*c^2*B^2*C + 5*a^2*c^4*B^2*C + 5*a^4*b^2*A*C^2 - 6*a^2*b^4*A*C^2 + b^6*A*C^2 - 2*a^2*b^2*c^2*A*C^2 + 2*b^4*c^2*A*C^2 - 3*b^2*c^4*A*C^2 - a^6*B*C^2 + 6*a^4*b^2*B*C^2 - 5*a^2*b^4*B*C^2 - 2*a^4*c^2*B*C^2 + 2*a^2*b^2*c^2*B*C^2 + 3*a^2*c^4*B*C^2 = 0";
				break;
			case 48: 
				equation = 	"-a^2*b^4*c^2*A^3 + b^6*c^2*A^3 + a^2*b^2*c^4*A^3 - b^2*c^6*A^3 - a^4*b^2*c^2*A^2*B + b^6*c^2*A^2*B + a^2*b^2*c^4*A^2*B - 3*b^4*c^4*A^2*B + 2*b^2*c^6*A^2*B - a^6*c^2*A*B^2 + a^2*b^4*c^2*A*B^2 + 3*a^4*c^4*A*B^2 - a^2*b^2*c^4*A*B^2 - 2*a^2*c^6*A*B^2 - a^6*c^2*B^3 + a^4*b^2*c^2*B^3 - a^2*b^2*c^4*B^3 + a^2*c^6*B^3 + a^4*b^2*c^2*A^2*C - a^2*b^4*c^2*A^2*C - 2*b^6*c^2*A^2*C + 3*b^4*c^4*A^2*C - b^2*c^6*A^2*C + 2*a^6*c^2*B^2*C + a^4*b^2*c^2*B^2*C - a^2*b^4*c^2*B^2*C - 3*a^4*c^4*B^2*C + a^2*c^6*B^2*C + a^6*b^2*A*C^2 - 3*a^4*b^4*A*C^2 + 2*a^2*b^6*A*C^2 + a^2*b^4*c^2*A*C^2 - a^2*b^2*c^4*A*C^2 - 2*a^6*b^2*B*C^2 + 3*a^4*b^4*B*C^2 - a^2*b^6*B*C^2 - a^4*b^2*c^2*B*C^2 + a^2*b^2*c^4*B*C^2 + a^6*b^2*C^3 - a^2*b^6*C^3 - a^4*b^2*c^2*C^3 + a^2*b^4*c^2*C^3 = 0";
				break;
			case 49: 
				equation = 	"a^6*A^2*B - 3*a^4*b^2*A^2*B + 3*a^2*b^4*A^2*B - b^6*A^2*B - 2*a^4*c^2*A^2*B + 2*a^2*b^2*c^2*A^2*B + a^2*c^4*A^2*B + b^2*c^4*A^2*B + a^6*A*B^2 - 3*a^4*b^2*A*B^2 + 3*a^2*b^4*A*B^2 - b^6*A*B^2 - 2*a^2*b^2*c^2*A*B^2 + 2*b^4*c^2*A*B^2 - a^2*c^4*A*B^2 - b^2*c^4*A*B^2 - a^6*A^2*C + 2*a^4*b^2*A^2*C - a^2*b^4*A^2*C + 3*a^4*c^2*A^2*C - 2*a^2*b^2*c^2*A^2*C - b^4*c^2*A^2*C - 3*a^2*c^4*A^2*C + c^6*A^2*C + a^4*b^2*B^2*C - 2*a^2*b^4*B^2*C + b^6*B^2*C + a^4*c^2*B^2*C + 2*a^2*b^2*c^2*B^2*C - 3*b^4*c^2*B^2*C + 3*b^2*c^4*B^2*C - c^6*B^2*C - a^6*A*C^2 + a^2*b^4*A*C^2 + 3*a^4*c^2*A*C^2 + 2*a^2*b^2*c^2*A*C^2 + b^4*c^2*A*C^2 - 3*a^2*c^4*A*C^2 - 2*b^2*c^4*A*C^2 + c^6*A*C^2 - a^4*b^2*B*C^2 + b^6*B*C^2 - a^4*c^2*B*C^2 - 2*a^2*b^2*c^2*B*C^2 - 3*b^4*c^2*B*C^2 + 2*a^2*c^4*B*C^2 + 3*b^2*c^4*B*C^2 - c^6*B*C^2 = 0";
				break;
			
			case 561:
				equation = "a^3*c^3*A^2*B - a^2*b*c^3*A^2*B + a*b^2*c^3*A^2*B - b^3*c^3*A^2*B - a^2*c^4*A^2*B - b^2*c^4*A^2*B - a*c^5*A^2*B + b*c^5*A^2*B + c^6*A^2*B + a^3*c^3*A*B^2 - a^2*b*c^3*A*B^2 + a*b^2*c^3*A*B^2 - b^3*c^3*A*B^2 + a^2*c^4*A*B^2 + b^2*c^4*A*B^2 - a*c^5*A*B^2 + b*c^5*A*B^2 - c^6*A*B^2 - a^3*b^3*A^2*C + a^2*b^4*A^2*C + a*b^5*A^2*C - b^6*A^2*C + a^2*b^3*c*A^2*C - b^5*c*A^2*C - a*b^3*c^2*A^2*C + b^4*c^2*A^2*C + b^3*c^3*A^2*C - a^5*b*A*B*C + 2*a^4*b^2*A*B*C - 2*a^2*b^4*A*B*C + a*b^5*A*B*C + a^5*c*A*B*C - a^3*b^2*c*A*B*C + a^2*b^3*c*A*B*C - b^5*c*A*B*C - 2*a^4*c^2*A*B*C + a^3*b*c^2*A*B*C - a*b^3*c^2*A*B*C + 2*b^4*c^2*A*B*C - a^2*b*c^3*A*B*C + a*b^2*c^3*A*B*C + 2*a^2*c^4*A*B*C - 2*b^2*c^4*A*B*C - a*c^5*A*B*C + b*c^5*A*B*C + a^6*B^2*C - a^5*b*B^2*C - a^4*b^2*B^2*C + a^3*b^3*B^2*C + a^5*c*B^2*C - a^3*b^2*c*B^2*C - a^4*c^2*B^2*C + a^3*b*c^2*B^2*C - a^3*c^3*B^2*C - a^3*b^3*A*C^2 - a^2*b^4*A*C^2 + a*b^5*A*C^2 + b^6*A*C^2 + a^2*b^3*c*A*C^2 - b^5*c*A*C^2 - a*b^3*c^2*A*C^2 - b^4*c^2*A*C^2 + b^3*c^3*A*C^2 - a^6*B*C^2 - a^5*b*B*C^2 + a^4*b^2*B*C^2 + a^3*b^3*B*C^2 + a^5*c*B*C^2 - a^3*b^2*c*B*C^2 + a^4*c^2*B*C^2 + a^3*b*c^2*B*C^2 - a^3*c^3*B*C^2 = 0";
				break;
			case 562:
				equation = "-a^10*c^2*A^2*B + 2*a^8*b^2*c^2*A^2*B - 2*a^4*b^6*c^2*A^2*B + a^2*b^8*c^2*A^2*B + 3*a^8*c^4*A^2*B - 3*a^6*b^2*c^4*A^2*B + a^4*b^4*c^4*A^2*B - a^2*b^6*c^4*A^2*B - 3*a^6*c^6*A^2*B - a^2*b^4*c^6*A^2*B + a^4*c^8*A^2*B + a^2*b^2*c^8*A^2*B - a^8*b^2*c^2*A*B^2 + 2*a^6*b^4*c^2*A*B^2 - 2*a^2*b^8*c^2*A*B^2 + b^10*c^2*A*B^2 + a^6*b^2*c^4*A*B^2 - a^4*b^4*c^4*A*B^2 + 3*a^2*b^6*c^4*A*B^2 - 3*b^8*c^4*A*B^2 + a^4*b^2*c^6*A*B^2 + 3*b^6*c^6*A*B^2 - a^2*b^2*c^8*A*B^2 - b^4*c^8*A*B^2 + a^10*b^2*A^2*C - 3*a^8*b^4*A^2*C + 3*a^6*b^6*A^2*C - a^4*b^8*A^2*C - 2*a^8*b^2*c^2*A^2*C + 3*a^6*b^4*c^2*A^2*C - a^2*b^8*c^2*A^2*C - a^4*b^4*c^4*A^2*C + a^2*b^6*c^4*A^2*C + 2*a^4*b^2*c^6*A^2*C + a^2*b^4*c^6*A^2*C - a^2*b^2*c^8*A^2*C + a^10*b^2*A*B*C - 2*a^8*b^4*A*B*C + 2*a^4*b^8*A*B*C - a^2*b^10*A*B*C - a^10*c^2*A*B*C + 5*a^6*b^4*c^2*A*B*C - 5*a^4*b^6*c^2*A*B*C + b^10*c^2*A*B*C + 2*a^8*c^4*A*B*C - 5*a^6*b^2*c^4*A*B*C + 5*a^2*b^6*c^4*A*B*C - 2*b^8*c^4*A*B*C + 5*a^4*b^2*c^6*A*B*C - 5*a^2*b^4*c^6*A*B*C - 2*a^4*c^8*A*B*C + 2*b^4*c^8*A*B*C + a^2*c^10*A*B*C - b^2*c^10*A*B*C + a^8*b^4*B^2*C - 3*a^6*b^6*B^2*C + 3*a^4*b^8*B^2*C - a^2*b^10*B^2*C + a^8*b^2*c^2*B^2*C - 3*a^4*b^6*c^2*B^2*C + 2*a^2*b^8*c^2*B^2*C - a^6*b^2*c^4*B^2*C + a^4*b^4*c^4*B^2*C - a^4*b^2*c^6*B^2*C - 2*a^2*b^4*c^6*B^2*C + a^2*b^2*c^8*B^2*C + a^8*b^2*c^2*A*C^2 - a^6*b^4*c^2*A*C^2 - a^4*b^6*c^2*A*C^2 + a^2*b^8*c^2*A*C^2 - 2*a^6*b^2*c^4*A*C^2 + a^4*b^4*c^4*A*C^2 + b^8*c^4*A*C^2 - 3*a^2*b^4*c^6*A*C^2 - 3*b^6*c^6*A*C^2 + 2*a^2*b^2*c^8*A*C^2 + 3*b^4*c^8*A*C^2 - b^2*c^10*A*C^2 - a^8*b^2*c^2*B*C^2 + a^6*b^4*c^2*B*C^2 + a^4*b^6*c^2*B*C^2 - a^2*b^8*c^2*B*C^2 - a^8*c^4*B*C^2 - a^4*b^4*c^4*B*C^2 + 2*a^2*b^6*c^4*B*C^2 + 3*a^6*c^6*B*C^2 + 3*a^4*b^2*c^6*B*C^2 - 3*a^4*c^8*B*C^2 - 2*a^2*b^2*c^8*B*C^2 + a^2*c^10*B*C^2 = 0";
				break;
			case 563:
				equation = "-a^8*c^4*A^2*B + a^2*b^6*c^4*A^2*B + 3*a^6*c^6*A^2*B + a^4*b^2*c^6*A^2*B - a^2*b^4*c^6*A^2*B - 3*a^4*c^8*A^2*B - a^2*b^2*c^8*A^2*B + a^2*c^10*A^2*B - a^6*b^2*c^4*A*B^2 + b^8*c^4*A*B^2 + a^4*b^2*c^6*A*B^2 - a^2*b^4*c^6*A*B^2 - 3*b^6*c^6*A*B^2 + a^2*b^2*c^8*A*B^2 + 3*b^4*c^8*A*B^2 - b^2*c^10*A*B^2 + a^8*b^4*A^2*C - 3*a^6*b^6*A^2*C + 3*a^4*b^8*A^2*C - a^2*b^10*A^2*C - a^4*b^6*c^2*A^2*C + a^2*b^8*c^2*A^2*C + a^2*b^6*c^4*A^2*C - a^2*b^4*c^6*A^2*C + a^10*b^2*A*B*C - 2*a^8*b^4*A*B*C + 2*a^4*b^8*A*B*C - a^2*b^10*A*B*C - a^10*c^2*A*B*C + 3*a^6*b^4*c^2*A*B*C - 3*a^4*b^6*c^2*A*B*C + b^10*c^2*A*B*C + 2*a^8*c^4*A*B*C - 3*a^6*b^2*c^4*A*B*C + 3*a^2*b^6*c^4*A*B*C - 2*b^8*c^4*A*B*C + 3*a^4*b^2*c^6*A*B*C - 3*a^2*b^4*c^6*A*B*C - 2*a^4*c^8*A*B*C + 2*b^4*c^8*A*B*C + a^2*c^10*A*B*C - b^2*c^10*A*B*C + a^10*b^2*B^2*C - 3*a^8*b^4*B^2*C + 3*a^6*b^6*B^2*C - a^4*b^8*B^2*C - a^8*b^2*c^2*B^2*C + a^6*b^4*c^2*B^2*C - a^6*b^2*c^4*B^2*C + a^4*b^2*c^6*B^2*C + a^6*b^4*c^2*A*C^2 - a^4*b^6*c^2*A*C^2 - a^2*b^8*c^2*A*C^2 + b^10*c^2*A*C^2 + a^2*b^6*c^4*A*C^2 - 3*b^8*c^4*A*C^2 + 3*b^6*c^6*A*C^2 - b^4*c^8*A*C^2 - a^10*c^2*B*C^2 + a^8*b^2*c^2*B*C^2 + a^6*b^4*c^2*B*C^2 - a^4*b^6*c^2*B*C^2 + 3*a^8*c^4*B*C^2 - a^6*b^2*c^4*B*C^2 - 3*a^6*c^6*B*C^2 + a^4*c^8*B*C^2 = 0";
				break;
			case 564:
				equation = "2*a^12*c^2*A^2*B - 3*a^10*b^2*c^2*A^2*B - 6*a^8*b^4*c^2*A^2*B + 14*a^6*b^6*c^2*A^2*B - 6*a^4*b^8*c^2*A^2*B - 3*a^2*b^10*c^2*A^2*B + 2*b^12*c^2*A^2*B - 3*a^10*c^4*A^2*B + 18*a^8*b^2*c^4*A^2*B - 15*a^6*b^4*c^4*A^2*B - 15*a^4*b^6*c^4*A^2*B + 18*a^2*b^8*c^4*A^2*B - 3*b^10*c^4*A^2*B - 6*a^8*c^6*A^2*B - 15*a^6*b^2*c^6*A^2*B + 42*a^4*b^4*c^6*A^2*B - 15*a^2*b^6*c^6*A^2*B - 6*b^8*c^6*A^2*B + 14*a^6*c^8*A^2*B - 15*a^4*b^2*c^8*A^2*B - 15*a^2*b^4*c^8*A^2*B + 14*b^6*c^8*A^2*B - 6*a^4*c^10*A^2*B + 18*a^2*b^2*c^10*A^2*B - 6*b^4*c^10*A^2*B - 3*a^2*c^12*A^2*B - 3*b^2*c^12*A^2*B + 2*c^14*A^2*B - 2*a^12*c^2*A*B^2 + 3*a^10*b^2*c^2*A*B^2 + 6*a^8*b^4*c^2*A*B^2 - 14*a^6*b^6*c^2*A*B^2 + 6*a^4*b^8*c^2*A*B^2 + 3*a^2*b^10*c^2*A*B^2 - 2*b^12*c^2*A*B^2 + 3*a^10*c^4*A*B^2 - 18*a^8*b^2*c^4*A*B^2 + 15*a^6*b^4*c^4*A*B^2 + 15*a^4*b^6*c^4*A*B^2 - 18*a^2*b^8*c^4*A*B^2 + 3*b^10*c^4*A*B^2 + 6*a^8*c^6*A*B^2 + 15*a^6*b^2*c^6*A*B^2 - 42*a^4*b^4*c^6*A*B^2 + 15*a^2*b^6*c^6*A*B^2 + 6*b^8*c^6*A*B^2 - 14*a^6*c^8*A*B^2 + 15*a^4*b^2*c^8*A*B^2 + 15*a^2*b^4*c^8*A*B^2 - 14*b^6*c^8*A*B^2 + 6*a^4*c^10*A*B^2 - 18*a^2*b^2*c^10*A*B^2 + 6*b^4*c^10*A*B^2 + 3*a^2*c^12*A*B^2 + 3*b^2*c^12*A*B^2 - 2*c^14*A*B^2 - 2*a^12*b^2*A^2*C + 3*a^10*b^4*A^2*C + 6*a^8*b^6*A^2*C - 14*a^6*b^8*A^2*C + 6*a^4*b^10*A^2*C + 3*a^2*b^12*A^2*C - 2*b^14*A^2*C + 3*a^10*b^2*c^2*A^2*C - 18*a^8*b^4*c^2*A^2*C + 15*a^6*b^6*c^2*A^2*C + 15*a^4*b^8*c^2*A^2*C - 18*a^2*b^10*c^2*A^2*C + 3*b^12*c^2*A^2*C + 6*a^8*b^2*c^4*A^2*C + 15*a^6*b^4*c^4*A^2*C - 42*a^4*b^6*c^4*A^2*C + 15*a^2*b^8*c^4*A^2*C + 6*b^10*c^4*A^2*C - 14*a^6*b^2*c^6*A^2*C + 15*a^4*b^4*c^6*A^2*C + 15*a^2*b^6*c^6*A^2*C - 14*b^8*c^6*A^2*C + 6*a^4*b^2*c^8*A^2*C - 18*a^2*b^4*c^8*A^2*C + 6*b^6*c^8*A^2*C + 3*a^2*b^2*c^10*A^2*C + 3*b^4*c^10*A^2*C - 2*b^2*c^12*A^2*C + 3*a^12*b^2*A*B*C - 15*a^10*b^4*A*B*C + 30*a^8*b^6*A*B*C - 30*a^6*b^8*A*B*C + 15*a^4*b^10*A*B*C - 3*a^2*b^12*A*B*C - 3*a^12*c^2*A*B*C + 9*a^8*b^4*c^2*A*B*C - 9*a^4*b^8*c^2*A*B*C + 3*b^12*c^2*A*B*C + 15*a^10*c^4*A*B*C - 9*a^8*b^2*c^4*A*B*C + 9*a^2*b^8*c^4*A*B*C - 15*b^10*c^4*A*B*C - 30*a^8*c^6*A*B*C + 30*b^8*c^6*A*B*C + 30*a^6*c^8*A*B*C + 9*a^4*b^2*c^8*A*B*C - 9*a^2*b^4*c^8*A*B*C - 30*b^6*c^8*A*B*C - 15*a^4*c^10*A*B*C + 15*b^4*c^10*A*B*C + 3*a^2*c^12*A*B*C - 3*b^2*c^12*A*B*C + 2*a^14*B^2*C - 3*a^12*b^2*B^2*C - 6*a^10*b^4*B^2*C + 14*a^8*b^6*B^2*C - 6*a^6*b^8*B^2*C - 3*a^4*b^10*B^2*C + 2*a^2*b^12*B^2*C - 3*a^12*c^2*B^2*C + 18*a^10*b^2*c^2*B^2*C - 15*a^8*b^4*c^2*B^2*C - 15*a^6*b^6*c^2*B^2*C + 18*a^4*b^8*c^2*B^2*C - 3*a^2*b^10*c^2*B^2*C - 6*a^10*c^4*B^2*C - 15*a^8*b^2*c^4*B^2*C + 42*a^6*b^4*c^4*B^2*C - 15*a^4*b^6*c^4*B^2*C - 6*a^2*b^8*c^4*B^2*C + 14*a^8*c^6*B^2*C - 15*a^6*b^2*c^6*B^2*C - 15*a^4*b^4*c^6*B^2*C + 14*a^2*b^6*c^6*B^2*C - 6*a^6*c^8*B^2*C + 18*a^4*b^2*c^8*B^2*C - 6*a^2*b^4*c^8*B^2*C - 3*a^4*c^10*B^2*C - 3*a^2*b^2*c^10*B^2*C + 2*a^2*c^12*B^2*C + 2*a^12*b^2*A*C^2 - 3*a^10*b^4*A*C^2 - 6*a^8*b^6*A*C^2 + 14*a^6*b^8*A*C^2 - 6*a^4*b^10*A*C^2 - 3*a^2*b^12*A*C^2 + 2*b^14*A*C^2 - 3*a^10*b^2*c^2*A*C^2 + 18*a^8*b^4*c^2*A*C^2 - 15*a^6*b^6*c^2*A*C^2 - 15*a^4*b^8*c^2*A*C^2 + 18*a^2*b^10*c^2*A*C^2 - 3*b^12*c^2*A*C^2 - 6*a^8*b^2*c^4*A*C^2 - 15*a^6*b^4*c^4*A*C^2 + 42*a^4*b^6*c^4*A*C^2 - 15*a^2*b^8*c^4*A*C^2 - 6*b^10*c^4*A*C^2 + 14*a^6*b^2*c^6*A*C^2 - 15*a^4*b^4*c^6*A*C^2 - 15*a^2*b^6*c^6*A*C^2 + 14*b^8*c^6*A*C^2 - 6*a^4*b^2*c^8*A*C^2 + 18*a^2*b^4*c^8*A*C^2 - 6*b^6*c^8*A*C^2 - 3*a^2*b^2*c^10*A*C^2 - 3*b^4*c^10*A*C^2 + 2*b^2*c^12*A*C^2 - 2*a^14*B*C^2 + 3*a^12*b^2*B*C^2 + 6*a^10*b^4*B*C^2 - 14*a^8*b^6*B*C^2 + 6*a^6*b^8*B*C^2 + 3*a^4*b^10*B*C^2 - 2*a^2*b^12*B*C^2 + 3*a^12*c^2*B*C^2 - 18*a^10*b^2*c^2*B*C^2 + 15*a^8*b^4*c^2*B*C^2 + 15*a^6*b^6*c^2*B*C^2 - 18*a^4*b^8*c^2*B*C^2 + 3*a^2*b^10*c^2*B*C^2 + 6*a^10*c^4*B*C^2 + 15*a^8*b^2*c^4*B*C^2 - 42*a^6*b^4*c^4*B*C^2 + 15*a^4*b^6*c^4*B*C^2 + 6*a^2*b^8*c^4*B*C^2 - 14*a^8*c^6*B*C^2 + 15*a^6*b^2*c^6*B*C^2 + 15*a^4*b^4*c^6*B*C^2 - 14*a^2*b^6*c^6*B*C^2 + 6*a^6*c^8*B*C^2 - 18*a^4*b^2*c^8*B*C^2 + 6*a^2*b^4*c^8*B*C^2 + 3*a^4*c^10*B*C^2 + 3*a^2*b^2*c^10*B*C^2 - 2*a^2*c^12*B*C^2 = 0";
				break;
			case 565:
				equation = "-2*a^4*c^2*A^2*B - a^2*b^2*c^2*A^2*B + 3*b^4*c^2*A^2*B + a^2*c^4*A^2*B - 4*b^2*c^4*A^2*B + c^6*A^2*B - 3*a^4*c^2*A*B^2 + a^2*b^2*c^2*A*B^2 + 2*b^4*c^2*A*B^2 + 4*a^2*c^4*A*B^2 - b^2*c^4*A*B^2 - c^6*A*B^2 + 2*a^4*b^2*A^2*C - a^2*b^4*A^2*C - b^6*A^2*C + a^2*b^2*c^2*A^2*C + 4*b^4*c^2*A^2*C - 3*b^2*c^4*A^2*C + 6*a^4*b^2*A*B*C - 6*a^2*b^4*A*B*C - 6*a^4*c^2*A*B*C + 6*b^4*c^2*A*B*C + 6*a^2*c^4*A*B*C - 6*b^2*c^4*A*B*C + a^6*B^2*C + a^4*b^2*B^2*C - 2*a^2*b^4*B^2*C - 4*a^4*c^2*B^2*C - a^2*b^2*c^2*B^2*C + 3*a^2*c^4*B^2*C + 3*a^4*b^2*A*C^2 - 4*a^2*b^4*A*C^2 + b^6*A*C^2 - a^2*b^2*c^2*A*C^2 + b^4*c^2*A*C^2 - 2*b^2*c^4*A*C^2 - a^6*B*C^2 + 4*a^4*b^2*B*C^2 - 3*a^2*b^4*B*C^2 - a^4*c^2*B*C^2 + a^2*b^2*c^2*B*C^2 + 2*a^2*c^4*B*C^2 = 0";
				break;
			case 567:
				equation = "a^2*c^2*A^2*B - b^2*c^2*A^2*B - a^2*c^2*A*B^2 + b^2*c^2*A*B^2 + a^2*b^2*A^2*C - b^2*c^2*A^2*C + 2*a^4*A*B*C - 2*a^2*b^2*A*B*C + 2*b^4*A*B*C - 2*a^2*c^2*A*B*C - 2*b^2*c^2*A*B*C + 2*c^4*A*B*C + a^2*b^2*B^2*C - a^2*c^2*B^2*C - a^2*b^2*A*C^2 + b^2*c^2*A*C^2 - a^2*b^2*B*C^2 + a^2*c^2*B*C^2 = 0";
				break;
			case 568:
				equation = "b^2*c^2*A^2*B - c^4*A^2*B + a^2*c^2*A*B^2 - c^4*A*B^2 - b^4*A^2*C + b^2*c^2*A^2*C - 2*a^4*A*B*C + 2*a^2*b^2*A*B*C - 2*b^4*A*B*C + 2*a^2*c^2*A*B*C + 2*b^2*c^2*A*B*C - 2*c^4*A*B*C - a^4*B^2*C + a^2*c^2*B^2*C + a^2*b^2*A*C^2 - b^4*A*C^2 - a^4*B*C^2 + a^2*b^2*B*C^2 = 0";
				break;
			case 569:
				equation = "a^4*c^2*A^2*B - 2*a^2*b^2*c^2*A^2*B + b^4*c^2*A^2*B - 3*a^2*c^4*A^2*B - 3*b^2*c^4*A^2*B + 2*c^6*A^2*B - a^4*c^2*A*B^2 + 2*a^2*b^2*c^2*A*B^2 - b^4*c^2*A*B^2 + 3*a^2*c^4*A*B^2 + 3*b^2*c^4*A*B^2 - 2*c^6*A*B^2 - a^4*b^2*A^2*C + 3*a^2*b^4*A^2*C - 2*b^6*A^2*C + 2*a^2*b^2*c^2*A^2*C + 3*b^4*c^2*A^2*C - b^2*c^4*A^2*C + 2*a^4*b^2*A*B*C - 2*a^2*b^4*A*B*C - 2*a^4*c^2*A*B*C + 2*b^4*c^2*A*B*C + 2*a^2*c^4*A*B*C - 2*b^2*c^4*A*B*C + 2*a^6*B^2*C - 3*a^4*b^2*B^2*C + a^2*b^4*B^2*C - 3*a^4*c^2*B^2*C - 2*a^2*b^2*c^2*B^2*C + a^2*c^4*B^2*C + a^4*b^2*A*C^2 - 3*a^2*b^4*A*C^2 + 2*b^6*A*C^2 - 2*a^2*b^2*c^2*A*C^2 - 3*b^4*c^2*A*C^2 + b^2*c^4*A*C^2 - 2*a^6*B*C^2 + 3*a^4*b^2*B*C^2 - a^2*b^4*B*C^2 + 3*a^4*c^2*B*C^2 + 2*a^2*b^2*c^2*B*C^2 - a^2*c^4*B*C^2 = 0";
				break;
			case 570:
				equation = "-a^4*c^4*A^2*B - b^4*c^4*A^2*B + a^2*c^6*A^2*B + b^2*c^6*A^2*B + a^4*c^4*A*B^2 + b^4*c^4*A*B^2 - a^2*c^6*A*B^2 - b^2*c^6*A*B^2 + a^4*b^4*A^2*C - a^2*b^6*A^2*C - b^6*c^2*A^2*C + b^4*c^4*A^2*C + a^6*b^2*A*B*C - a^2*b^6*A*B*C - a^6*c^2*A*B*C + b^6*c^2*A*B*C + a^2*c^6*A*B*C - b^2*c^6*A*B*C + a^6*b^2*B^2*C - a^4*b^4*B^2*C + a^6*c^2*B^2*C - a^4*c^4*B^2*C - a^4*b^4*A*C^2 + a^2*b^6*A*C^2 + b^6*c^2*A*C^2 - b^4*c^4*A*C^2 - a^6*b^2*B*C^2 + a^4*b^4*B*C^2 - a^6*c^2*B*C^2 + a^4*c^4*B*C^2 = 0";
				break;
			case 571:
				equation = "-a^2*c*A^2*B + b^2*c*A^2*B + a*c^2*A^2*B + b*c^2*A^2*B - a^2*c*A*B^2 + b^2*c*A*B^2 - a*c^2*A*B^2 - b*c^2*A*B^2 + a^2*b*A^2*C - a*b^2*A^2*C - b^2*c*A^2*C - b*c^2*A^2*C + 2*a^2*b*A*B*C - 2*a*b^2*A*B*C - 2*a^2*c*A*B*C + 2*b^2*c*A*B*C + 2*a*c^2*A*B*C - 2*b*c^2*A*B*C + a^2*b*B^2*C - a*b^2*B^2*C + a^2*c*B^2*C + a*c^2*B^2*C + a^2*b*A*C^2 + a*b^2*A*C^2 + b^2*c*A*C^2 - b*c^2*A*C^2 - a^2*b*B*C^2 - a*b^2*B*C^2 - a^2*c*B*C^2 + a*c^2*B*C^2 = 0";
				break;
			case 572:
				equation = "-a^3*c*A^2*B + a^2*b*c*A^2*B + a*b^2*c*A^2*B - b^3*c*A^2*B + 2*a^2*c^2*A^2*B - 2*a*b*c^2*A^2*B - a*c^3*A^2*B + b*c^3*A^2*B - a^3*c*A*B^2 + a^2*b*c*A*B^2 + a*b^2*c*A*B^2 - b^3*c*A*B^2 - 2*a*b*c^2*A*B^2 + 2*b^2*c^2*A*B^2 + a*c^3*A*B^2 - b*c^3*A*B^2 - a^3*b*A^2*C + 2*a^2*b^2*A^2*C - a*b^3*A^2*C + a^2*b*c*A^2*C - 2*a*b^2*c*A^2*C + b^3*c*A^2*C + a*b*c^2*A^2*C - b*c^3*A^2*C - 2*a^3*b*A*B*C + 4*a^2*b^2*A*B*C - 2*a*b^3*A*B*C - 2*a^3*c*A*B*C + 2*a^2*b*c*A*B*C + 2*a*b^2*c*A*B*C - 2*b^3*c*A*B*C + 4*a^2*c^2*A*B*C + 2*a*b*c^2*A*B*C + 4*b^2*c^2*A*B*C - 2*a*c^3*A*B*C - 2*b*c^3*A*B*C - a^3*b*B^2*C + 2*a^2*b^2*B^2*C - a*b^3*B^2*C + a^3*c*B^2*C - 2*a^2*b*c*B^2*C + a*b^2*c*B^2*C + a*b*c^2*B^2*C - a*c^3*B^2*C - a^3*b*A*C^2 + a*b^3*A*C^2 + a^2*b*c*A*C^2 - 2*a*b^2*c*A*C^2 - b^3*c*A*C^2 + a*b*c^2*A*C^2 + 2*b^2*c^2*A*C^2 - b*c^3*A*C^2 + a^3*b*B*C^2 - a*b^3*B*C^2 - a^3*c*B*C^2 - 2*a^2*b*c*B*C^2 + a*b^2*c*B*C^2 + 2*a^2*c^2*B*C^2 + a*b*c^2*B*C^2 - a*c^3*B*C^2 = 0";
				break;
			case 573:
				equation = "2*a^6*A^2*B - 6*a^4*b^2*A^2*B + 6*a^2*b^4*A^2*B - 2*b^6*A^2*B - 17*a^5*c*A^2*B + 13*a^4*b*c*A^2*B + 34*a^3*b^2*c*A^2*B - 26*a^2*b^3*c*A^2*B - 17*a*b^4*c*A^2*B + 13*b^5*c*A^2*B + 28*a^4*c^2*A^2*B - 10*a^3*b*c^2*A^2*B - 30*a^2*b^2*c^2*A^2*B + 10*a*b^3*c^2*A^2*B + 2*b^4*c^2*A^2*B - 2*a^3*c^3*A^2*B + 12*a^2*b*c^3*A^2*B - 18*a*b^2*c^3*A^2*B - 24*b^3*c^3*A^2*B - 26*a^2*c^4*A^2*B - 26*a*b*c^4*A^2*B + 4*b^2*c^4*A^2*B + 19*a*c^5*A^2*B + 11*b*c^5*A^2*B - 4*c^6*A^2*B + 2*a^6*A*B^2 - 6*a^4*b^2*A*B^2 + 6*a^2*b^4*A*B^2 - 2*b^6*A*B^2 - 13*a^5*c*A*B^2 + 17*a^4*b*c*A*B^2 + 26*a^3*b^2*c*A*B^2 - 34*a^2*b^3*c*A*B^2 - 13*a*b^4*c*A*B^2 + 17*b^5*c*A*B^2 - 2*a^4*c^2*A*B^2 - 10*a^3*b*c^2*A*B^2 + 30*a^2*b^2*c^2*A*B^2 + 10*a*b^3*c^2*A*B^2 - 28*b^4*c^2*A*B^2 + 24*a^3*c^3*A*B^2 + 18*a^2*b*c^3*A*B^2 - 12*a*b^2*c^3*A*B^2 + 2*b^3*c^3*A*B^2 - 4*a^2*c^4*A*B^2 + 26*a*b*c^4*A*B^2 + 26*b^2*c^4*A*B^2 - 11*a*c^5*A*B^2 - 19*b*c^5*A*B^2 + 4*c^6*A*B^2 - 2*a^6*A^2*C + 17*a^5*b*A^2*C - 28*a^4*b^2*A^2*C + 2*a^3*b^3*A^2*C + 26*a^2*b^4*A^2*C - 19*a*b^5*A^2*C + 4*b^6*A^2*C - 13*a^4*b*c*A^2*C + 10*a^3*b^2*c*A^2*C - 12*a^2*b^3*c*A^2*C + 26*a*b^4*c*A^2*C - 11*b^5*c*A^2*C + 6*a^4*c^2*A^2*C - 34*a^3*b*c^2*A^2*C + 30*a^2*b^2*c^2*A^2*C + 18*a*b^3*c^2*A^2*C - 4*b^4*c^2*A^2*C + 26*a^2*b*c^3*A^2*C - 10*a*b^2*c^3*A^2*C + 24*b^3*c^3*A^2*C - 6*a^2*c^4*A^2*C + 17*a*b*c^4*A^2*C - 2*b^2*c^4*A^2*C - 13*b*c^5*A^2*C + 2*c^6*A^2*C + 36*a^5*b*A*B*C - 72*a^4*b^2*A*B*C + 72*a^2*b^4*A*B*C - 36*a*b^5*A*B*C - 36*a^5*c*A*B*C + 108*a^3*b^2*c*A*B*C - 108*a^2*b^3*c*A*B*C + 36*b^5*c*A*B*C + 72*a^4*c^2*A*B*C - 108*a^3*b*c^2*A*B*C + 108*a*b^3*c^2*A*B*C - 72*b^4*c^2*A*B*C + 108*a^2*b*c^3*A*B*C - 108*a*b^2*c^3*A*B*C - 72*a^2*c^4*A*B*C + 72*b^2*c^4*A*B*C + 36*a*c^5*A*B*C - 36*b*c^5*A*B*C - 4*a^6*B^2*C + 19*a^5*b*B^2*C - 26*a^4*b^2*B^2*C - 2*a^3*b^3*B^2*C + 28*a^2*b^4*B^2*C - 17*a*b^5*B^2*C + 2*b^6*B^2*C + 11*a^5*c*B^2*C - 26*a^4*b*c*B^2*C + 12*a^3*b^2*c*B^2*C - 10*a^2*b^3*c*B^2*C + 13*a*b^4*c*B^2*C + 4*a^4*c^2*B^2*C - 18*a^3*b*c^2*B^2*C - 30*a^2*b^2*c^2*B^2*C + 34*a*b^3*c^2*B^2*C - 6*b^4*c^2*B^2*C - 24*a^3*c^3*B^2*C + 10*a^2*b*c^3*B^2*C - 26*a*b^2*c^3*B^2*C + 2*a^2*c^4*B^2*C - 17*a*b*c^4*B^2*C + 6*b^2*c^4*B^2*C + 13*a*c^5*B^2*C - 2*c^6*B^2*C - 2*a^6*A*C^2 + 13*a^5*b*A*C^2 + 2*a^4*b^2*A*C^2 - 24*a^3*b^3*A*C^2 + 4*a^2*b^4*A*C^2 + 11*a*b^5*A*C^2 - 4*b^6*A*C^2 - 17*a^4*b*c*A*C^2 + 10*a^3*b^2*c*A*C^2 - 18*a^2*b^3*c*A*C^2 - 26*a*b^4*c*A*C^2 + 19*b^5*c*A*C^2 + 6*a^4*c^2*A*C^2 - 26*a^3*b*c^2*A*C^2 - 30*a^2*b^2*c^2*A*C^2 + 12*a*b^3*c^2*A*C^2 - 26*b^4*c^2*A*C^2 + 34*a^2*b*c^3*A*C^2 - 10*a*b^2*c^3*A*C^2 - 2*b^3*c^3*A*C^2 - 6*a^2*c^4*A*C^2 + 13*a*b*c^4*A*C^2 + 28*b^2*c^4*A*C^2 - 17*b*c^5*A*C^2 + 2*c^6*A*C^2 + 4*a^6*B*C^2 - 11*a^5*b*B*C^2 - 4*a^4*b^2*B*C^2 + 24*a^3*b^3*B*C^2 - 2*a^2*b^4*B*C^2 - 13*a*b^5*B*C^2 + 2*b^6*B*C^2 - 19*a^5*c*B*C^2 + 26*a^4*b*c*B*C^2 + 18*a^3*b^2*c*B*C^2 - 10*a^2*b^3*c*B*C^2 + 17*a*b^4*c*B*C^2 + 26*a^4*c^2*B*C^2 - 12*a^3*b*c^2*B*C^2 + 30*a^2*b^2*c^2*B*C^2 + 26*a*b^3*c^2*B*C^2 - 6*b^4*c^2*B*C^2 + 2*a^3*c^3*B*C^2 + 10*a^2*b*c^3*B*C^2 - 34*a*b^2*c^3*B*C^2 - 28*a^2*c^4*B*C^2 - 13*a*b*c^4*B*C^2 + 6*b^2*c^4*B*C^2 + 17*a*c^5*B*C^2 - 2*c^6*B*C^2 = 0"; 
				break;
			case 574:
				equation = "2*a^4*A^2*B - 4*a^3*b*A^2*B + 4*a*b^3*A^2*B - 2*b^4*A^2*B - 3*a^3*c*A^2*B + 5*a^2*b*c*A^2*B - a*b^2*c*A^2*B - b^3*c*A^2*B - 4*a^2*c^2*A^2*B - 2*a*b*c^2*A^2*B + 6*b^2*c^2*A^2*B + 9*a*c^3*A^2*B + b*c^3*A^2*B - 4*c^4*A^2*B + 2*a^4*A*B^2 - 4*a^3*b*A*B^2 + 4*a*b^3*A*B^2 - 2*b^4*A*B^2 + a^3*c*A*B^2 + a^2*b*c*A*B^2 - 5*a*b^2*c*A*B^2 + 3*b^3*c*A*B^2 - 6*a^2*c^2*A*B^2 + 2*a*b*c^2*A*B^2 + 4*b^2*c^2*A*B^2 - a*c^3*A*B^2 - 9*b*c^3*A*B^2 + 4*c^4*A*B^2 - 2*a^4*A^2*C + 3*a^3*b*A^2*C + 4*a^2*b^2*A^2*C - 9*a*b^3*A^2*C + 4*b^4*A^2*C + 4*a^3*c*A^2*C - 5*a^2*b*c*A^2*C + 2*a*b^2*c*A^2*C - b^3*c*A^2*C + a*b*c^2*A^2*C - 6*b^2*c^2*A^2*C - 4*a*c^3*A^2*C + b*c^3*A^2*C + 2*c^4*A^2*C - 4*a^4*B^2*C + 9*a^3*b*B^2*C - 4*a^2*b^2*B^2*C - 3*a*b^3*B^2*C + 2*b^4*B^2*C + a^3*c*B^2*C - 2*a^2*b*c*B^2*C + 5*a*b^2*c*B^2*C - 4*b^3*c*B^2*C + 6*a^2*c^2*B^2*C - a*b*c^2*B^2*C - a*c^3*B^2*C + 4*b*c^3*B^2*C - 2*c^4*B^2*C - 2*a^4*A*C^2 - a^3*b*A*C^2 + 6*a^2*b^2*A*C^2 + a*b^3*A*C^2 - 4*b^4*A*C^2 + 4*a^3*c*A*C^2 - a^2*b*c*A*C^2 - 2*a*b^2*c*A*C^2 + 9*b^3*c*A*C^2 + 5*a*b*c^2*A*C^2 - 4*b^2*c^2*A*C^2 - 4*a*c^3*A*C^2 - 3*b*c^3*A*C^2 + 2*c^4*A*C^2 + 4*a^4*B*C^2 - a^3*b*B*C^2 - 6*a^2*b^2*B*C^2 + a*b^3*B*C^2 + 2*b^4*B*C^2 - 9*a^3*c*B*C^2 + 2*a^2*b*c*B*C^2 + a*b^2*c*B*C^2 - 4*b^3*c*B*C^2 + 4*a^2*c^2*B*C^2 - 5*a*b*c^2*B*C^2 + 3*a*c^3*B*C^2 + 4*b*c^3*B*C^2 - 2*c^4*B*C^2 = 0"; 
				break;
			case 575:
				equation = "a^6*A^2*B - 3*a^4*b^2*A^2*B + 3*a^2*b^4*A^2*B - b^6*A^2*B - 4*a^5*c*A^2*B + 2*a^4*b*c*A^2*B + 8*a^3*b^2*c*A^2*B - 4*a^2*b^3*c*A^2*B - 4*a*b^4*c*A^2*B + 2*b^5*c*A^2*B + 5*a^4*c^2*A^2*B - 5*a^3*b*c^2*A^2*B - 6*a^2*b^2*c^2*A^2*B + 5*a*b^3*c^2*A^2*B + b^4*c^2*A^2*B - a^3*c^3*A^2*B + 6*a^2*b*c^3*A^2*B - 3*b^3*c^3*A^2*B - 4*a^2*c^4*A^2*B - 4*a*b*c^4*A^2*B + 2*b^2*c^4*A^2*B + 5*a*c^5*A^2*B + b*c^5*A^2*B - 2*c^6*A^2*B + a^6*A*B^2 - 3*a^4*b^2*A*B^2 + 3*a^2*b^4*A*B^2 - b^6*A*B^2 - 2*a^5*c*A*B^2 + 4*a^4*b*c*A*B^2 + 4*a^3*b^2*c*A*B^2 - 8*a^2*b^3*c*A*B^2 - 2*a*b^4*c*A*B^2 + 4*b^5*c*A*B^2 - a^4*c^2*A*B^2 - 5*a^3*b*c^2*A*B^2 + 6*a^2*b^2*c^2*A*B^2 + 5*a*b^3*c^2*A*B^2 - 5*b^4*c^2*A*B^2 + 3*a^3*c^3*A*B^2 - 6*a*b^2*c^3*A*B^2 + b^3*c^3*A*B^2 - 2*a^2*c^4*A*B^2 + 4*a*b*c^4*A*B^2 + 4*b^2*c^4*A*B^2 - a*c^5*A*B^2 - 5*b*c^5*A*B^2 + 2*c^6*A*B^2 - a^6*A^2*C + 4*a^5*b*A^2*C - 5*a^4*b^2*A^2*C + a^3*b^3*A^2*C + 4*a^2*b^4*A^2*C - 5*a*b^5*A^2*C + 2*b^6*A^2*C - 2*a^4*b*c*A^2*C + 5*a^3*b^2*c*A^2*C - 6*a^2*b^3*c*A^2*C + 4*a*b^4*c*A^2*C - b^5*c*A^2*C + 3*a^4*c^2*A^2*C - 8*a^3*b*c^2*A^2*C + 6*a^2*b^2*c^2*A^2*C - 2*b^4*c^2*A^2*C + 4*a^2*b*c^3*A^2*C - 5*a*b^2*c^3*A^2*C + 3*b^3*c^3*A^2*C - 3*a^2*c^4*A^2*C + 4*a*b*c^4*A^2*C - b^2*c^4*A^2*C - 2*b*c^5*A^2*C + c^6*A^2*C + 9*a^5*b*A*B*C - 18*a^4*b^2*A*B*C + 18*a^2*b^4*A*B*C - 9*a*b^5*A*B*C - 9*a^5*c*A*B*C + 27*a^3*b^2*c*A*B*C - 27*a^2*b^3*c*A*B*C + 9*b^5*c*A*B*C + 18*a^4*c^2*A*B*C - 27*a^3*b*c^2*A*B*C + 27*a*b^3*c^2*A*B*C - 18*b^4*c^2*A*B*C + 27*a^2*b*c^3*A*B*C - 27*a*b^2*c^3*A*B*C - 18*a^2*c^4*A*B*C + 18*b^2*c^4*A*B*C + 9*a*c^5*A*B*C - 9*b*c^5*A*B*C - 2*a^6*B^2*C + 5*a^5*b*B^2*C - 4*a^4*b^2*B^2*C - a^3*b^3*B^2*C + 5*a^2*b^4*B^2*C - 4*a*b^5*B^2*C + b^6*B^2*C + a^5*c*B^2*C - 4*a^4*b*c*B^2*C + 6*a^3*b^2*c*B^2*C - 5*a^2*b^3*c*B^2*C + 2*a*b^4*c*B^2*C + 2*a^4*c^2*B^2*C - 6*a^2*b^2*c^2*B^2*C + 8*a*b^3*c^2*B^2*C - 3*b^4*c^2*B^2*C - 3*a^3*c^3*B^2*C + 5*a^2*b*c^3*B^2*C - 4*a*b^2*c^3*B^2*C + a^2*c^4*B^2*C - 4*a*b*c^4*B^2*C + 3*b^2*c^4*B^2*C + 2*a*c^5*B^2*C - c^6*B^2*C - a^6*A*C^2 + 2*a^5*b*A*C^2 + a^4*b^2*A*C^2 - 3*a^3*b^3*A*C^2 + 2*a^2*b^4*A*C^2 + a*b^5*A*C^2 - 2*b^6*A*C^2 - 4*a^4*b*c*A*C^2 + 5*a^3*b^2*c*A*C^2 - 4*a*b^4*c*A*C^2 + 5*b^5*c*A*C^2 + 3*a^4*c^2*A*C^2 - 4*a^3*b*c^2*A*C^2 - 6*a^2*b^2*c^2*A*C^2 + 6*a*b^3*c^2*A*C^2 - 4*b^4*c^2*A*C^2 + 8*a^2*b*c^3*A*C^2 - 5*a*b^2*c^3*A*C^2 - b^3*c^3*A*C^2 - 3*a^2*c^4*A*C^2 + 2*a*b*c^4*A*C^2 + 5*b^2*c^4*A*C^2 - 4*b*c^5*A*C^2 + c^6*A*C^2 + 2*a^6*B*C^2 - a^5*b*B*C^2 - 2*a^4*b^2*B*C^2 + 3*a^3*b^3*B*C^2 - a^2*b^4*B*C^2 - 2*a*b^5*B*C^2 + b^6*B*C^2 - 5*a^5*c*B*C^2 + 4*a^4*b*c*B*C^2 - 5*a^2*b^3*c*B*C^2 + 4*a*b^4*c*B*C^2 + 4*a^4*c^2*B*C^2 - 6*a^3*b*c^2*B*C^2 + 6*a^2*b^2*c^2*B*C^2 + 4*a*b^3*c^2*B*C^2 - 3*b^4*c^2*B*C^2 + a^3*c^3*B*C^2 + 5*a^2*b*c^3*B*C^2 - 8*a*b^2*c^3*B*C^2 - 5*a^2*c^4*B*C^2 - 2*a*b*c^4*B*C^2 + 3*b^2*c^4*B*C^2 + 4*a*c^5*B*C^2 - c^6*B*C^2 = 0"; 
				break;
			case 576:
				equation = "-a^6*b^2*c^6*A^2*B - a^4*b^4*c^6*A^2*B + a^2*b^6*c^6*A^2*B + b^8*c^6*A^2*B + a^4*b^2*c^8*A^2*B - 2*a^2*b^4*c^8*A^2*B - 3*b^6*c^8*A^2*B + a^2*b^2*c^10*A^2*B + 3*b^4*c^10*A^2*B - b^2*c^12*A^2*B - a^8*c^6*A*B^2 - a^6*b^2*c^6*A*B^2 + a^4*b^4*c^6*A*B^2 + a^2*b^6*c^6*A*B^2 + 3*a^6*c^8*A*B^2 + 2*a^4*b^2*c^8*A*B^2 - a^2*b^4*c^8*A*B^2 - 3*a^4*c^10*A*B^2 - a^2*b^2*c^10*A*B^2 + a^2*c^12*A*B^2 + a^6*b^6*c^2*A^2*C - a^4*b^8*c^2*A^2*C - a^2*b^10*c^2*A^2*C + b^12*c^2*A^2*C + a^4*b^6*c^4*A^2*C + 2*a^2*b^8*c^4*A^2*C - 3*b^10*c^4*A^2*C - a^2*b^6*c^6*A^2*C + 3*b^8*c^6*A^2*C - b^6*c^8*A^2*C - a^12*c^2*B^2*C + a^10*b^2*c^2*B^2*C + a^8*b^4*c^2*B^2*C - a^6*b^6*c^2*B^2*C + 3*a^10*c^4*B^2*C - 2*a^8*b^2*c^4*B^2*C - a^6*b^4*c^4*B^2*C - 3*a^8*c^6*B^2*C + a^6*b^2*c^6*B^2*C + a^6*c^8*B^2*C + a^8*b^6*A*C^2 - 3*a^6*b^8*A*C^2 + 3*a^4*b^10*A*C^2 - a^2*b^12*A*C^2 + a^6*b^6*c^2*A*C^2 - 2*a^4*b^8*c^2*A*C^2 + a^2*b^10*c^2*A*C^2 - a^4*b^6*c^4*A*C^2 + a^2*b^8*c^4*A*C^2 - a^2*b^6*c^6*A*C^2 + a^12*b^2*B*C^2 - 3*a^10*b^4*B*C^2 + 3*a^8*b^6*B*C^2 - a^6*b^8*B*C^2 - a^10*b^2*c^2*B*C^2 + 2*a^8*b^4*c^2*B*C^2 - a^6*b^6*c^2*B*C^2 - a^8*b^2*c^4*B*C^2 + a^6*b^4*c^4*B*C^2 + a^6*b^2*c^6*B*C^2 = 0"; 
				break;
			case 577:
				equation = "-a*c^2*A^2*B + b*c^2*A^2*B + c^3*A^2*B - a*c^2*A*B^2 + b*c^2*A*B^2 - c^3*A*B^2 + a*b^2*A^2*C - b^3*A^2*C - b^2*c*A^2*C + 2*a^2*b*A*B*C - 2*a*b^2*A*B*C - 2*a^2*c*A*B*C + 2*b^2*c*A*B*C + 2*a*c^2*A*B*C - 2*b*c^2*A*B*C + a^3*B^2*C - a^2*b*B^2*C + a^2*c*B^2*C + a*b^2*A*C^2 + b^3*A*C^2 - b^2*c*A*C^2 - a^3*B*C^2 - a^2*b*B*C^2 + a^2*c*B*C^2 = 0"; 
				break;
			case 578:
				equation = "-a^3*b*c^2*A^2*B + 2*a^2*b^2*c^2*A^2*B - a*b^3*c^2*A^2*B + a^3*c^3*A^2*B - 2*a*b^2*c^3*A^2*B + b^3*c^3*A^2*B - 2*a^2*c^4*A^2*B + 2*a*b*c^4*A^2*B + a*c^5*A^2*B - b*c^5*A^2*B - a^3*b*c^2*A*B^2 + 2*a^2*b^2*c^2*A*B^2 - a*b^3*c^2*A*B^2 + a^3*c^3*A*B^2 - 2*a^2*b*c^3*A*B^2 + b^3*c^3*A*B^2 + 2*a*b*c^4*A*B^2 - 2*b^2*c^4*A*B^2 - a*c^5*A*B^2 + b*c^5*A*B^2 + a^3*b^3*A^2*C - 2*a^2*b^4*A^2*C + a*b^5*A^2*C - a^3*b^2*c*A^2*C + 2*a*b^4*c*A^2*C - b^5*c*A^2*C + 2*a^2*b^2*c^2*A^2*C - 2*a*b^3*c^2*A^2*C - a*b^2*c^3*A^2*C + b^3*c^3*A^2*C + a^5*b*A*B*C - 2*a^4*b^2*A*B*C + 2*a^3*b^3*A*B*C - 2*a^2*b^4*A*B*C + a*b^5*A*B*C + a^5*c*A*B*C - 2*a^4*b*c*A*B*C + a^3*b^2*c*A*B*C + a^2*b^3*c*A*B*C - 2*a*b^4*c*A*B*C + b^5*c*A*B*C - 2*a^4*c^2*A*B*C + a^3*b*c^2*A*B*C + a*b^3*c^2*A*B*C - 2*b^4*c^2*A*B*C + 2*a^3*c^3*A*B*C + a^2*b*c^3*A*B*C + a*b^2*c^3*A*B*C + 2*b^3*c^3*A*B*C - 2*a^2*c^4*A*B*C - 2*a*b*c^4*A*B*C - 2*b^2*c^4*A*B*C + a*c^5*A*B*C + b*c^5*A*B*C + a^5*b*B^2*C - 2*a^4*b^2*B^2*C + a^3*b^3*B^2*C - a^5*c*B^2*C + 2*a^4*b*c*B^2*C - a^2*b^3*c*B^2*C - 2*a^3*b*c^2*B^2*C + 2*a^2*b^2*c^2*B^2*C + a^3*c^3*B^2*C - a^2*b*c^3*B^2*C + a^3*b^3*A*C^2 - a*b^5*A*C^2 - a^3*b^2*c*A*C^2 - 2*a^2*b^3*c*A*C^2 + 2*a*b^4*c*A*C^2 + b^5*c*A*C^2 + 2*a^2*b^2*c^2*A*C^2 - 2*b^4*c^2*A*C^2 - a*b^2*c^3*A*C^2 + b^3*c^3*A*C^2 - a^5*b*B*C^2 + a^3*b^3*B*C^2 + a^5*c*B*C^2 + 2*a^4*b*c*B*C^2 - 2*a^3*b^2*c*B*C^2 - a^2*b^3*c*B*C^2 - 2*a^4*c^2*B*C^2 + 2*a^2*b^2*c^2*B*C^2 + a^3*c^3*B*C^2 - a^2*b*c^3*B*C^2 = 0"; 
				break;
			case 579:
				 
				break;
			case 580:
				equation = "a^6*A^2*B - 3*a^4*b^2*A^2*B + 3*a^2*b^4*A^2*B - b^6*A^2*B - a^4*c^2*A^2*B + a^2*b^2*c^2*A^2*B + b^2*c^4*A^2*B + a^6*A*B^2 - 3*a^4*b^2*A*B^2 + 3*a^2*b^4*A*B^2 - b^6*A*B^2 - a^2*b^2*c^2*A*B^2 + b^4*c^2*A*B^2 - a^2*c^4*A*B^2 - a^6*A^2*C + a^4*b^2*A^2*C + 3*a^4*c^2*A^2*C - a^2*b^2*c^2*A^2*C - b^4*c^2*A^2*C - 3*a^2*c^4*A^2*C + c^6*A^2*C - 2*a^4*b^2*A*B*C + 2*a^2*b^4*A*B*C + 2*a^4*c^2*A*B*C - 2*b^4*c^2*A*B*C - 2*a^2*c^4*A*B*C + 2*b^2*c^4*A*B*C - a^2*b^4*B^2*C + b^6*B^2*C + a^4*c^2*B^2*C + a^2*b^2*c^2*B^2*C - 3*b^4*c^2*B^2*C + 3*b^2*c^4*B^2*C - c^6*B^2*C - a^6*A*C^2 + a^2*b^4*A*C^2 + 3*a^4*c^2*A*C^2 + a^2*b^2*c^2*A*C^2 - 3*a^2*c^4*A*C^2 - b^2*c^4*A*C^2 + c^6*A*C^2 - a^4*b^2*B*C^2 + b^6*B*C^2 - a^2*b^2*c^2*B*C^2 - 3*b^4*c^2*B*C^2 + a^2*c^4*B*C^2 + 3*b^2*c^4*B*C^2 - c^6*B*C^2 = 0";
				break;
			case 581:
				equation = "-a^4 c^2 A^2 B + 3 a^2 b^2 c^2 A^2 B - 2 b^4 c^2 A^2 B + a^2 c^4 A^2 B + 2 b^2 c^4 A^2 B + 2 a^4 c^2 A B^2 - 3 a^2 b^2 c^2 A B^2 + b^4 c^2 A B^2 - 2 a^2 c^4 A B^2 - b^2 c^4 A B^2 + a^4 b^2 A^2 C - a^2 b^4 A^2 C - 3 a^2 b^2 c^2 A^2 C - 2 b^4 c^2 A^2 C + 2 b^2 c^4 A^2 C + 2 a^4 b^2 A B C - 2 a^2 b^4 A B C - 2 a^4 c^2 A B C + 2 b^4 c^2 A B C + 2 a^2 c^4 A B C - 2 b^2 c^4 A B C + a^4 b^2 B^2 C - a^2 b^4 B^2 C + 2 a^4 c^2 B^2 C + 3 a^2 b^2 c^2 B^2 C - 2 a^2 c^4 B^2 C - 2 a^4 b^2 A C^2 + 2 a^2 b^4 A C^2 + 3 a^2 b^2 c^2 A C^2 + b^4 c^2 A C^2 - b^2 c^4 A C^2 - 2 a^4 b^2 B C^2 + 2 a^2 b^4 B C^2 - a^4 c^2 B C^2 - 3 a^2 b^2 c^2 B C^2 + a^2 c^4 B C^2 = 0";
				break;
			case 582:
				equation = "-a^6 c^2 A^2 B - 4 a^4 b^2 c^2 A^2 B + 7 a^2 b^4 c^2 A^2 B - 2 b^6 c^2 A^2 B + 6 a^2 b^2 c^4 A^2 B + a^2 c^6 A^2 B + 2 b^2 c^6 A^2 B + 2 a^6 c^2 A B^2 - 7 a^4 b^2 c^2 A B^2 + 4 a^2 b^4 c^2 A B^2 + b^6 c^2 A B^2 - 6 a^2 b^2 c^4 A B^2 - 2 a^2 c^6 A B^2 - b^2 c^6 A B^2 + a^6 b^2 A^2 C - a^2 b^6 A^2 C + 4 a^4 b^2 c^2 A^2 C - 6 a^2 b^4 c^2 A^2 C - 2 b^6 c^2 A^2 C - 7 a^2 b^2 c^4 A^2 C + 2 b^2 c^6 A^2 C + 2 a^6 b^2 A B C - 2 a^2 b^6 A B C - 2 a^6 c^2 A B C + 2 b^6 c^2 A B C + 2 a^2 c^6 A B C - 2 b^2 c^6 A B C + a^6 b^2 B^2 C - a^2 b^6 B^2 C + 2 a^6 c^2 B^2 C + 6 a^4 b^2 c^2 B^2 C - 4 a^2 b^4 c^2 B^2 C + 7 a^2 b^2 c^4 B^2 C - 2 a^2 c^6 B^2 C - 2 a^6 b^2 A C^2 + 2 a^2 b^6 A C^2 + 7 a^4 b^2 c^2 A C^2 + 6 a^2 b^4 c^2 A C^2 + b^6 c^2 A C^2 - 4 a^2 b^2 c^4 A C^2 - b^2 c^6 A C^2 - 2 a^6 b^2 B C^2 + 2 a^2 b^6 B C^2 - a^6 c^2 B C^2 - 6 a^4 b^2 c^2 B C^2 - 7 a^2 b^4 c^2 B C^2 + 4 a^2 b^2 c^4 B C^2 + a^2 c^6 B C^2 = 0";
				break;
			case 583:
				equation = "a^14 A^3 - 4 a^12 b^2 A^3 + 6 a^10 b^4 A^3 - 4 a^8 b^6 A^3 + a^6 b^8 A^3 - 4 a^12 c^2 A^3 + 12 a^10 b^2 c^2 A^3 - 12 a^8 b^4 c^2 A^3 + 4 a^6 b^6 c^2 A^3 + 6 a^10 c^4 A^3 - 12 a^8 b^2 c^4 A^3 + 6 a^6 b^4 c^4 A^3 - 4 a^8 c^6 A^3 + 4 a^6 b^2 c^6 A^3 + a^6 c^8 A^3 + 3 a^12 b^2 A^2 B - 12 a^10 b^4 A^2 B + 18 a^8 b^6 A^2 B - 12 a^6 b^8 A^2 B + 3 a^4 b^10 A^2 B - 4 a^10 b^2 c^2 A^2 B + 12 a^8 b^4 c^2 A^2 B - 12 a^6 b^6 c^2 A^2 B + 4 a^4 b^8 c^2 A^2 B - 2 a^8 b^2 c^4 A^2 B + 4 a^6 b^4 c^4 A^2 B - 2 a^4 b^6 c^4 A^2 B + 4 a^6 b^2 c^6 A^2 B - 4 a^4 b^4 c^6 A^2 B - a^4 b^2 c^8 A^2 B + 3 a^10 b^4 A B^2 - 12 a^8 b^6 A B^2 + 18 a^6 b^8 A B^2 - 12 a^4 b^10 A B^2 + 3 a^2 b^12 A B^2 + 4 a^8 b^4 c^2 A B^2 - 12 a^6 b^6 c^2 A B^2 + 12 a^4 b^8 c^2 A B^2 - 4 a^2 b^10 c^2 A B^2 - 2 a^6 b^4 c^4 A B^2 + 4 a^4 b^6 c^4 A B^2 - 2 a^2 b^8 c^4 A B^2 - 4 a^4 b^4 c^6 A B^2 + 4 a^2 b^6 c^6 A B^2 - a^2 b^4 c^8 A B^2 + a^8 b^6 B^3 - 4 a^6 b^8 B^3 + 6 a^4 b^10 B^3 - 4 a^2 b^12 B^3 + b^14 B^3 + 4 a^6 b^6 c^2 B^3 - 12 a^4 b^8 c^2 B^3 + 12 a^2 b^10 c^2 B^3 - 4 b^12 c^2 B^3 + 6 a^4 b^6 c^4 B^3 - 12 a^2 b^8 c^4 B^3 + 6 b^10 c^4 B^3 + 4 a^2 b^6 c^6 B^3 - 4 b^8 c^6 B^3 +b^6 c^8 B^3 + 3 a^12 c^2 A^2 C - 4 a^10 b^2 c^2 A^2 C - 2 a^8 b^4 c^2 A^2 C + 4 a^6 b^6 c^2 A^2 C - a^4 b^8 c^2 A^2 C - 12 a^10 c^4 A^2 C + 12 a^8 b^2 c^4 A^2 C + 4 a^6 b^4 c^4 A^2 C - 4 a^4 b^6 c^4 A^2 C + 18 a^8 c^6 A^2 C - 12 a^6 b^2 c^6 A^2 C - 2 a^4 b^4 c^6 A^2 C - 12 a^6 c^8 A^2 C + 4 a^4 b^2 c^8 A^2 C + 3 a^4 c^10 A^2 C + 3 a^10 b^2 c^2 A B C - 4 a^8 b^4 c^2 A B C + 2 a^6 b^6 c^2 A B C - 4 a^4 b^8 c^2 A B C + 3 a^2 b^10 c^2 A B C - 4 a^8 b^2 c^4 A B C + 4 a^6 b^4 c^4 A B C + 4 a^4 b^6 c^4 A B C - 4 a^2 b^8 c^4 A B C + 2 a^6 b^2 c^6 A B C + 4 a^4 b^4 c^6 A B C + 2 a^2 b^6 c^6 A B C - 4 a^4 b^2 c^8 A B C - 4 a^2 b^4 c^8 A B C + 3 a^2 b^2 c^10 A B C - a^8 b^4 c^2 B^2 C + 4 a^6 b^6 c^2 B^2 C - 2 a^4 b^8 c^2 B^2 C - 4 a^2 b^10 c^2 B^2 C + 3 b^12 c^2 B^2 C - 4 a^6 b^4 c^4 B^2 C + 4 a^4 b^6 c^4 B^2 C + 12 a^2 b^8 c^4 B^2 C - 12 b^10 c^4 B^2 C - 2 a^4 b^4 c^6 B^2 C - 12 a^2 b^6 c^6 B^2 C + 18 b^8 c^6 B^2 C + 4 a^2 b^4 c^8 B^2 C - 12 b^6 c^8 B^2 C + 3 b^4 c^10 B^2 C + 3 a^10 c^4 A C^2 + 4 a^8 b^2 c^4 A C^2 - 2 a^6 b^4 c^4 A C^2 - 4 a^4 b^6 c^4 A C^2 - a^2 b^8 c^4 A C^2 - 12 a^8 c^6 A C^2 - 12 a^6 b^2 c^6 A C^2 + 4 a^4 b^4 c^6 A C^2 + 4 a^2 b^6 c^6 A C^2 + 18 a^6 c^8 A C^2 + 12 a^4 b^2 c^8 A C^2 - 2 a^2 b^4 c^8 A C^2 - 12 a^4 c^10 A C^2 - 4 a^2 b^2 c^10 A C^2 + 3 a^2 c^12 A C^2 - a^8 b^2 c^4 B C^2 - 4 a^6 b^4 c^4 B C^2 - 2 a^4 b^6 c^4 B C^2 + 4 a^2 b^8 c^4 B C^2 + 3 b^10 c^4 B C^2 + 4 a^6 b^2 c^6 B C^2 + 4 a^4 b^4 c^6 B C^2 - 12 a^2 b^6 c^6 B C^2 - 12 b^8 c^6 B C^2 - 2 a^4 b^2 c^8 B C^2 + 12 a^2 b^4 c^8 B C^2 + 18 b^6 c^8 B C^2 - 4 a^2 b^2 c^10 B C^2 - 12 b^4 c^10 B C^2 + 3 b^2 c^12 B C^2 + a^8 c^6 C^3 + 4 a^6 b^2 c^6 C^3 + 6 a^4 b^4 c^6 C^3 + 4 a^2 b^6 c^6 C^3 + b^8 c^6 C^3 - 4 a^6 c^8 C^3 - 12 a^4 b^2 c^8 C^3 - 12 a^2 b^4 c^8 C^3 - 4 b^6 c^8 C^3 + 6 a^4 c^10 C^3 + 12 a^2 b^2 c^10 C^3 + 6 b^4 c^10 C^3 - 4 a^2 c^12 C^3 - 4 b^2 c^12 C^3 + c^14 C^3 = 0";
				break;
			
			
			case 589:
				equation = "a^8*b^4*A^2*B - 4*a^6*b^6*A^2*B + 6*a^4*b^8*A^2*B - 4*a^2*b^10*A^2*B + b^12*A^2*B - a^8*b^2*c^2*A^2*B + 3*a^6*b^4*c^2*A^2*B - 5*a^4*b^6*c^2*A^2*B + 5*a^2*b^8*c^2*A^2*B - 2*b^10*c^2*A^2*B + a^6*b^2*c^4*A^2*B - a^4*b^4*c^4*A^2*B - a^2*b^6*c^4*A^2*B + b^8*c^4*A^2*B + a^12*A*B^2 - 4*a^10*b^2*A*B^2 + 6*a^8*b^4*A*B^2 - 4*a^6*b^6*A*B^2 + a^4*b^8*A*B^2 - 2*a^10*c^2*A*B^2 + 5*a^8*b^2*c^2*A*B^2 - 5*a^6*b^4*c^2*A*B^2 + 3*a^4*b^6*c^2*A*B^2 - a^2*b^8*c^2*A*B^2 + a^8*c^4*A*B^2 - a^6*b^2*c^4*A*B^2 - a^4*b^4*c^4*A*B^2 + a^2*b^6*c^4*A*B^2 - a^8*b^2*c^2*A^2*C + a^6*b^4*c^2*A^2*C + a^8*c^4*A^2*C + 3*a^6*b^2*c^4*A^2*C - a^4*b^4*c^4*A^2*C - 4*a^6*c^6*A^2*C - 5*a^4*b^2*c^6*A^2*C - a^2*b^4*c^6*A^2*C + 6*a^4*c^8*A^2*C + 5*a^2*b^2*c^8*A^2*C + b^4*c^8*A^2*C - 4*a^2*c^10*A^2*C - 2*b^2*c^10*A^2*C + c^12*A^2*C - 2*a^12*A*B*C + 6*a^10*b^2*A*B*C - 6*a^8*b^4*A*B*C + 4*a^6*b^6*A*B*C - 6*a^4*b^8*A*B*C + 6*a^2*b^10*A*B*C - 2*b^12*A*B*C + 6*a^10*c^2*A*B*C - 12*a^8*b^2*c^2*A*B*C + 6*a^6*b^4*c^2*A*B*C + 6*a^4*b^6*c^2*A*B*C - 12*a^2*b^8*c^2*A*B*C + 6*b^10*c^2*A*B*C - 6*a^8*c^4*A*B*C + 6*a^6*b^2*c^4*A*B*C - 6*a^4*b^4*c^4*A*B*C + 6*a^2*b^6*c^4*A*B*C - 6*b^8*c^4*A*B*C + 4*a^6*c^6*A*B*C + 6*a^4*b^2*c^6*A*B*C + 6*a^2*b^4*c^6*A*B*C + 4*b^6*c^6*A*B*C - 6*a^4*c^8*A*B*C - 12*a^2*b^2*c^8*A*B*C - 6*b^4*c^8*A*B*C + 6*a^2*c^10*A*B*C + 6*b^2*c^10*A*B*C - 2*c^12*A*B*C + a^4*b^6*c^2*B^2*C - a^2*b^8*c^2*B^2*C - a^4*b^4*c^4*B^2*C + 3*a^2*b^6*c^4*B^2*C + b^8*c^4*B^2*C - a^4*b^2*c^6*B^2*C - 5*a^2*b^4*c^6*B^2*C - 4*b^6*c^6*B^2*C + a^4*c^8*B^2*C + 5*a^2*b^2*c^8*B^2*C + 6*b^4*c^8*B^2*C - 2*a^2*c^10*B^2*C - 4*b^2*c^10*B^2*C + c^12*B^2*C + a^12*A*C^2 - 2*a^10*b^2*A*C^2 + a^8*b^4*A*C^2 - 4*a^10*c^2*A*C^2 + 5*a^8*b^2*c^2*A*C^2 - a^6*b^4*c^2*A*C^2 + 6*a^8*c^4*A*C^2 - 5*a^6*b^2*c^4*A*C^2 - a^4*b^4*c^4*A*C^2 - 4*a^6*c^6*A*C^2 + 3*a^4*b^2*c^6*A*C^2 + a^2*b^4*c^6*A*C^2 + a^4*c^8*A*C^2 - a^2*b^2*c^8*A*C^2 + a^4*b^8*B*C^2 - 2*a^2*b^10*B*C^2 + b^12*B*C^2 - a^4*b^6*c^2*B*C^2 + 5*a^2*b^8*c^2*B*C^2 - 4*b^10*c^2*B*C^2 - a^4*b^4*c^4*B*C^2 - 5*a^2*b^6*c^4*B*C^2 + 6*b^8*c^4*B*C^2 + a^4*b^2*c^6*B*C^2 + 3*a^2*b^4*c^6*B*C^2 - 4*b^6*c^6*B*C^2 - a^2*b^2*c^8*B*C^2 + b^4*c^8*B*C^2 = 0";
				break;
			case 590:
				equation = "a^4*b^10*c^4*A^2*B - 2*a^2*b^12*c^4*A^2*B + b^14*c^4*A^2*B - a^4*b^8*c^6*A^2*B + 5*a^2*b^10*c^6*A^2*B - 4*b^12*c^6*A^2*B - a^4*b^6*c^8*A^2*B - 5*a^2*b^8*c^8*A^2*B + 6*b^10*c^8*A^2*B + a^4*b^4*c^10*A^2*B + 3*a^2*b^6*c^10*A^2*B - 4*b^8*c^10*A^2*B - a^2*b^4*c^12*A^2*B + b^6*c^12*A^2*B + a^14*c^4*A*B^2 - 2*a^12*b^2*c^4*A*B^2 + a^10*b^4*c^4*A*B^2 - 4*a^12*c^6*A*B^2 + 5*a^10*b^2*c^6*A*B^2 - a^8*b^4*c^6*A*B^2 + 6*a^10*c^8*A*B^2 - 5*a^8*b^2*c^8*A*B^2 - a^6*b^4*c^8*A*B^2 - 4*a^8*c^10*A*B^2 + 3*a^6*b^2*c^10*A*B^2 + a^4*b^4*c^10*A*B^2 + a^6*c^12*A*B^2 - a^4*b^2*c^12*A*B^2 + a^4*b^10*c^4*A^2*C - a^2*b^12*c^4*A^2*C - a^4*b^8*c^6*A^2*C + 3*a^2*b^10*c^6*A^2*C + b^12*c^6*A^2*C - a^4*b^6*c^8*A^2*C - 5*a^2*b^8*c^8*A^2*C - 4*b^10*c^8*A^2*C + a^4*b^4*c^10*A^2*C + 5*a^2*b^6*c^10*A^2*C + 6*b^8*c^10*A^2*C - 2*a^2*b^4*c^12*A^2*C - 4*b^6*c^12*A^2*C + b^4*c^14*A^2*C - 2*a^14*b^2*c^2*A*B*C + 6*a^12*b^4*c^2*A*B*C - 6*a^10*b^6*c^2*A*B*C + 4*a^8*b^8*c^2*A*B*C - 6*a^6*b^10*c^2*A*B*C + 6*a^4*b^12*c^2*A*B*C - 2*a^2*b^14*c^2*A*B*C + 6*a^12*b^2*c^4*A*B*C - 12*a^10*b^4*c^4*A*B*C + 6*a^8*b^6*c^4*A*B*C + 6*a^6*b^8*c^4*A*B*C - 12*a^4*b^10*c^4*A*B*C + 6*a^2*b^12*c^4*A*B*C - 6*a^10*b^2*c^6*A*B*C + 6*a^8*b^4*c^6*A*B*C - 6*a^6*b^6*c^6*A*B*C + 6*a^4*b^8*c^6*A*B*C - 6*a^2*b^10*c^6*A*B*C + 4*a^8*b^2*c^8*A*B*C + 6*a^6*b^4*c^8*A*B*C + 6*a^4*b^6*c^8*A*B*C + 4*a^2*b^8*c^8*A*B*C - 6*a^6*b^2*c^10*A*B*C - 12*a^4*b^4*c^10*A*B*C - 6*a^2*b^6*c^10*A*B*C + 6*a^4*b^2*c^12*A*B*C + 6*a^2*b^4*c^12*A*B*C - 2*a^2*b^2*c^14*A*B*C - a^12*b^2*c^4*B^2*C + a^10*b^4*c^4*B^2*C + a^12*c^6*B^2*C + 3*a^10*b^2*c^6*B^2*C - a^8*b^4*c^6*B^2*C - 4*a^10*c^8*B^2*C - 5*a^8*b^2*c^8*B^2*C - a^6*b^4*c^8*B^2*C + 6*a^8*c^10*B^2*C + 5*a^6*b^2*c^10*B^2*C + a^4*b^4*c^10*B^2*C - 4*a^6*c^12*B^2*C - 2*a^4*b^2*c^12*B^2*C + a^4*c^14*B^2*C + a^14*b^4*A*C^2 - 4*a^12*b^6*A*C^2 + 6*a^10*b^8*A*C^2 - 4*a^8*b^10*A*C^2 + a^6*b^12*A*C^2 - 2*a^12*b^4*c^2*A*C^2 + 5*a^10*b^6*c^2*A*C^2 - 5*a^8*b^8*c^2*A*C^2 + 3*a^6*b^10*c^2*A*C^2 - a^4*b^12*c^2*A*C^2 + a^10*b^4*c^4*A*C^2 - a^8*b^6*c^4*A*C^2 - a^6*b^8*c^4*A*C^2 + a^4*b^10*c^4*A*C^2 + a^12*b^6*B*C^2 - 4*a^10*b^8*B*C^2 + 6*a^8*b^10*B*C^2 - 4*a^6*b^12*B*C^2 + a^4*b^14*B*C^2 - a^12*b^4*c^2*B*C^2 + 3*a^10*b^6*c^2*B*C^2 - 5*a^8*b^8*c^2*B*C^2 + 5*a^6*b^10*c^2*B*C^2 - 2*a^4*b^12*c^2*B*C^2 + a^10*b^4*c^4*B*C^2 - a^8*b^6*c^4*B*C^2 - a^6*b^8*c^4*B*C^2 + a^4*b^10*c^4*B*C^2 = 0";
				break;
			case 591:
				equation = "-a^4*b^2*A^3 + 2*a^2*b^4*A^3 - b^6*A^3 + a^4*c^2*A^3 - b^4*c^2*A^3 - 2*a^2*c^4*A^3 + b^2*c^4*A^3 + c^6*A^3 - a^4*b^2*A^2*B + 2*a^2*b^4*A^2*B - b^6*A^2*B + a^4*c^2*A^2*B - 2*a^2*b^2*c^2*A^2*B + b^4*c^2*A^2*B + b^2*c^4*A^2*B - c^6*A^2*B + a^6*A*B^2 - 2*a^4*b^2*A*B^2 + a^2*b^4*A*B^2 - a^4*c^2*A*B^2 + 2*a^2*b^2*c^2*A*B^2 - b^4*c^2*A*B^2 - a^2*c^4*A*B^2 + c^6*A*B^2 + a^6*B^3 - 2*a^4*b^2*B^3 + a^2*b^4*B^3 + a^4*c^2*B^3 - b^4*c^2*B^3 - a^2*c^4*B^3 + 2*b^2*c^4*B^3 - c^6*B^3 - a^4*b^2*A^2*C + b^6*A^2*C + a^4*c^2*A^2*C + 2*a^2*b^2*c^2*A^2*C - b^4*c^2*A^2*C - 2*a^2*c^4*A^2*C - b^2*c^4*A^2*C + c^6*A^2*C - 4*a^4*b^2*A*B*C + 4*a^2*b^4*A*B*C + 4*a^4*c^2*A*B*C - 4*b^4*c^2*A*B*C - 4*a^2*c^4*A*B*C + 4*b^2*c^4*A*B*C - a^6*B^2*C + a^2*b^4*B^2*C + a^4*c^2*B^2*C - 2*a^2*b^2*c^2*B^2*C - b^4*c^2*B^2*C + a^2*c^4*B^2*C + 2*b^2*c^4*B^2*C - c^6*B^2*C - a^6*A*C^2 + a^4*b^2*A*C^2 + a^2*b^4*A*C^2 - b^6*A*C^2 + 2*a^4*c^2*A*C^2 - 2*a^2*b^2*c^2*A*C^2 - a^2*c^4*A*C^2 + b^2*c^4*A*C^2 + a^6*B*C^2 - a^4*b^2*B*C^2 - a^2*b^4*B*C^2 + b^6*B*C^2 + 2*a^2*b^2*c^2*B*C^2 - 2*b^4*c^2*B*C^2 - a^2*c^4*B*C^2 + b^2*c^4*B*C^2 - a^6*C^3 - a^4*b^2*C^3 + a^2*b^4*C^3 + b^6*C^3 + 2*a^4*c^2*C^3 - 2*b^4*c^2*C^3 - a^2*c^4*C^3 + b^2*c^4*C^3 = 0";
				break;
			case 592:
				equation = "a^12*b^6*A^3 - 4*a^10*b^8*A^3 + 5*a^8*b^10*A^3 - 5*a^4*b^14*A^3 + 4*a^2*b^16*A^3 - b^18*A^3 - 3*a^12*b^4*c^2*A^3 + 8*a^10*b^6*c^2*A^3 - 3*a^8*b^8*c^2*A^3 - 12*a^6*b^10*c^2*A^3 + 19*a^4*b^12*c^2*A^3 - 12*a^2*b^14*c^2*A^3 + 3*b^16*c^2*A^3 + 3*a^12*b^2*c^4*A^3 - 16*a^8*b^6*c^4*A^3 + 24*a^6*b^8*c^4*A^3 - 16*a^4*b^10*c^4*A^3 + 8*a^2*b^12*c^4*A^3 - 3*b^14*c^4*A^3 - a^12*c^6*A^3 - 8*a^10*b^2*c^6*A^3 + 16*a^8*b^4*c^6*A^3 - 12*a^4*b^8*c^6*A^3 + 4*a^2*b^10*c^6*A^3 + 3*b^12*c^6*A^3 + 4*a^10*c^8*A^3 + 3*a^8*b^2*c^8*A^3 - 24*a^6*b^4*c^8*A^3 + 12*a^4*b^6*c^8*A^3 - 6*b^10*c^8*A^3 - 5*a^8*c^10*A^3 + 12*a^6*b^2*c^10*A^3 + 16*a^4*b^4*c^10*A^3 - 4*a^2*b^6*c^10*A^3 + 6*b^8*c^10*A^3 - 19*a^4*b^2*c^12*A^3 - 8*a^2*b^4*c^12*A^3 - 3*b^6*c^12*A^3 + 5*a^4*c^14*A^3 + 12*a^2*b^2*c^14*A^3 + 3*b^4*c^14*A^3 - 4*a^2*c^16*A^3 - 3*b^2*c^16*A^3 + c^18*A^3 + 3*a^14*b^4*A^2*B - 12*a^12*b^6*A^2*B + 15*a^10*b^8*A^2*B - 15*a^6*b^12*A^2*B + 12*a^4*b^14*A^2*B - 3*a^2*b^16*A^2*B - 6*a^14*b^2*c^2*A^2*B + 12*a^12*b^4*c^2*A^2*B + 11*a^10*b^6*c^2*A^2*B - 53*a^8*b^8*c^2*A^2*B + 60*a^6*b^10*c^2*A^2*B - 26*a^4*b^12*c^2*A^2*B - a^2*b^14*c^2*A^2*B + 3*b^16*c^2*A^2*B + 3*a^14*c^4*A^2*B + 12*a^12*b^2*c^4*A^2*B - 51*a^10*b^4*c^4*A^2*B + 62*a^8*b^6*c^4*A^2*B - 23*a^6*b^8*c^4*A^2*B - 19*a^4*b^10*c^4*A^2*B + 27*a^2*b^12*c^4*A^2*B - 11*b^14*c^4*A^2*B - 12*a^12*c^6*A^2*B + 9*a^10*b^2*c^6*A^2*B + 30*a^8*b^4*c^6*A^2*B - 67*a^6*b^6*c^6*A^2*B + 60*a^4*b^8*c^6*A^2*B - 32*a^2*b^10*c^6*A^2*B + 14*b^12*c^6*A^2*B + 16*a^10*c^8*A^2*B - 34*a^8*b^2*c^8*A^2*B + 33*a^6*b^4*c^8*A^2*B - 6*a^4*b^6*c^8*A^2*B - 9*a^2*b^8*c^8*A^2*B - 9*b^10*c^8*A^2*B - 5*a^8*c^10*A^2*B + 17*a^6*b^2*c^10*A^2*B - 32*a^4*b^4*c^10*A^2*B + 25*a^2*b^6*c^10*A^2*B + 10*b^8*c^10*A^2*B - 5*a^6*c^12*A^2*B + 9*a^4*b^2*c^12*A^2*B - a^2*b^4*c^12*A^2*B - 13*b^6*c^12*A^2*B + 2*a^4*c^14*A^2*B - 8*a^2*b^2*c^14*A^2*B + 6*b^4*c^14*A^2*B + 2*a^2*c^16*A^2*B + b^2*c^16*A^2*B - c^18*A^2*B + 3*a^16*b^2*A*B^2 - 12*a^14*b^4*A*B^2 + 15*a^12*b^6*A*B^2 - 15*a^8*b^10*A*B^2 + 12*a^6*b^12*A*B^2 - 3*a^4*b^14*A*B^2 - 3*a^16*c^2*A*B^2 + a^14*b^2*c^2*A*B^2 + 26*a^12*b^4*c^2*A*B^2 - 60*a^10*b^6*c^2*A*B^2 + 53*a^8*b^8*c^2*A*B^2 - 11*a^6*b^10*c^2*A*B^2 - 12*a^4*b^12*c^2*A*B^2 + 6*a^2*b^14*c^2*A*B^2 + 11*a^14*c^4*A*B^2 - 27*a^12*b^2*c^4*A*B^2 + 19*a^10*b^4*c^4*A*B^2 + 23*a^8*b^6*c^4*A*B^2 - 62*a^6*b^8*c^4*A*B^2 + 51*a^4*b^10*c^4*A*B^2 - 12*a^2*b^12*c^4*A*B^2 - 3*b^14*c^4*A*B^2 - 14*a^12*c^6*A*B^2 + 32*a^10*b^2*c^6*A*B^2 - 60*a^8*b^4*c^6*A*B^2 + 67*a^6*b^6*c^6*A*B^2 - 30*a^4*b^8*c^6*A*B^2 - 9*a^2*b^10*c^6*A*B^2 + 12*b^12*c^6*A*B^2 + 9*a^10*c^8*A*B^2 + 9*a^8*b^2*c^8*A*B^2 + 6*a^6*b^4*c^8*A*B^2 - 33*a^4*b^6*c^8*A*B^2 + 34*a^2*b^8*c^8*A*B^2 - 16*b^10*c^8*A*B^2 - 10*a^8*c^10*A*B^2 - 25*a^6*b^2*c^10*A*B^2 + 32*a^4*b^4*c^10*A*B^2 - 17*a^2*b^6*c^10*A*B^2 + 5*b^8*c^10*A*B^2 + 13*a^6*c^12*A*B^2 + a^4*b^2*c^12*A*B^2 - 9*a^2*b^4*c^12*A*B^2 + 5*b^6*c^12*A*B^2 - 6*a^4*c^14*A*B^2 + 8*a^2*b^2*c^14*A*B^2 - 2*b^4*c^14*A*B^2 - a^2*c^16*A*B^2 - 2*b^2*c^16*A*B^2 + c^18*A*B^2 + a^18*B^3 - 4*a^16*b^2*B^3 + 5*a^14*b^4*B^3 - 5*a^10*b^8*B^3 + 4*a^8*b^10*B^3 - a^6*b^12*B^3 - 3*a^16*c^2*B^3 + 12*a^14*b^2*c^2*B^3 - 19*a^12*b^4*c^2*B^3 + 12*a^10*b^6*c^2*B^3 + 3*a^8*b^8*c^2*B^3 - 8*a^6*b^10*c^2*B^3 + 3*a^4*b^12*c^2*B^3 + 3*a^14*c^4*B^3 - 8*a^12*b^2*c^4*B^3 + 16*a^10*b^4*c^4*B^3 - 24*a^8*b^6*c^4*B^3 + 16*a^6*b^8*c^4*B^3 - 3*a^2*b^12*c^4*B^3 - 3*a^12*c^6*B^3 - 4*a^10*b^2*c^6*B^3 + 12*a^8*b^4*c^6*B^3 - 16*a^4*b^8*c^6*B^3 + 8*a^2*b^10*c^6*B^3 + b^12*c^6*B^3 + 6*a^10*c^8*B^3 - 12*a^6*b^4*c^8*B^3 + 24*a^4*b^6*c^8*B^3 - 3*a^2*b^8*c^8*B^3 - 4*b^10*c^8*B^3 - 6*a^8*c^10*B^3 + 4*a^6*b^2*c^10*B^3 - 16*a^4*b^4*c^10*B^3 - 12*a^2*b^6*c^10*B^3 + 5*b^8*c^10*B^3 + 3*a^6*c^12*B^3 + 8*a^4*b^2*c^12*B^3 + 19*a^2*b^4*c^12*B^3 - 3*a^4*c^14*B^3 - 12*a^2*b^2*c^14*B^3 - 5*b^4*c^14*B^3 + 3*a^2*c^16*B^3 + 4*b^2*c^16*B^3 - c^18*B^3 - 3*a^14*b^4*A^2*C + 12*a^12*b^6*A^2*C - 16*a^10*b^8*A^2*C + 5*a^8*b^10*A^2*C + 5*a^6*b^12*A^2*C - 2*a^4*b^14*A^2*C - 2*a^2*b^16*A^2*C + b^18*A^2*C + 6*a^14*b^2*c^2*A^2*C - 12*a^12*b^4*c^2*A^2*C - 9*a^10*b^6*c^2*A^2*C + 34*a^8*b^8*c^2*A^2*C - 17*a^6*b^10*c^2*A^2*C - 9*a^4*b^12*c^2*A^2*C + 8*a^2*b^14*c^2*A^2*C - b^16*c^2*A^2*C - 3*a^14*c^4*A^2*C - 12*a^12*b^2*c^4*A^2*C + 51*a^10*b^4*c^4*A^2*C - 30*a^8*b^6*c^4*A^2*C - 33*a^6*b^8*c^4*A^2*C + 32*a^4*b^10*c^4*A^2*C + a^2*b^12*c^4*A^2*C - 6*b^14*c^4*A^2*C + 12*a^12*c^6*A^2*C - 11*a^10*b^2*c^6*A^2*C - 62*a^8*b^4*c^6*A^2*C + 67*a^6*b^6*c^6*A^2*C + 6*a^4*b^8*c^6*A^2*C - 25*a^2*b^10*c^6*A^2*C + 13*b^12*c^6*A^2*C - 15*a^10*c^8*A^2*C + 53*a^8*b^2*c^8*A^2*C + 23*a^6*b^4*c^8*A^2*C - 60*a^4*b^6*c^8*A^2*C + 9*a^2*b^8*c^8*A^2*C - 10*b^10*c^8*A^2*C - 60*a^6*b^2*c^10*A^2*C + 19*a^4*b^4*c^10*A^2*C + 32*a^2*b^6*c^10*A^2*C + 9*b^8*c^10*A^2*C + 15*a^6*c^12*A^2*C + 26*a^4*b^2*c^12*A^2*C - 27*a^2*b^4*c^12*A^2*C - 14*b^6*c^12*A^2*C - 12*a^4*c^14*A^2*C + a^2*b^2*c^14*A^2*C + 11*b^4*c^14*A^2*C + 3*a^2*c^16*A^2*C - 3*b^2*c^16*A^2*C - 3*a^16*b^2*A*B*C + 7*a^14*b^4*A*B*C + 7*a^12*b^6*A*B*C - 35*a^10*b^8*A*B*C + 35*a^8*b^10*A*B*C - 7*a^6*b^12*A*B*C - 7*a^4*b^14*A*B*C + 3*a^2*b^16*A*B*C + 3*a^16*c^2*A*B*C - 31*a^12*b^4*c^2*A*B*C + 50*a^10*b^6*c^2*A*B*C - 50*a^6*b^10*c^2*A*B*C + 31*a^4*b^12*c^2*A*B*C - 3*b^16*c^2*A*B*C - 7*a^14*c^4*A*B*C + 31*a^12*b^2*c^4*A*B*C - 90*a^8*b^6*c^4*A*B*C + 90*a^6*b^8*c^4*A*B*C - 31*a^2*b^12*c^4*A*B*C + 7*b^14*c^4*A*B*C - 7*a^12*c^6*A*B*C - 50*a^10*b^2*c^6*A*B*C + 90*a^8*b^4*c^6*A*B*C - 90*a^4*b^8*c^6*A*B*C + 50*a^2*b^10*c^6*A*B*C + 7*b^12*c^6*A*B*C + 35*a^10*c^8*A*B*C - 90*a^6*b^4*c^8*A*B*C + 90*a^4*b^6*c^8*A*B*C - 35*b^10*c^8*A*B*C - 35*a^8*c^10*A*B*C + 50*a^6*b^2*c^10*A*B*C - 50*a^2*b^6*c^10*A*B*C + 35*b^8*c^10*A*B*C + 7*a^6*c^12*A*B*C - 31*a^4*b^2*c^12*A*B*C + 31*a^2*b^4*c^12*A*B*C - 7*b^6*c^12*A*B*C + 7*a^4*c^14*A*B*C - 7*b^4*c^14*A*B*C - 3*a^2*c^16*A*B*C + 3*b^2*c^16*A*B*C - a^18*B^2*C + 2*a^16*b^2*B^2*C + 2*a^14*b^4*B^2*C - 5*a^12*b^6*B^2*C - 5*a^10*b^8*B^2*C + 16*a^8*b^10*B^2*C - 12*a^6*b^12*B^2*C + 3*a^4*b^14*B^2*C + a^16*c^2*B^2*C - 8*a^14*b^2*c^2*B^2*C + 9*a^12*b^4*c^2*B^2*C + 17*a^10*b^6*c^2*B^2*C - 34*a^8*b^8*c^2*B^2*C + 9*a^6*b^10*c^2*B^2*C + 12*a^4*b^12*c^2*B^2*C - 6*a^2*b^14*c^2*B^2*C + 6*a^14*c^4*B^2*C - a^12*b^2*c^4*B^2*C - 32*a^10*b^4*c^4*B^2*C + 33*a^8*b^6*c^4*B^2*C + 30*a^6*b^8*c^4*B^2*C - 51*a^4*b^10*c^4*B^2*C + 12*a^2*b^12*c^4*B^2*C + 3*b^14*c^4*B^2*C - 13*a^12*c^6*B^2*C + 25*a^10*b^2*c^6*B^2*C - 6*a^8*b^4*c^6*B^2*C - 67*a^6*b^6*c^6*B^2*C + 62*a^4*b^8*c^6*B^2*C + 11*a^2*b^10*c^6*B^2*C - 12*b^12*c^6*B^2*C + 10*a^10*c^8*B^2*C - 9*a^8*b^2*c^8*B^2*C + 60*a^6*b^4*c^8*B^2*C - 23*a^4*b^6*c^8*B^2*C - 53*a^2*b^8*c^8*B^2*C + 15*b^10*c^8*B^2*C - 9*a^8*c^10*B^2*C - 32*a^6*b^2*c^10*B^2*C - 19*a^4*b^4*c^10*B^2*C + 60*a^2*b^6*c^10*B^2*C + 14*a^6*c^12*B^2*C + 27*a^4*b^2*c^12*B^2*C - 26*a^2*b^4*c^12*B^2*C - 15*b^6*c^12*B^2*C - 11*a^4*c^14*B^2*C - a^2*b^2*c^14*B^2*C + 12*b^4*c^14*B^2*C + 3*a^2*c^16*B^2*C - 3*b^2*c^16*B^2*C + 3*a^16*b^2*A*C^2 - 11*a^14*b^4*A*C^2 + 14*a^12*b^6*A*C^2 - 9*a^10*b^8*A*C^2 + 10*a^8*b^10*A*C^2 - 13*a^6*b^12*A*C^2 + 6*a^4*b^14*A*C^2 + a^2*b^16*A*C^2 - b^18*A*C^2 - 3*a^16*c^2*A*C^2 - a^14*b^2*c^2*A*C^2 + 27*a^12*b^4*c^2*A*C^2 - 32*a^10*b^6*c^2*A*C^2 - 9*a^8*b^8*c^2*A*C^2 + 25*a^6*b^10*c^2*A*C^2 - a^4*b^12*c^2*A*C^2 - 8*a^2*b^14*c^2*A*C^2 + 2*b^16*c^2*A*C^2 + 12*a^14*c^4*A*C^2 - 26*a^12*b^2*c^4*A*C^2 - 19*a^10*b^4*c^4*A*C^2 + 60*a^8*b^6*c^4*A*C^2 - 6*a^6*b^8*c^4*A*C^2 - 32*a^4*b^10*c^4*A*C^2 + 9*a^2*b^12*c^4*A*C^2 + 2*b^14*c^4*A*C^2 - 15*a^12*c^6*A*C^2 + 60*a^10*b^2*c^6*A*C^2 - 23*a^8*b^4*c^6*A*C^2 - 67*a^6*b^6*c^6*A*C^2 + 33*a^4*b^8*c^6*A*C^2 + 17*a^2*b^10*c^6*A*C^2 - 5*b^12*c^6*A*C^2 - 53*a^8*b^2*c^8*A*C^2 + 62*a^6*b^4*c^8*A*C^2 + 30*a^4*b^6*c^8*A*C^2 - 34*a^2*b^8*c^8*A*C^2 - 5*b^10*c^8*A*C^2 + 15*a^8*c^10*A*C^2 + 11*a^6*b^2*c^10*A*C^2 - 51*a^4*b^4*c^10*A*C^2 + 9*a^2*b^6*c^10*A*C^2 + 16*b^8*c^10*A*C^2 - 12*a^6*c^12*A*C^2 + 12*a^4*b^2*c^12*A*C^2 + 12*a^2*b^4*c^12*A*C^2 - 12*b^6*c^12*A*C^2 + 3*a^4*c^14*A*C^2 - 6*a^2*b^2*c^14*A*C^2 + 3*b^4*c^14*A*C^2 + a^18*B*C^2 - a^16*b^2*B*C^2 - 6*a^14*b^4*B*C^2 + 13*a^12*b^6*B*C^2 - 10*a^10*b^8*B*C^2 + 9*a^8*b^10*B*C^2 - 14*a^6*b^12*B*C^2 + 11*a^4*b^14*B*C^2 - 3*a^2*b^16*B*C^2 - 2*a^16*c^2*B*C^2 + 8*a^14*b^2*c^2*B*C^2 + a^12*b^4*c^2*B*C^2 - 25*a^10*b^6*c^2*B*C^2 + 9*a^8*b^8*c^2*B*C^2 + 32*a^6*b^10*c^2*B*C^2 - 27*a^4*b^12*c^2*B*C^2 + a^2*b^14*c^2*B*C^2 + 3*b^16*c^2*B*C^2 - 2*a^14*c^4*B*C^2 - 9*a^12*b^2*c^4*B*C^2 + 32*a^10*b^4*c^4*B*C^2 + 6*a^8*b^6*c^4*B*C^2 - 60*a^6*b^8*c^4*B*C^2 + 19*a^4*b^10*c^4*B*C^2 + 26*a^2*b^12*c^4*B*C^2 - 12*b^14*c^4*B*C^2 + 5*a^12*c^6*B*C^2 - 17*a^10*b^2*c^6*B*C^2 - 33*a^8*b^4*c^6*B*C^2 + 67*a^6*b^6*c^6*B*C^2 + 23*a^4*b^8*c^6*B*C^2 - 60*a^2*b^10*c^6*B*C^2 + 15*b^12*c^6*B*C^2 + 5*a^10*c^8*B*C^2 + 34*a^8*b^2*c^8*B*C^2 - 30*a^6*b^4*c^8*B*C^2 - 62*a^4*b^6*c^8*B*C^2 + 53*a^2*b^8*c^8*B*C^2 - 16*a^8*c^10*B*C^2 - 9*a^6*b^2*c^10*B*C^2 + 51*a^4*b^4*c^10*B*C^2 - 11*a^2*b^6*c^10*B*C^2 - 15*b^8*c^10*B*C^2 + 12*a^6*c^12*B*C^2 - 12*a^4*b^2*c^12*B*C^2 - 12*a^2*b^4*c^12*B*C^2 + 12*b^6*c^12*B*C^2 - 3*a^4*c^14*B*C^2 + 6*a^2*b^2*c^14*B*C^2 - 3*b^4*c^14*B*C^2 - a^18*C^3 + 3*a^16*b^2*C^3 - 3*a^14*b^4*C^3 + 3*a^12*b^6*C^3 - 6*a^10*b^8*C^3 + 6*a^8*b^10*C^3 - 3*a^6*b^12*C^3 + 3*a^4*b^14*C^3 - 3*a^2*b^16*C^3 + b^18*C^3 + 4*a^16*c^2*C^3 - 12*a^14*b^2*c^2*C^3 + 8*a^12*b^4*c^2*C^3 + 4*a^10*b^6*c^2*C^3 - 4*a^6*b^10*c^2*C^3 - 8*a^4*b^12*c^2*C^3 + 12*a^2*b^14*c^2*C^3 - 4*b^16*c^2*C^3 - 5*a^14*c^4*C^3 + 19*a^12*b^2*c^4*C^3 - 16*a^10*b^4*c^4*C^3 - 12*a^8*b^6*c^4*C^3 + 12*a^6*b^8*c^4*C^3 + 16*a^4*b^10*c^4*C^3 - 19*a^2*b^12*c^4*C^3 + 5*b^14*c^4*C^3 - 12*a^10*b^2*c^6*C^3 + 24*a^8*b^4*c^6*C^3 - 24*a^4*b^8*c^6*C^3 + 12*a^2*b^10*c^6*C^3 + 5*a^10*c^8*C^3 - 3*a^8*b^2*c^8*C^3 - 16*a^6*b^4*c^8*C^3 + 16*a^4*b^6*c^8*C^3 + 3*a^2*b^8*c^8*C^3 - 5*b^10*c^8*C^3 - 4*a^8*c^10*C^3 + 8*a^6*b^2*c^10*C^3 - 8*a^2*b^6*c^10*C^3 + 4*b^8*c^10*C^3 + a^6*c^12*C^3 - 3*a^4*b^2*c^12*C^3 + 3*a^2*b^4*c^12*C^3 - b^6*c^12*C^3 = 0";
				break;
			case 593:
				equation = "a^8*b^6*A^3 + 5*a^6*b^8*A^3 - 9*a^4*b^10*A^3 + 7*a^2*b^12*A^3 - 2*b^14*A^3 + 3*a^8*b^4*c^2*A^3 - 10*a^6*b^6*c^2*A^3 + 12*a^4*b^8*c^2*A^3 - 8*a^2*b^10*c^2*A^3 + 3*b^12*c^2*A^3 - 3*a^8*b^2*c^4*A^3 + 9*a^4*b^6*c^4*A^3 - 5*a^2*b^8*c^4*A^3 - 3*b^10*c^4*A^3 + a^8*c^6*A^3 + 10*a^6*b^2*c^6*A^3 - 9*a^4*b^4*c^6*A^3 + 8*b^8*c^6*A^3 - 5*a^6*c^8*A^3 - 12*a^4*b^2*c^8*A^3 + 5*a^2*b^4*c^8*A^3 - 8*b^6*c^8*A^3 + 9*a^4*c^10*A^3 + 8*a^2*b^2*c^10*A^3 + 3*b^4*c^10*A^3 - 7*a^2*c^12*A^3 - 3*b^2*c^12*A^3 + 2*c^14*A^3 + 3*a^8*b^6*A^2*B - 9*a^6*b^8*A^2*B + 9*a^4*b^10*A^2*B - 3*a^2*b^12*A^2*B - 9*a^8*b^4*c^2*A^2*B + 20*a^6*b^6*c^2*A^2*B - 16*a^4*b^8*c^2*A^2*B + 2*a^2*b^10*c^2*A^2*B + 3*b^12*c^2*A^2*B + 9*a^8*b^2*c^4*A^2*B - 6*a^6*b^4*c^4*A^2*B - 3*a^4*b^6*c^4*A^2*B + 9*a^2*b^8*c^4*A^2*B - 11*b^10*c^4*A^2*B - 3*a^8*c^6*A^2*B - 12*a^6*b^2*c^6*A^2*B + 15*a^4*b^4*c^6*A^2*B - 6*a^2*b^6*c^6*A^2*B + 16*b^8*c^6*A^2*B + 7*a^6*c^8*A^2*B - 2*a^4*b^2*c^8*A^2*B - 11*a^2*b^4*c^8*A^2*B - 14*b^6*c^8*A^2*B - 3*a^4*c^10*A^2*B + 12*a^2*b^2*c^10*A^2*B + 11*b^4*c^10*A^2*B - 3*a^2*c^12*A^2*B - 7*b^2*c^12*A^2*B + 2*c^14*A^2*B + 3*a^12*b^2*A*B^2 - 9*a^10*b^4*A*B^2 + 9*a^8*b^6*A*B^2 - 3*a^6*b^8*A*B^2 - 3*a^12*c^2*A*B^2 - 2*a^10*b^2*c^2*A*B^2 + 16*a^8*b^4*c^2*A*B^2 - 20*a^6*b^6*c^2*A*B^2 + 9*a^4*b^8*c^2*A*B^2 + 11*a^10*c^4*A*B^2 - 9*a^8*b^2*c^4*A*B^2 + 3*a^6*b^4*c^4*A*B^2 + 6*a^4*b^6*c^4*A*B^2 - 9*a^2*b^8*c^4*A*B^2 - 16*a^8*c^6*A*B^2 + 6*a^6*b^2*c^6*A*B^2 - 15*a^4*b^4*c^6*A*B^2 + 12*a^2*b^6*c^6*A*B^2 + 3*b^8*c^6*A*B^2 + 14*a^6*c^8*A*B^2 + 11*a^4*b^2*c^8*A*B^2 + 2*a^2*b^4*c^8*A*B^2 - 7*b^6*c^8*A*B^2 - 11*a^4*c^10*A*B^2 - 12*a^2*b^2*c^10*A*B^2 + 3*b^4*c^10*A*B^2 + 7*a^2*c^12*A*B^2 + 3*b^2*c^12*A*B^2 - 2*c^14*A*B^2 + 2*a^14*B^3 - 7*a^12*b^2*B^3 + 9*a^10*b^4*B^3 - 5*a^8*b^6*B^3 + a^6*b^8*B^3 - 3*a^12*c^2*B^3 + 8*a^10*b^2*c^2*B^3 - 12*a^8*b^4*c^2*B^3 + 10*a^6*b^6*c^2*B^3 - 3*a^4*b^8*c^2*B^3 + 3*a^10*c^4*B^3 + 5*a^8*b^2*c^4*B^3 - 9*a^6*b^4*c^4*B^3 + 3*a^2*b^8*c^4*B^3 - 8*a^8*c^6*B^3 + 9*a^4*b^4*c^6*B^3 - 10*a^2*b^6*c^6*B^3 - b^8*c^6*B^3 + 8*a^6*c^8*B^3 - 5*a^4*b^2*c^8*B^3 + 12*a^2*b^4*c^8*B^3 + 5*b^6*c^8*B^3 - 3*a^4*c^10*B^3 - 8*a^2*b^2*c^10*B^3 - 9*b^4*c^10*B^3 + 3*a^2*c^12*B^3 + 7*b^2*c^12*B^3 - 2*c^14*B^3 + 3*a^8*b^6*A^2*C - 7*a^6*b^8*A^2*C + 3*a^4*b^10*A^2*C + 3*a^2*b^12*A^2*C - 2*b^14*A^2*C - 9*a^8*b^4*c^2*A^2*C + 12*a^6*b^6*c^2*A^2*C + 2*a^4*b^8*c^2*A^2*C - 12*a^2*b^10*c^2*A^2*C + 7*b^12*c^2*A^2*C + 9*a^8*b^2*c^4*A^2*C + 6*a^6*b^4*c^4*A^2*C - 15*a^4*b^6*c^4*A^2*C + 11*a^2*b^8*c^4*A^2*C - 11*b^10*c^4*A^2*C - 3*a^8*c^6*A^2*C - 20*a^6*b^2*c^6*A^2*C + 3*a^4*b^4*c^6*A^2*C + 6*a^2*b^6*c^6*A^2*C + 14*b^8*c^6*A^2*C + 9*a^6*c^8*A^2*C + 16*a^4*b^2*c^8*A^2*C - 9*a^2*b^4*c^8*A^2*C - 16*b^6*c^8*A^2*C - 9*a^4*c^10*A^2*C - 2*a^2*b^2*c^10*A^2*C + 11*b^4*c^10*A^2*C + 3*a^2*c^12*A^2*C - 3*b^2*c^12*A^2*C + 6*a^12*b^2*A*B*C - 20*a^10*b^4*A*B*C + 30*a^8*b^6*A*B*C - 30*a^6*b^8*A*B*C + 20*a^4*b^10*A*B*C - 6*a^2*b^12*A*B*C - 6*a^12*c^2*A*B*C + 10*a^8*b^4*c^2*A*B*C - 10*a^4*b^8*c^2*A*B*C + 6*b^12*c^2*A*B*C + 20*a^10*c^4*A*B*C - 10*a^8*b^2*c^4*A*B*C + 10*a^2*b^8*c^4*A*B*C - 20*b^10*c^4*A*B*C - 30*a^8*c^6*A*B*C + 30*b^8*c^6*A*B*C + 30*a^6*c^8*A*B*C + 10*a^4*b^2*c^8*A*B*C - 10*a^2*b^4*c^8*A*B*C - 30*b^6*c^8*A*B*C - 20*a^4*c^10*A*B*C + 20*b^4*c^10*A*B*C + 6*a^2*c^12*A*B*C - 6*b^2*c^12*A*B*C + 2*a^14*B^2*C - 3*a^12*b^2*B^2*C - 3*a^10*b^4*B^2*C + 7*a^8*b^6*B^2*C - 3*a^6*b^8*B^2*C - 7*a^12*c^2*B^2*C + 12*a^10*b^2*c^2*B^2*C - 2*a^8*b^4*c^2*B^2*C - 12*a^6*b^6*c^2*B^2*C + 9*a^4*b^8*c^2*B^2*C + 11*a^10*c^4*B^2*C - 11*a^8*b^2*c^4*B^2*C + 15*a^6*b^4*c^4*B^2*C - 6*a^4*b^6*c^4*B^2*C - 9*a^2*b^8*c^4*B^2*C - 14*a^8*c^6*B^2*C - 6*a^6*b^2*c^6*B^2*C - 3*a^4*b^4*c^6*B^2*C + 20*a^2*b^6*c^6*B^2*C + 3*b^8*c^6*B^2*C + 16*a^6*c^8*B^2*C + 9*a^4*b^2*c^8*B^2*C - 16*a^2*b^4*c^8*B^2*C - 9*b^6*c^8*B^2*C - 11*a^4*c^10*B^2*C + 2*a^2*b^2*c^10*B^2*C + 9*b^4*c^10*B^2*C + 3*a^2*c^12*B^2*C - 3*b^2*c^12*B^2*C + 3*a^12*b^2*A*C^2 - 11*a^10*b^4*A*C^2 + 16*a^8*b^6*A*C^2 - 14*a^6*b^8*A*C^2 + 11*a^4*b^10*A*C^2 - 7*a^2*b^12*A*C^2 + 2*b^14*A*C^2 - 3*a^12*c^2*A*C^2 + 2*a^10*b^2*c^2*A*C^2 + 9*a^8*b^4*c^2*A*C^2 - 6*a^6*b^6*c^2*A*C^2 - 11*a^4*b^8*c^2*A*C^2 + 12*a^2*b^10*c^2*A*C^2 - 3*b^12*c^2*A*C^2 + 9*a^10*c^4*A*C^2 - 16*a^8*b^2*c^4*A*C^2 - 3*a^6*b^4*c^4*A*C^2 + 15*a^4*b^6*c^4*A*C^2 - 2*a^2*b^8*c^4*A*C^2 - 3*b^10*c^4*A*C^2 - 9*a^8*c^6*A*C^2 + 20*a^6*b^2*c^6*A*C^2 - 6*a^4*b^4*c^6*A*C^2 - 12*a^2*b^6*c^6*A*C^2 + 7*b^8*c^6*A*C^2 + 3*a^6*c^8*A*C^2 - 9*a^4*b^2*c^8*A*C^2 + 9*a^2*b^4*c^8*A*C^2 - 3*b^6*c^8*A*C^2 - 2*a^14*B*C^2 + 7*a^12*b^2*B*C^2 - 11*a^10*b^4*B*C^2 + 14*a^8*b^6*B*C^2 - 16*a^6*b^8*B*C^2 + 11*a^4*b^10*B*C^2 - 3*a^2*b^12*B*C^2 + 3*a^12*c^2*B*C^2 - 12*a^10*b^2*c^2*B*C^2 + 11*a^8*b^4*c^2*B*C^2 + 6*a^6*b^6*c^2*B*C^2 - 9*a^4*b^8*c^2*B*C^2 - 2*a^2*b^10*c^2*B*C^2 + 3*b^12*c^2*B*C^2 + 3*a^10*c^4*B*C^2 + 2*a^8*b^2*c^4*B*C^2 - 15*a^6*b^4*c^4*B*C^2 + 3*a^4*b^6*c^4*B*C^2 + 16*a^2*b^8*c^4*B*C^2 - 9*b^10*c^4*B*C^2 - 7*a^8*c^6*B*C^2 + 12*a^6*b^2*c^6*B*C^2 + 6*a^4*b^4*c^6*B*C^2 - 20*a^2*b^6*c^6*B*C^2 + 9*b^8*c^6*B*C^2 + 3*a^6*c^8*B*C^2 - 9*a^4*b^2*c^8*B*C^2 + 9*a^2*b^4*c^8*B*C^2 - 3*b^6*c^8*B*C^2 - 2*a^14*C^3 + 3*a^12*b^2*C^3 - 3*a^10*b^4*C^3 + 8*a^8*b^6*C^3 - 8*a^6*b^8*C^3 + 3*a^4*b^10*C^3 - 3*a^2*b^12*C^3 + 2*b^14*C^3 + 7*a^12*c^2*C^3 - 8*a^10*b^2*c^2*C^3 - 5*a^8*b^4*c^2*C^3 + 5*a^4*b^8*c^2*C^3 + 8*a^2*b^10*c^2*C^3 - 7*b^12*c^2*C^3 - 9*a^10*c^4*C^3 + 12*a^8*b^2*c^4*C^3 + 9*a^6*b^4*c^4*C^3 - 9*a^4*b^6*c^4*C^3 - 12*a^2*b^8*c^4*C^3 + 9*b^10*c^4*C^3 + 5*a^8*c^6*C^3 - 10*a^6*b^2*c^6*C^3 + 10*a^2*b^6*c^6*C^3 - 5*b^8*c^6*C^3 - a^6*c^8*C^3 + 3*a^4*b^2*c^8*C^3 - 3*a^2*b^4*c^8*C^3 + b^6*c^8*C^3 = 0";
				break;
			case 594:
				equation = "2*a^3*b*c^2*A^2*B - a^2*b^2*c^2*A^2*B - 2*a*b^3*c^2*A^2*B + b^4*c^2*A^2*B - 2*a^2*b*c^3*A^2*B + 2*a*b^2*c^3*A^2*B - b^2*c^4*A^2*B - a^4*c^2*A*B^2 + 2*a^3*b*c^2*A*B^2 + a^2*b^2*c^2*A*B^2 - 2*a*b^3*c^2*A*B^2 - 2*a^2*b*c^3*A*B^2 + 2*a*b^2*c^3*A*B^2 + a^2*c^4*A*B^2 - 2*a^3*b^2*c*A^2*C + 2*a^2*b^3*c*A^2*C + a^2*b^2*c^2*A^2*C - 2*a*b^3*c^2*A^2*C + b^4*c^2*A^2*C + 2*a*b^2*c^3*A^2*C - b^2*c^4*A^2*C - 4*a^3*b^2*c*A*B*C + 4*a^2*b^3*c*A*B*C + 4*a^3*b*c^2*A*B*C - 4*a*b^3*c^2*A*B*C - 4*a^2*b*c^3*A*B*C + 4*a*b^2*c^3*A*B*C - 2*a^3*b^2*c*B^2*C + 2*a^2*b^3*c*B^2*C - a^4*c^2*B^2*C + 2*a^3*b*c^2*B^2*C - a^2*b^2*c^2*B^2*C - 2*a^2*b*c^3*B^2*C + a^2*c^4*B^2*C + a^4*b^2*A*C^2 - a^2*b^4*A*C^2 - 2*a^3*b^2*c*A*C^2 + 2*a^2*b^3*c*A*C^2 - a^2*b^2*c^2*A*C^2 - 2*a*b^3*c^2*A*C^2 + 2*a*b^2*c^3*A*C^2 + a^4*b^2*B*C^2 - a^2*b^4*B*C^2 - 2*a^3*b^2*c*B*C^2 + 2*a^2*b^3*c*B*C^2 + 2*a^3*b*c^2*B*C^2 + a^2*b^2*c^2*B*C^2 - 2*a^2*b*c^3*B*C^2 = 0";
				break;
			case 595:
				equation = "2*a^8*b^2*A^2*B - 5*a^6*b^4*A^2*B + 3*a^4*b^6*A^2*B + a^2*b^8*A^2*B - b^10*A^2*B + 2*a^8*c^2*A^2*B - 4*a^6*b^2*c^2*A^2*B + 2*a^4*b^4*c^2*A^2*B - a^2*b^6*c^2*A^2*B + b^8*c^2*A^2*B - 3*a^6*c^4*A^2*B + 5*a^4*b^2*c^4*A^2*B - a^2*b^4*c^4*A^2*B - 2*b^6*c^4*A^2*B + 2*a^4*c^6*A^2*B - a^2*b^2*c^6*A^2*B + 2*b^4*c^6*A^2*B - 2*a^2*c^8*A^2*B - b^2*c^8*A^2*B + c^10*A^2*B + a^10*A*B^2 - a^8*b^2*A*B^2 - 3*a^6*b^4*A*B^2 + 5*a^4*b^6*A*B^2 - 2*a^2*b^8*A*B^2 - a^8*c^2*A*B^2 + a^6*b^2*c^2*A*B^2 - 2*a^4*b^4*c^2*A*B^2 + 4*a^2*b^6*c^2*A*B^2 - 2*b^8*c^2*A*B^2 + 2*a^6*c^4*A*B^2 + a^4*b^2*c^4*A*B^2 - 5*a^2*b^4*c^4*A*B^2 + 3*b^6*c^4*A*B^2 - 2*a^4*c^6*A*B^2 + a^2*b^2*c^6*A*B^2 - 2*b^4*c^6*A*B^2 + a^2*c^8*A*B^2 + 2*b^2*c^8*A*B^2 - c^10*A*B^2 - 2*a^8*b^2*A^2*C + 3*a^6*b^4*A^2*C - 2*a^4*b^6*A^2*C + 2*a^2*b^8*A^2*C - b^10*A^2*C - 2*a^8*c^2*A^2*C + 4*a^6*b^2*c^2*A^2*C - 5*a^4*b^4*c^2*A^2*C + a^2*b^6*c^2*A^2*C + b^8*c^2*A^2*C + 5*a^6*c^4*A^2*C - 2*a^4*b^2*c^4*A^2*C + a^2*b^4*c^4*A^2*C - 2*b^6*c^4*A^2*C - 3*a^4*c^6*A^2*C + a^2*b^2*c^6*A^2*C + 2*b^4*c^6*A^2*C - a^2*c^8*A^2*C - b^2*c^8*A^2*C + c^10*A^2*C - 2*a^8*b^2*A*B*C + 4*a^6*b^4*A*B*C - 4*a^4*b^6*A*B*C + 2*a^2*b^8*A*B*C + 2*a^8*c^2*A*B*C - 2*b^8*c^2*A*B*C - 4*a^6*c^4*A*B*C + 4*b^6*c^4*A*B*C + 4*a^4*c^6*A*B*C - 4*b^4*c^6*A*B*C - 2*a^2*c^8*A*B*C + 2*b^2*c^8*A*B*C + a^10*B^2*C - 2*a^8*b^2*B^2*C + 2*a^6*b^4*B^2*C - 3*a^4*b^6*B^2*C + 2*a^2*b^8*B^2*C - a^8*c^2*B^2*C - a^6*b^2*c^2*B^2*C + 5*a^4*b^4*c^2*B^2*C - 4*a^2*b^6*c^2*B^2*C + 2*b^8*c^2*B^2*C + 2*a^6*c^4*B^2*C - a^4*b^2*c^4*B^2*C + 2*a^2*b^4*c^4*B^2*C - 5*b^6*c^4*B^2*C - 2*a^4*c^6*B^2*C - a^2*b^2*c^6*B^2*C + 3*b^4*c^6*B^2*C + a^2*c^8*B^2*C + b^2*c^8*B^2*C - c^10*B^2*C - a^10*A*C^2 + a^8*b^2*A*C^2 - 2*a^6*b^4*A*C^2 + 2*a^4*b^6*A*C^2 - a^2*b^8*A*C^2 + b^10*A*C^2 + a^8*c^2*A*C^2 - a^6*b^2*c^2*A*C^2 - a^4*b^4*c^2*A*C^2 - a^2*b^6*c^2*A*C^2 - 2*b^8*c^2*A*C^2 + 3*a^6*c^4*A*C^2 + 2*a^4*b^2*c^4*A*C^2 + 5*a^2*b^4*c^4*A*C^2 + 2*b^6*c^4*A*C^2 - 5*a^4*c^6*A*C^2 - 4*a^2*b^2*c^6*A*C^2 - 3*b^4*c^6*A*C^2 + 2*a^2*c^8*A*C^2 + 2*b^2*c^8*A*C^2 - a^10*B*C^2 + a^8*b^2*B*C^2 - 2*a^6*b^4*B*C^2 + 2*a^4*b^6*B*C^2 - a^2*b^8*B*C^2 + b^10*B*C^2 + 2*a^8*c^2*B*C^2 + a^6*b^2*c^2*B*C^2 + a^4*b^4*c^2*B*C^2 + a^2*b^6*c^2*B*C^2 - b^8*c^2*B*C^2 - 2*a^6*c^4*B*C^2 - 5*a^4*b^2*c^4*B*C^2 - 2*a^2*b^4*c^4*B*C^2 - 3*b^6*c^4*B*C^2 + 3*a^4*c^6*B*C^2 + 4*a^2*b^2*c^6*B*C^2 + 5*b^4*c^6*B*C^2 - 2*a^2*c^8*B*C^2 - 2*b^2*c^8*B*C^2 = 0";
				break;
			case 596:
				equation = "2*a^8*b^2*A^2*B - 5*a^6*b^4*A^2*B + 3*a^4*b^6*A^2*B + a^2*b^8*A^2*B - b^10*A^2*B - 2*a^8*c^2*A^2*B + 2*a^4*b^4*c^2*A^2*B - 5*a^2*b^6*c^2*A^2*B + 5*b^8*c^2*A^2*B + 5*a^6*c^4*A^2*B + a^4*b^2*c^4*A^2*B + 3*a^2*b^4*c^4*A^2*B - 10*b^6*c^4*A^2*B - 6*a^4*c^6*A^2*B - a^2*b^2*c^6*A^2*B + 10*b^4*c^6*A^2*B + 2*a^2*c^8*A^2*B - 5*b^2*c^8*A^2*B + c^10*A^2*B + a^10*A*B^2 - a^8*b^2*A*B^2 - 3*a^6*b^4*A*B^2 + 5*a^4*b^6*A*B^2 - 2*a^2*b^8*A*B^2 - 5*a^8*c^2*A*B^2 + 5*a^6*b^2*c^2*A*B^2 - 2*a^4*b^4*c^2*A*B^2 + 2*b^8*c^2*A*B^2 + 10*a^6*c^4*A*B^2 - 3*a^4*b^2*c^4*A*B^2 - a^2*b^4*c^4*A*B^2 - 5*b^6*c^4*A*B^2 - 10*a^4*c^6*A*B^2 + a^2*b^2*c^6*A*B^2 + 6*b^4*c^6*A*B^2 + 5*a^2*c^8*A*B^2 - 2*b^2*c^8*A*B^2 - c^10*A*B^2 + 2*a^8*b^2*A^2*C - 5*a^6*b^4*A^2*C + 6*a^4*b^6*A^2*C - 2*a^2*b^8*A^2*C - b^10*A^2*C - 2*a^8*c^2*A^2*C - a^4*b^4*c^2*A^2*C + a^2*b^6*c^2*A^2*C + 5*b^8*c^2*A^2*C + 5*a^6*c^4*A^2*C - 2*a^4*b^2*c^4*A^2*C - 3*a^2*b^4*c^4*A^2*C - 10*b^6*c^4*A^2*C - 3*a^4*c^6*A^2*C + 5*a^2*b^2*c^6*A^2*C + 10*b^4*c^6*A^2*C - a^2*c^8*A^2*C - 5*b^2*c^8*A^2*C + c^10*A^2*C + 6*a^8*b^2*A*B*C - 12*a^6*b^4*A*B*C + 12*a^4*b^6*A*B*C - 6*a^2*b^8*A*B*C - 6*a^8*c^2*A*B*C + 6*b^8*c^2*A*B*C + 12*a^6*c^4*A*B*C - 12*b^6*c^4*A*B*C - 12*a^4*c^6*A*B*C + 12*b^4*c^6*A*B*C + 6*a^2*c^8*A*B*C - 6*b^2*c^8*A*B*C + a^10*B^2*C + 2*a^8*b^2*B^2*C - 6*a^6*b^4*B^2*C + 5*a^4*b^6*B^2*C - 2*a^2*b^8*B^2*C - 5*a^8*c^2*B^2*C - a^6*b^2*c^2*B^2*C + a^4*b^4*c^2*B^2*C + 2*b^8*c^2*B^2*C + 10*a^6*c^4*B^2*C + 3*a^4*b^2*c^4*B^2*C + 2*a^2*b^4*c^4*B^2*C - 5*b^6*c^4*B^2*C - 10*a^4*c^6*B^2*C - 5*a^2*b^2*c^6*B^2*C + 3*b^4*c^6*B^2*C + 5*a^2*c^8*B^2*C + b^2*c^8*B^2*C - c^10*B^2*C - a^10*A*C^2 + 5*a^8*b^2*A*C^2 - 10*a^6*b^4*A*C^2 + 10*a^4*b^6*A*C^2 - 5*a^2*b^8*A*C^2 + b^10*A*C^2 + a^8*c^2*A*C^2 - 5*a^6*b^2*c^2*A*C^2 + 3*a^4*b^4*c^2*A*C^2 - a^2*b^6*c^2*A*C^2 + 2*b^8*c^2*A*C^2 + 3*a^6*c^4*A*C^2 + 2*a^4*b^2*c^4*A*C^2 + a^2*b^4*c^4*A*C^2 - 6*b^6*c^4*A*C^2 - 5*a^4*c^6*A*C^2 + 5*b^4*c^6*A*C^2 + 2*a^2*c^8*A*C^2 - 2*b^2*c^8*A*C^2 - a^10*B*C^2 + 5*a^8*b^2*B*C^2 - 10*a^6*b^4*B*C^2 + 10*a^4*b^6*B*C^2 - 5*a^2*b^8*B*C^2 + b^10*B*C^2 - 2*a^8*c^2*B*C^2 + a^6*b^2*c^2*B*C^2 - 3*a^4*b^4*c^2*B*C^2 + 5*a^2*b^6*c^2*B*C^2 - b^8*c^2*B*C^2 + 6*a^6*c^4*B*C^2 - a^4*b^2*c^4*B*C^2 - 2*a^2*b^4*c^4*B*C^2 - 3*b^6*c^4*B*C^2 - 5*a^4*c^6*B*C^2 + 5*b^4*c^6*B*C^2 + 2*a^2*c^8*B*C^2 - 2*b^2*c^8*B*C^2 = 0";
				break;
			case 597:
				equation = "2*a^16*b^2*A^2*B - 13*a^14*b^4*A^2*B + 35*a^12*b^6*A^2*B - 49*a^10*b^8*A^2*B + 35*a^8*b^10*A^2*B - 7*a^6*b^12*A^2*B - 7*a^4*b^14*A^2*B + 5*a^2*b^16*A^2*B - b^18*A^2*B + 2*a^16*c^2*A^2*B - 2*a^14*b^2*c^2*A^2*B - 8*a^12*b^4*c^2*A^2*B + a^10*b^6*c^2*A^2*B + 45*a^8*b^8*c^2*A^2*B - 72*a^6*b^10*c^2*A^2*B + 42*a^4*b^12*c^2*A^2*B - 7*a^2*b^14*c^2*A^2*B - b^16*c^2*A^2*B - 5*a^14*c^4*A^2*B + 17*a^12*b^2*c^4*A^2*B + 25*a^10*b^4*c^4*A^2*B - 120*a^8*b^6*c^4*A^2*B + 106*a^6*b^8*c^4*A^2*B + 2*a^4*b^10*c^4*A^2*B - 34*a^2*b^12*c^4*A^2*B + 9*b^14*c^4*A^2*B - 4*a^12*c^6*A^2*B - 41*a^10*b^2*c^6*A^2*B + 86*a^8*b^4*c^6*A^2*B + 36*a^6*b^6*c^6*A^2*B - 142*a^4*b^8*c^6*A^2*B + 66*a^2*b^10*c^6*A^2*B - b^12*c^6*A^2*B + 24*a^10*c^8*A^2*B - a^8*b^2*c^8*A^2*B - 136*a^6*b^4*c^8*A^2*B + 128*a^4*b^6*c^8*A^2*B + 10*a^2*b^8*c^8*A^2*B - 26*b^10*c^8*A^2*B - 25*a^8*c^10*A^2*B + 66*a^6*b^2*c^10*A^2*B + 18*a^4*b^4*c^10*A^2*B - 80*a^2*b^6*c^10*A^2*B + 26*b^8*c^10*A^2*B + 3*a^6*c^12*A^2*B - 51*a^4*b^2*c^12*A^2*B + 37*a^2*b^4*c^12*A^2*B + b^6*c^12*A^2*B + 10*a^4*c^14*A^2*B + 9*a^2*b^2*c^14*A^2*B - 9*b^4*c^14*A^2*B - 6*a^2*c^16*A^2*B + b^2*c^16*A^2*B + c^18*A^2*B + a^18*A*B^2 - 5*a^16*b^2*A*B^2 + 7*a^14*b^4*A*B^2 + 7*a^12*b^6*A*B^2 - 35*a^10*b^8*A*B^2 + 49*a^8*b^10*A*B^2 - 35*a^6*b^12*A*B^2 + 13*a^4*b^14*A*B^2 - 2*a^2*b^16*A*B^2 + a^16*c^2*A*B^2 + 7*a^14*b^2*c^2*A*B^2 - 42*a^12*b^4*c^2*A*B^2 + 72*a^10*b^6*c^2*A*B^2 - 45*a^8*b^8*c^2*A*B^2 - a^6*b^10*c^2*A*B^2 + 8*a^4*b^12*c^2*A*B^2 + 2*a^2*b^14*c^2*A*B^2 - 2*b^16*c^2*A*B^2 - 9*a^14*c^4*A*B^2 + 34*a^12*b^2*c^4*A*B^2 - 2*a^10*b^4*c^4*A*B^2 - 106*a^8*b^6*c^4*A*B^2 + 120*a^6*b^8*c^4*A*B^2 - 25*a^4*b^10*c^4*A*B^2 - 17*a^2*b^12*c^4*A*B^2 + 5*b^14*c^4*A*B^2 + a^12*c^6*A*B^2 - 66*a^10*b^2*c^6*A*B^2 + 142*a^8*b^4*c^6*A*B^2 - 36*a^6*b^6*c^6*A*B^2 - 86*a^4*b^8*c^6*A*B^2 + 41*a^2*b^10*c^6*A*B^2 + 4*b^12*c^6*A*B^2 + 26*a^10*c^8*A*B^2 - 10*a^8*b^2*c^8*A*B^2 - 128*a^6*b^4*c^8*A*B^2 + 136*a^4*b^6*c^8*A*B^2 + a^2*b^8*c^8*A*B^2 - 24*b^10*c^8*A*B^2 - 26*a^8*c^10*A*B^2 + 80*a^6*b^2*c^10*A*B^2 - 18*a^4*b^4*c^10*A*B^2 - 66*a^2*b^6*c^10*A*B^2 + 25*b^8*c^10*A*B^2 - a^6*c^12*A*B^2 - 37*a^4*b^2*c^12*A*B^2 + 51*a^2*b^4*c^12*A*B^2 - 3*b^6*c^12*A*B^2 + 9*a^4*c^14*A*B^2 - 9*a^2*b^2*c^14*A*B^2 - 10*b^4*c^14*A*B^2 - a^2*c^16*A*B^2 + 6*b^2*c^16*A*B^2 - c^18*A*B^2 - 2*a^16*b^2*A^2*C + 5*a^14*b^4*A^2*C + 4*a^12*b^6*A^2*C - 24*a^10*b^8*A^2*C + 25*a^8*b^10*A^2*C - 3*a^6*b^12*A^2*C - 10*a^4*b^14*A^2*C + 6*a^2*b^16*A^2*C - b^18*A^2*C - 2*a^16*c^2*A^2*C + 2*a^14*b^2*c^2*A^2*C - 17*a^12*b^4*c^2*A^2*C + 41*a^10*b^6*c^2*A^2*C + a^8*b^8*c^2*A^2*C - 66*a^6*b^10*c^2*A^2*C + 51*a^4*b^12*c^2*A^2*C - 9*a^2*b^14*c^2*A^2*C - b^16*c^2*A^2*C + 13*a^14*c^4*A^2*C + 8*a^12*b^2*c^4*A^2*C - 25*a^10*b^4*c^4*A^2*C - 86*a^8*b^6*c^4*A^2*C + 136*a^6*b^8*c^4*A^2*C - 18*a^4*b^10*c^4*A^2*C - 37*a^2*b^12*c^4*A^2*C + 9*b^14*c^4*A^2*C - 35*a^12*c^6*A^2*C - a^10*b^2*c^6*A^2*C + 120*a^8*b^4*c^6*A^2*C - 36*a^6*b^6*c^6*A^2*C - 128*a^4*b^8*c^6*A^2*C + 80*a^2*b^10*c^6*A^2*C - b^12*c^6*A^2*C + 49*a^10*c^8*A^2*C - 45*a^8*b^2*c^8*A^2*C - 106*a^6*b^4*c^8*A^2*C + 142*a^4*b^6*c^8*A^2*C - 10*a^2*b^8*c^8*A^2*C - 26*b^10*c^8*A^2*C - 35*a^8*c^10*A^2*C + 72*a^6*b^2*c^10*A^2*C - 2*a^4*b^4*c^10*A^2*C - 66*a^2*b^6*c^10*A^2*C + 26*b^8*c^10*A^2*C + 7*a^6*c^12*A^2*C - 42*a^4*b^2*c^12*A^2*C + 34*a^2*b^4*c^12*A^2*C + b^6*c^12*A^2*C + 7*a^4*c^14*A^2*C + 7*a^2*b^2*c^14*A^2*C - 9*b^4*c^14*A^2*C - 5*a^2*c^16*A^2*C + b^2*c^16*A^2*C + c^18*A^2*C - 2*a^16*b^2*A*B*C + 28*a^12*b^6*A*B*C - 70*a^10*b^8*A*B*C + 70*a^8*b^10*A*B*C - 28*a^6*b^12*A*B*C + 2*a^2*b^16*A*B*C + 2*a^16*c^2*A*B*C - 54*a^12*b^4*c^2*A*B*C + 100*a^10*b^6*c^2*A*B*C - 100*a^6*b^10*c^2*A*B*C + 54*a^4*b^12*c^2*A*B*C - 2*b^16*c^2*A*B*C + 54*a^12*b^2*c^4*A*B*C - 260*a^8*b^6*c^4*A*B*C + 260*a^6*b^8*c^4*A*B*C - 54*a^2*b^12*c^4*A*B*C - 28*a^12*c^6*A*B*C - 100*a^10*b^2*c^6*A*B*C + 260*a^8*b^4*c^6*A*B*C - 260*a^4*b^8*c^6*A*B*C + 100*a^2*b^10*c^6*A*B*C + 28*b^12*c^6*A*B*C + 70*a^10*c^8*A*B*C - 260*a^6*b^4*c^8*A*B*C + 260*a^4*b^6*c^8*A*B*C - 70*b^10*c^8*A*B*C - 70*a^8*c^10*A*B*C + 100*a^6*b^2*c^10*A*B*C - 100*a^2*b^6*c^10*A*B*C + 70*b^8*c^10*A*B*C + 28*a^6*c^12*A*B*C - 54*a^4*b^2*c^12*A*B*C + 54*a^2*b^4*c^12*A*B*C - 28*b^6*c^12*A*B*C - 2*a^2*c^16*A*B*C + 2*b^2*c^16*A*B*C + a^18*B^2*C - 6*a^16*b^2*B^2*C + 10*a^14*b^4*B^2*C + 3*a^12*b^6*B^2*C - 25*a^10*b^8*B^2*C + 24*a^8*b^10*B^2*C - 4*a^6*b^12*B^2*C - 5*a^4*b^14*B^2*C + 2*a^2*b^16*B^2*C + a^16*c^2*B^2*C + 9*a^14*b^2*c^2*B^2*C - 51*a^12*b^4*c^2*B^2*C + 66*a^10*b^6*c^2*B^2*C - a^8*b^8*c^2*B^2*C - 41*a^6*b^10*c^2*B^2*C + 17*a^4*b^12*c^2*B^2*C - 2*a^2*b^14*c^2*B^2*C + 2*b^16*c^2*B^2*C - 9*a^14*c^4*B^2*C + 37*a^12*b^2*c^4*B^2*C + 18*a^10*b^4*c^4*B^2*C - 136*a^8*b^6*c^4*B^2*C + 86*a^6*b^8*c^4*B^2*C + 25*a^4*b^10*c^4*B^2*C - 8*a^2*b^12*c^4*B^2*C - 13*b^14*c^4*B^2*C + a^12*c^6*B^2*C - 80*a^10*b^2*c^6*B^2*C + 128*a^8*b^4*c^6*B^2*C + 36*a^6*b^6*c^6*B^2*C - 120*a^4*b^8*c^6*B^2*C + a^2*b^10*c^6*B^2*C + 35*b^12*c^6*B^2*C + 26*a^10*c^8*B^2*C + 10*a^8*b^2*c^8*B^2*C - 142*a^6*b^4*c^8*B^2*C + 106*a^4*b^6*c^8*B^2*C + 45*a^2*b^8*c^8*B^2*C - 49*b^10*c^8*B^2*C - 26*a^8*c^10*B^2*C + 66*a^6*b^2*c^10*B^2*C + 2*a^4*b^4*c^10*B^2*C - 72*a^2*b^6*c^10*B^2*C + 35*b^8*c^10*B^2*C - a^6*c^12*B^2*C - 34*a^4*b^2*c^12*B^2*C + 42*a^2*b^4*c^12*B^2*C - 7*b^6*c^12*B^2*C + 9*a^4*c^14*B^2*C - 7*a^2*b^2*c^14*B^2*C - 7*b^4*c^14*B^2*C - a^2*c^16*B^2*C + 5*b^2*c^16*B^2*C - c^18*B^2*C - a^18*A*C^2 - a^16*b^2*A*C^2 + 9*a^14*b^4*A*C^2 - a^12*b^6*A*C^2 - 26*a^10*b^8*A*C^2 + 26*a^8*b^10*A*C^2 + a^6*b^12*A*C^2 - 9*a^4*b^14*A*C^2 + a^2*b^16*A*C^2 + b^18*A*C^2 + 5*a^16*c^2*A*C^2 - 7*a^14*b^2*c^2*A*C^2 - 34*a^12*b^4*c^2*A*C^2 + 66*a^10*b^6*c^2*A*C^2 + 10*a^8*b^8*c^2*A*C^2 - 80*a^6*b^10*c^2*A*C^2 + 37*a^4*b^12*c^2*A*C^2 + 9*a^2*b^14*c^2*A*C^2 - 6*b^16*c^2*A*C^2 - 7*a^14*c^4*A*C^2 + 42*a^12*b^2*c^4*A*C^2 + 2*a^10*b^4*c^4*A*C^2 - 142*a^8*b^6*c^4*A*C^2 + 128*a^6*b^8*c^4*A*C^2 + 18*a^4*b^10*c^4*A*C^2 - 51*a^2*b^12*c^4*A*C^2 + 10*b^14*c^4*A*C^2 - 7*a^12*c^6*A*C^2 - 72*a^10*b^2*c^6*A*C^2 + 106*a^8*b^4*c^6*A*C^2 + 36*a^6*b^6*c^6*A*C^2 - 136*a^4*b^8*c^6*A*C^2 + 66*a^2*b^10*c^6*A*C^2 + 3*b^12*c^6*A*C^2 + 35*a^10*c^8*A*C^2 + 45*a^8*b^2*c^8*A*C^2 - 120*a^6*b^4*c^8*A*C^2 + 86*a^4*b^6*c^8*A*C^2 - a^2*b^8*c^8*A*C^2 - 25*b^10*c^8*A*C^2 - 49*a^8*c^10*A*C^2 + a^6*b^2*c^10*A*C^2 + 25*a^4*b^4*c^10*A*C^2 - 41*a^2*b^6*c^10*A*C^2 + 24*b^8*c^10*A*C^2 + 35*a^6*c^12*A*C^2 - 8*a^4*b^2*c^12*A*C^2 + 17*a^2*b^4*c^12*A*C^2 - 4*b^6*c^12*A*C^2 - 13*a^4*c^14*A*C^2 - 2*a^2*b^2*c^14*A*C^2 - 5*b^4*c^14*A*C^2 + 2*a^2*c^16*A*C^2 + 2*b^2*c^16*A*C^2 - a^18*B*C^2 - a^16*b^2*B*C^2 + 9*a^14*b^4*B*C^2 - a^12*b^6*B*C^2 - 26*a^10*b^8*B*C^2 + 26*a^8*b^10*B*C^2 + a^6*b^12*B*C^2 - 9*a^4*b^14*B*C^2 + a^2*b^16*B*C^2 + b^18*B*C^2 + 6*a^16*c^2*B*C^2 - 9*a^14*b^2*c^2*B*C^2 - 37*a^12*b^4*c^2*B*C^2 + 80*a^10*b^6*c^2*B*C^2 - 10*a^8*b^8*c^2*B*C^2 - 66*a^6*b^10*c^2*B*C^2 + 34*a^4*b^12*c^2*B*C^2 + 7*a^2*b^14*c^2*B*C^2 - 5*b^16*c^2*B*C^2 - 10*a^14*c^4*B*C^2 + 51*a^12*b^2*c^4*B*C^2 - 18*a^10*b^4*c^4*B*C^2 - 128*a^8*b^6*c^4*B*C^2 + 142*a^6*b^8*c^4*B*C^2 - 2*a^4*b^10*c^4*B*C^2 - 42*a^2*b^12*c^4*B*C^2 + 7*b^14*c^4*B*C^2 - 3*a^12*c^6*B*C^2 - 66*a^10*b^2*c^6*B*C^2 + 136*a^8*b^4*c^6*B*C^2 - 36*a^6*b^6*c^6*B*C^2 - 106*a^4*b^8*c^6*B*C^2 + 72*a^2*b^10*c^6*B*C^2 + 7*b^12*c^6*B*C^2 + 25*a^10*c^8*B*C^2 + a^8*b^2*c^8*B*C^2 - 86*a^6*b^4*c^8*B*C^2 + 120*a^4*b^6*c^8*B*C^2 - 45*a^2*b^8*c^8*B*C^2 - 35*b^10*c^8*B*C^2 - 24*a^8*c^10*B*C^2 + 41*a^6*b^2*c^10*B*C^2 - 25*a^4*b^4*c^10*B*C^2 - a^2*b^6*c^10*B*C^2 + 49*b^8*c^10*B*C^2 + 4*a^6*c^12*B*C^2 - 17*a^4*b^2*c^12*B*C^2 + 8*a^2*b^4*c^12*B*C^2 - 35*b^6*c^12*B*C^2 + 5*a^4*c^14*B*C^2 + 2*a^2*b^2*c^14*B*C^2 + 13*b^4*c^14*B*C^2 - 2*a^2*c^16*B*C^2 - 2*b^2*c^16*B*C^2 = 0";
				break;
			case 598:
				
				break;
			case 599:
				equation = "a^4*c^2*A^2*B - 2*a^2*b^2*c^2*A^2*B + b^4*c^2*A^2*B - 3*a^2*c^4*A^2*B + b^2*c^4*A^2*B + 2*c^6*A^2*B - a^4*c^2*A*B^2 + 2*a^2*b^2*c^2*A*B^2 - b^4*c^2*A*B^2 - a^2*c^4*A*B^2 + 3*b^2*c^4*A*B^2 - 2*c^6*A*B^2 - a^4*b^2*A^2*C + 3*a^2*b^4*A^2*C - 2*b^6*A^2*C + 2*a^2*b^2*c^2*A^2*C - b^4*c^2*A^2*C - b^2*c^4*A^2*C - 2*a^4*b^2*A*B*C + 2*a^2*b^4*A*B*C + 2*a^4*c^2*A*B*C - 2*b^4*c^2*A*B*C - 2*a^2*c^4*A*B*C + 2*b^2*c^4*A*B*C + 2*a^6*B^2*C - 3*a^4*b^2*B^2*C + a^2*b^4*B^2*C + a^4*c^2*B^2*C - 2*a^2*b^2*c^2*B^2*C + a^2*c^4*B^2*C + a^4*b^2*A*C^2 + a^2*b^4*A*C^2 + 2*b^6*A*C^2 - 2*a^2*b^2*c^2*A*C^2 - 3*b^4*c^2*A*C^2 + b^2*c^4*A*C^2 - 2*a^6*B*C^2 - a^4*b^2*B*C^2 - a^2*b^4*B*C^2 + 3*a^4*c^2*B*C^2 + 2*a^2*b^2*c^2*B*C^2 - a^2*c^4*B*C^2 = 0";
				break;
			case 600:
				equation = "a^6*A^3 - 2*a^4*b^2*A^3 + a^2*b^4*A^3 - 2*a^4*c^2*A^3 + 2*a^2*b^2*c^2*A^3 + a^2*c^4*A^3 - a^6*A^2*B + 3*a^2*b^4*A^2*B - 2*b^6*A^2*B + 3*a^4*c^2*A^2*B - 2*a^2*b^2*c^2*A^2*B - b^4*c^2*A^2*B - 3*a^2*c^4*A^2*B + 2*b^2*c^4*A^2*B + c^6*A^2*B - 2*a^6*A*B^2 + 3*a^4*b^2*A*B^2 - b^6*A*B^2 - a^4*c^2*A*B^2 - 2*a^2*b^2*c^2*A*B^2 + 3*b^4*c^2*A*B^2 + 2*a^2*c^4*A*B^2 - 3*b^2*c^4*A*B^2 + c^6*A*B^2 + a^4*b^2*B^3 - 2*a^2*b^4*B^3 + b^6*B^3 + 2*a^2*b^2*c^2*B^3 - 2*b^4*c^2*B^3 + b^2*c^4*B^3 - a^6*A^2*C + 3*a^4*b^2*A^2*C - 3*a^2*b^4*A^2*C + b^6*A^2*C - 2*a^2*b^2*c^2*A^2*C + 2*b^4*c^2*A^2*C + 3*a^2*c^4*A^2*C - b^2*c^4*A^2*C - 2*c^6*A^2*C + 3*a^6*A*B*C - 3*a^4*b^2*A*B*C - 3*a^2*b^4*A*B*C + 3*b^6*A*B*C - 3*a^4*c^2*A*B*C + 6*a^2*b^2*c^2*A*B*C - 3*b^4*c^2*A*B*C - 3*a^2*c^4*A*B*C - 3*b^2*c^4*A*B*C + 3*c^6*A*B*C + a^6*B^2*C - 3*a^4*b^2*B^2*C + 3*a^2*b^4*B^2*C - b^6*B^2*C + 2*a^4*c^2*B^2*C - 2*a^2*b^2*c^2*B^2*C - a^2*c^4*B^2*C + 3*b^2*c^4*B^2*C - 2*c^6*B^2*C - 2*a^6*A*C^2 - a^4*b^2*A*C^2 + 2*a^2*b^4*A*C^2 + b^6*A*C^2 + 3*a^4*c^2*A*C^2 - 2*a^2*b^2*c^2*A*C^2 - 3*b^4*c^2*A*C^2 + 3*b^2*c^4*A*C^2 - c^6*A*C^2 + a^6*B*C^2 + 2*a^4*b^2*B*C^2 - a^2*b^4*B*C^2 - 2*b^6*B*C^2 - 3*a^4*c^2*B*C^2 - 2*a^2*b^2*c^2*B*C^2 + 3*b^4*c^2*B*C^2 + 3*a^2*c^4*B*C^2 - c^6*B*C^2 + a^4*c^2*C^3 + 2*a^2*b^2*c^2*C^3 + b^4*c^2*C^3 - 2*a^2*c^4*C^3 - 2*b^2*c^4*C^3 + c^6*C^3 = 0";
				break;
			case 601:
				equation = "-a^18*A^2*B + 4*a^16*b^2*A^2*B - 3*a^14*b^4*A^2*B - 10*a^12*b^6*A^2*B +25*a^10*b^8*A^2*B - 24*a^8*b^10*A^2*B + 11*a^6*b^12*A^2*B -2*a^4*b^14*A^2*B + 2*a^16*c^2*A^2*B - 14*a^14*b^2*c^2*A^2*B +33*a^12*b^4*c^2*A^2*B - 28*a^10*b^6*c^2*A^2*B - 8*a^8*b^8*c^2*A^2*B +30*a^6*b^10*c^2*A^2*B - 19*a^4*b^12*c^2*A^2*B + 4*a^2*b^14*c^2*A^2*B +2*a^14*c^4*A^2*B + 4*a^12*b^2*c^4*A^2*B -39*a^10*b^4*c^4*A^2*B + 80*a^8*b^6*c^4*A^2*B - 68*a^6*b^8*c^4*A^2*B +18*a^4*b^10*c^4*A^2*B + 5*a^2*b^12*c^4*A^2*B - 2*b^14*c^4*A^2*B -7*a^12*c^6*A^2*B + 22*a^10*b^2*c^6*A^2*B -30*a^8*b^4*c^6*A^2*B - 4*a^6*b^6*c^6*A^2*B + 46*a^4*b^8*c^6*A^2*B -30*a^2*b^10*c^6*A^2*B + 3*b^12*c^6*A^2*B + 5*a^10*c^8*A^2*B -8*a^8*b^2*c^8*A^2*B + 42*a^6*b^4*c^8*A^2*B - 64*a^4*b^6*c^8*A^2*B +19*a^2*b^8*c^8*A^2*B + 6*b^10*c^8*A^2*B - 4*a^8*c^10*A^2*B -20*a^6*b^2*c^10*A^2*B + 18*a^4*b^4*c^10*A^2*B + 20*a^2*b^6*c^10*A^2*B -14*b^8*c^10*A^2*B + 8*a^6*c^12*A^2*B + 10*a^4*b^2*c^12*A^2*B -24*a^2*b^4*c^12*A^2*B + 6*b^6*c^12*A^2*B - 7*a^4*c^14*A^2*B +4*a^2*b^2*c^14*A^2*B + 3*b^4*c^14*A^2*B + 2*a^2*c^16*A^2*B -2*b^2*c^16*A^2*B - 2*a^14*b^4*A*B^2 + 11*a^12*b^6*A*B^2 -24*a^10*b^8*A*B^2 +25*a^8*b^10*A*B^2 - 10*a^6*b^12*A*B^2 - 3*a^4*b^14*A*B^2 +4*a^2*b^16*A*B^2 - b^18*A*B^2 + 4*a^14*b^2*c^2*A*B^2 -19*a^12*b^4*c^2*A*B^2 + 30*a^10*b^6*c^2*A*B^2 - 8*a^8*b^8*c^2*A*B^2 -28*a^6*b^10*c^2*A*B^2 + 33*a^4*b^12*c^2*A*B^2 - 14*a^2*b^14*c^2*A*B^2 +2*b^16*c^2*A*B^2 - 2*a^14*c^4*A*B^2 + 5*a^12*b^2*c^4*A*B^2 +18*a^10*b^4*c^4*A*B^2 - 68*a^8*b^6*c^4*A*B^2 +80*a^6*b^8*c^4*A*B^2 - 39*a^4*b^10*c^4*A*B^2 + 4*a^2*b^12*c^4*A*B^2 +2*b^14*c^4*A*B^2 + 3*a^12*c^6*A*B^2 - 30*a^10*b^2*c^6*A*B^2 +46*a^8*b^4*c^6*A*B^2 - 4*a^6*b^6*c^6*A*B^2 - 30*a^4*b^8*c^6*A*B^2 +22*a^2*b^10*c^6*A*B^2 - 7*b^12*c^6*A*B^2 + 6*a^10*c^8*A*B^2 +19*a^8*b^2*c^8*A*B^2 - 64*a^6*b^4*c^8*A*B^2 + 42*a^4*b^6*c^8*A*B^2 -8*a^2*b^8*c^8*A*B^2 + 5*b^10*c^8*A*B^2 - 14*a^8*c^10*A*B^2 +20*a^6*b^2*c^10*A*B^2 + 18*a^4*b^4*c^10*A*B^2 - 20*a^2*b^6*c^10*A*B^2 -4*b^8*c^10*A*B^2 + 6*a^6*c^12*A*B^2 - 24*a^4*b^2*c^12*A*B^2 +10*a^2*b^4*c^12*A*B^2 + 8*b^6*c^12*A*B^2 + 3*a^4*c^14*A*B^2 +4*a^2*b^2*c^14*A*B^2 - 7*b^4*c^14*A*B^2 - 2*a^2*c^16*A*B^2 +2*b^2*c^16*A*B^2 - a^18*A^2*C + 2*a^16*b^2*A^2*C + 2*a^14*b^4*A^2*C -7*a^12*b^6*A^2*C + 5*a^10*b^8*A^2*C - 4*a^8*b^10*A^2*C +8*a^6*b^12*A^2*C - 7*a^4*b^14*A^2*C + 2*a^2*b^16*A^2*C +4*a^16*c^2*A^2*C - 14*a^14*b^2*c^2*A^2*C + 4*a^12*b^4*c^2*A^2*C +22*a^10*b^6*c^2*A^2*C - 8*a^8*b^8*c^2*A^2*C - 20*a^6*b^10*c^2*A^2*C +10*a^4*b^12*c^2*A^2*C + 4*a^2*b^14*c^2*A^2*C - 2*b^16*c^2*A^2*C -3*a^14*c^4*A^2*C + 33*a^12*b^2*c^4*A^2*C - 39*a^10*b^4*c^4*A^2*C -30*a^8*b^6*c^4*A^2*C + 42*a^6*b^8*c^4*A^2*C +18*a^4*b^10*c^4*A^2*C - 24*a^2*b^12*c^4*A^2*C + 3*b^14*c^4*A^2*C -10*a^12*c^6*A^2*C - 28*a^10*b^2*c^6*A^2*C + 80*a^8*b^4*c^6*A^2*C -4*a^6*b^6*c^6*A^2*C - 64*a^4*b^8*c^6*A^2*C +20*a^2*b^10*c^6*A^2*C + 6*b^12*c^6*A^2*C + 25*a^10*c^8*A^2*C -8*a^8*b^2*c^8*A^2*C - 68*a^6*b^4*c^8*A^2*C + 46*a^4*b^6*c^8*A^2*C +19*a^2*b^8*c^8*A^2*C - 14*b^10*c^8*A^2*C - 24*a^8*c^10*A^2*C +30*a^6*b^2*c^10*A^2*C + 18*a^4*b^4*c^10*A^2*C - 30*a^2*b^6*c^10*A^2*C +6*b^8*c^10*A^2*C + 11*a^6*c^12*A^2*C - 19*a^4*b^2*c^12*A^2*C +5*a^2*b^4*c^12*A^2*C + 3*b^6*c^12*A^2*C - 2*a^4*c^14*A^2*C +4*a^2*b^2*c^14*A^2*C - 2*b^4*c^14*A^2*C + 2*a^16*b^2*B^2*C -7*a^14*b^4*B^2*C + 8*a^12*b^6*B^2*C - 4*a^10*b^8*B^2*C + 5*a^8*b^10*B^2*C -7*a^6*b^12*B^2*C + 2*a^4*b^14*B^2*C + 2*a^2*b^16*B^2*C -b^18*B^2*C - 2*a^16*c^2*B^2*C + 4*a^14*b^2*c^2*B^2*C +10*a^12*b^4*c^2*B^2*C - 20*a^10*b^6*c^2*B^2*C - 8*a^8*b^8*c^2*B^2*C +22*a^6*b^10*c^2*B^2*C + 4*a^4*b^12*c^2*B^2*C - 14*a^2*b^14*c^2*B^2*C +4*b^16*c^2*B^2*C + 3*a^14*c^4*B^2*C - 24*a^12*b^2*c^4*B^2*C +18*a^10*b^4*c^4*B^2*C + 42*a^8*b^6*c^4*B^2*C - 30*a^6*b^8*c^4*B^2*C -39*a^4*b^10*c^4*B^2*C + 33*a^2*b^12*c^4*B^2*C - 3*b^14*c^4*B^2*C +6*a^12*c^6*B^2*C + 20*a^10*b^2*c^6*B^2*C - 64*a^8*b^4*c^6*B^2*C -4*a^6*b^6*c^6*B^2*C + 80*a^4*b^8*c^6*B^2*C - 28*a^2*b^10*c^6*B^2*C -10*b^12*c^6*B^2*C - 14*a^10*c^8*B^2*C + 19*a^8*b^2*c^8*B^2*C +46*a^6*b^4*c^8*B^2*C - 68*a^4*b^6*c^8*B^2*C - 8*a^2*b^8*c^8*B^2*C +25*b^10*c^8*B^2*C + 6*a^8*c^10*B^2*C - 30*a^6*b^2*c^10*B^2*C +18*a^4*b^4*c^10*B^2*C + 30*a^2*b^6*c^10*B^2*C - 24*b^8*c^10*B^2*C +3*a^6*c^12*B^2*C + 5*a^4*b^2*c^12*B^2*C - 19*a^2*b^4*c^12*B^2*C +11*b^6*c^12*B^2*C - 2*a^4*c^14*B^2*C + 4*a^2*b^2*c^14*B^2*C -2*b^4*c^14*B^2*C - 2*a^14*b^4*A*C^2 + 3*a^12*b^6*A*C^2 +6*a^10*b^8*A*C^2 - 14*a^8*b^10*A*C^2 + 6*a^6*b^12*A*C^2 +3*a^4*b^14*A*C^2 - 2*a^2*b^16*A*C^2 + 4*a^14*b^2*c^2*A*C^2 +5*a^12*b^4*c^2*A*C^2 - 30*a^10*b^6*c^2*A*C^2 + 19*a^8*b^8*c^2*A*C^2 +20*a^6*b^10*c^2*A*C^2 - 24*a^4*b^12*c^2*A*C^2 + 4*a^2*b^14*c^2*A*C^2 +2*b^16*c^2*A*C^2 - 2*a^14*c^4*A*C^2 - 19*a^12*b^2*c^4*A*C^2 +18*a^10*b^4*c^4*A*C^2 + 46*a^8*b^6*c^4*A*C^2 -64*a^6*b^8*c^4*A*C^2 + 18*a^4*b^10*c^4*A*C^2 + 10*a^2*b^12*c^4*A*C^2 -7*b^14*c^4*A*C^2 + 11*a^12*c^6*A*C^2 + 30*a^10*b^2*c^6*A*C^2 -68*a^8*b^4*c^6*A*C^2 - 4*a^6*b^6*c^6*A*C^2 +42*a^4*b^8*c^6*A*C^2 - 20*a^2*b^10*c^6*A*C^2 + 8*b^12*c^6*A*C^2 -24*a^10*c^8*A*C^2 - 8*a^8*b^2*c^8*A*C^2 + 80*a^6*b^4*c^8*A*C^2 -30*a^4*b^6*c^8*A*C^2 - 8*a^2*b^8*c^8*A*C^2 - 4*b^10*c^8*A*C^2 +25*a^8*c^10*A*C^2 - 28*a^6*b^2*c^10*A*C^2 - 39*a^4*b^4*c^10*A*C^2 +22*a^2*b^6*c^10*A*C^2 + 5*b^8*c^10*A*C^2 - 10*a^6*c^12*A*C^2 +33*a^4*b^2*c^12*A*C^2 + 4*a^2*b^4*c^12*A*C^2 - 7*b^6*c^12*A*C^2 -3*a^4*c^14*A*C^2 - 14*a^2*b^2*c^14*A*C^2 + 2*b^4*c^14*A*C^2 +4*a^2*c^16*A*C^2 + 2*b^2*c^16*A*C^2 - c^18*A*C^2 - 2*a^16*b^2*B*C^2 +3*a^14*b^4*B*C^2 + 6*a^12*b^6*B*C^2 - 14*a^10*b^8*B*C^2 +6*a^8*b^10*B*C^2 + 3*a^6*b^12*B*C^2 - 2*a^4*b^14*B*C^2 +2*a^16*c^2*B*C^2 + 4*a^14*b^2*c^2*B*C^2 - 24*a^12*b^4*c^2*B*C^2 +20*a^10*b^6*c^2*B*C^2 + 19*a^8*b^8*c^2*B*C^2 - 30*a^6*b^10*c^2*B*C^2 +5*a^4*b^12*c^2*B*C^2 + 4*a^2*b^14*c^2*B*C^2 - 7*a^14*c^4*B*C^2 +10*a^12*b^2*c^4*B*C^2 + 18*a^10*b^4*c^4*B*C^2 - 64*a^8*b^6*c^4*B*C^2 +46*a^6*b^8*c^4*B*C^2 + 18*a^4*b^10*c^4*B*C^2 -19*a^2*b^12*c^4*B*C^2 - 2*b^14*c^4*B*C^2 + 8*a^12*c^6*B*C^2 -20*a^10*b^2*c^6*B*C^2 + 42*a^8*b^4*c^6*B*C^2 - 4*a^6*b^6*c^6*B*C^2 -68*a^4*b^8*c^6*B*C^2 + 30*a^2*b^10*c^6*B*C^2 + 11*b^12*c^6*B*C^2 -4*a^10*c^8*B*C^2 - 8*a^8*b^2*c^8*B*C^2 - 30*a^6*b^4*c^8*B*C^2 +80*a^4*b^6*c^8*B*C^2 - 8*a^2*b^8*c^8*B*C^2 - 24*b^10*c^8*B*C^2 +5*a^8*c^10*B*C^2 + 22*a^6*b^2*c^10*B*C^2 - 39*a^4*b^4*c^10*B*C^2 -28*a^2*b^6*c^10*B*C^2 + 25*b^8*c^10*B*C^2 - 7*a^6*c^12*B*C^2 +4*a^4*b^2*c^12*B*C^2 + 33*a^2*b^4*c^12*B*C^2 - 10*b^6*c^12*B*C^2 +2*a^4*c^14*B*C^2 - 14*a^2*b^2*c^14*B*C^2 - 3*b^4*c^14*B*C^2 +2*a^2*c^16*B*C^2 + 4*b^2*c^16*B*C^2 - c^18*B*C^2 = 0";
				break;
			case 602:
				equation = "a^4 b^2 A^2 B + b^6 A^2 B - a^4 c^2 A^2 B - 2 a^2 b^2 c^2 A^2 B - 3 b^4 c^2 A^2 B + 2 a^2 c^4 A^2 B + 4 b^2 c^4 A^2 B - 2 c^6 A^2 B + a^6 A B^2 + a^2 b^4 A B^2 - 3 a^4 c^2 A B^2 - 2 a^2 b^2 c^2 A B^2 - b^4 c^2 A B^2 + 4 a^2 c^4 A B^2 + 2 b^2 c^4 A B^2 - 2 c^6 A B^2 - a^4 b^2 A^2 C + 2 a^2 b^4 A^2 C - 2 b^6 A^2 C + a^4 c^2 A^2 C - 2 a^2 b^2 c^2 A^2 C + 4 b^4 c^2 A^2 C - 3 b^2 c^4 A^2 C + c^6 A^2 C - 2 a^6 B^2 C + 2 a^4 b^2 B^2 C - a^2 b^4 B^2 C + 4 a^4 c^2 B^2 C - 2 a^2 b^2 c^2 B^2 C + b^4 c^2 B^2 C - 3 a^2 c^4 B^2 C + c^6 B^2 C + a^6 A C^2 - 3 a^4 b^2 A C^2 + 4 a^2 b^4 A C^2 - 2 b^6 A C^2 - 2 a^2 b^2 c^2 A C^2 + 2 b^4 c^2 A C^2 + a^2 c^4 A C^2 - b^2 c^4 A C^2 - 2 a^6 B C^2 + 4 a^4 b^2 B C^2 - 3 a^2 b^4 B C^2 + b^6 B C^2 + 2 a^4 c^2 B C^2 - 2 a^2 b^2 c^2 B C^2 - a^2 c^4 B C^2 + b^2 c^4 B C^2 = 0";
				break;
			case 603:
				equation = "a^3 b^2 c A^2 B - a^2 b^3 c A^2 B - a b^4 c A^2 B + b^5 c A^2 B - a^3 b c^2 A^2 B + 5 a b^3 c^2 A^2 B - 2 b^4 c^2 A^2 B + a^2 b c^3 A^2 B - 6 a b^2 c^3 A^2 B - b^3 c^3 A^2 B + 2 a b c^4 A^2 B + 4 b^2 c^4 A^2 B - 2 b c^5 A^2 B + a^5 c A B^2 - a^4 b c A B^2 - a^3 b^2 c A B^2 + a^2 b^3 c A B^2 - 2 a^4 c^2 A B^2 + 5 a^3 b c^2 A B^2 - a b^3 c^2 A B^2 - a^3 c^3 A B^2 - 6 a^2 b c^3 A B^2 + a b^2 c^3 A B^2 + 4 a^2 c^4 A B^2 + 2 a b c^4 A B^2 - 2 a c^5 A B^2 - a^3 b^2 c A^2 C + a^2 b^3 c A^2 C + 2 a b^4 c A^2 C - 2 b^5 c A^2 C + a^3 b c^2 A^2 C - 6 a b^3 c^2 A^2 C + 4 b^4 c^2 A^2 C - a^2 b c^3 A^2 C + 5 a b^2 c^3 A^2 C - b^3 c^3 A^2 C - a b c^4 A^2 C - 2 b^2 c^4 A^2 C + b c^5 A^2 C - 2 a^5 c B^2 C + 2 a^4 b c B^2 C + a^3 b^2 c B^2 C - a^2 b^3 c B^2 C + 4 a^4 c^2 B^2 C - 6 a^3 b c^2 B^2 C + a b^3 c^2 B^2 C - a^3 c^3 B^2 C + 5 a^2 b c^3 B^2 C - a b^2 c^3 B^2 C - 2 a^2 c^4 B^2 C - a b c^4 B^2 C + a c^5 B^2 C + a^5 b A C^2 - 2 a^4 b^2 A C^2 - a^3 b^3 A C^2 + 4 a^2 b^4 A C^2 - 2 a b^5 A C^2 - a^4 b c A C^2 + 5 a^3 b^2 c A C^2 - 6 a^2 b^3 c A C^2 + 2 a b^4 c A C^2 - a^3 b c^2 A C^2 + a b^3 c^2 A C^2 + a^2 b c^3 A C^2 - a b^2 c^3 A C^2 - 2 a^5 b B C^2 + 4 a^4 b^2 B C^2 - a^3 b^3 B C^2 - 2 a^2 b^4 B C^2 + a b^5 B C^2 + 2 a^4 b c B C^2 - 6 a^3 b^2 c B C^2 + 5 a^2 b^3 c B C^2 - a b^4 c B C^2 + a^3 b c^2 B C^2 - a b^3 c^2 B C^2 - a^2 b c^3 B C^2 + a b^2 c^3 B C^2 = 0";
				break;
				
				
			case 609:
				equation = "4*b^4*c^2*A^3 - 4*b^2*c^4*A^3 + a^4*c^2*A^2*B + 4*a^2*b^2*c^2*A^2*B -b^4*c^2*A^2*B - 2*a^2*c^4*A^2*B + c^6*A^2*B + a^4*c^2*A*B^2 -4*a^2*b^2*c^2*A*B^2 - b^4*c^2*A*B^2 + 2*b^2*c^4*A*B^2 - c^6*A*B^2 -4*a^4*c^2*B^3 + 4*a^2*c^4*B^3 - a^4*b^2*A^2*C + 2*a^2*b^4*A^2*C -b^6*A^2*C - 4*a^2*b^2*c^2*A^2*C + b^2*c^4*A^2*C + a^6*B^2*C -2*a^4*b^2*B^2*C + a^2*b^4*B^2*C + 4*a^2*b^2*c^2*B^2*C - a^2*c^4*B^2*C -a^4*b^2*A*C^2 + b^6*A*C^2 + 4*a^2*b^2*c^2*A*C^2 - 2*b^4*c^2*A*C^2 +b^2*c^4*A*C^2 - a^6*B*C^2 + a^2*b^4*B*C^2 + 2*a^4*c^2*B*C^2 -4*a^2*b^2*c^2*B*C^2 - a^2*c^4*B*C^2 + 4*a^4*b^2*C^3 - 4*a^2*b^4*C^3 = 0";
				break;
			default:
				poly.setUndefined();
				return;
		}

		equation = equation.replace("A", Astr);
		equation = equation.replace("B", Bstr);
		equation = equation.replace("C", Cstr);
		equation = equation.replace("a", "" + a);
		equation = equation.replace("b", "" + b);
		equation = equation.replace("c", "" + c);
		
		Parser parser = getKernel().getParser();
		AlgebraProcessor algebraProcessor = getKernel().getAlgebraProcessor();
		
	 	ValidExpression ve = null;
		try{ 
		 	ve = parser.parseGeoGebraExpression(equation); 
		 	GeoImplicitPoly result = (GeoImplicitPoly)(algebraProcessor.processEquation((Equation) ve)[0]); 
		 	result.remove();
		 	poly.setCoeff(result.getCoeff());
		 	poly.setDefined();
		} catch(ParseException e) 
	 	{ 
			poly.setUndefined();
	 		throw new MyError(app, "Error"); 
	 	} 
	}

	// TODO Consider locusequability

	
	
	
	
}